####
# globalParamaters
#
# Difinition of Global Paramaters
#
# version info : 1.2.4  2022/10/24  H.Ogawa
#

import math


# 定数 / 円周率
#pi = float(math.pi)
pi = 3.14159265358979323846264338327950288419716939937510


# plot procedure
#support_plot_procedures = False
support_plot_procedures = True
SS_range = 0.5


# 色定義 (print 文)
_ColorForPrint = {'BLACK':'\033[30m', 
                  'RED':'\033[31m', 
                  'GREEN':'\033[32m', 
                  'YELLOW':'\033[33m', 
                  'BLUE':'\033[34m', 
                  'MAGENTA':'\033[35m', 
                  'CYAN':'\033[36m', 
                  'WHITE':'\033[37m', 
                  'REVERSE':'\033[7m', 
                  'UNDERLINE':'\033[4m', 
                  'BOLD':'\033[1m', 
                  'END':'\033[0m'}

def black(s): return(BLACK+s+END)
def red(s): return(RED+s+END)
def green(s): return(GREEN+s+END)
def yellow(s): return(YELLOW+s+END)
def blue(s): return(BLUE+s+END)
def magenta(s): return(MAGENTA+s+END)
def cyan(s): return(CYAN+s+END)
def white(s): return(WHITE+s+END)
def reverse(s): return(REVERSE+s+END)
def bold(s): return(BOLD+s+END)
def underline(s): return(UNDERLINE+s+END)

BLACK = _ColorForPrint['BLACK']
RED = _ColorForPrint['RED']
GREEN = _ColorForPrint['GREEN']
YELLOW = _ColorForPrint['YELLOW']
BLUE = _ColorForPrint['BLUE']
MAGENTA = _ColorForPrint['MAGENTA']
CYAN = _ColorForPrint['CYAN']
WHITE = _ColorForPrint['WHITE']
REVERSE = _ColorForPrint['REVERSE']
UNDERLINE = _ColorForPrint['UNDERLINE']
BOLD = _ColorForPrint['BOLD']
END = _ColorForPrint['END']


# 変換用辞書
dictPSL2 = {
    'A':[[0,1],[-1,0]], 'B':[[1,1],[-1,0]], # (3.4)
    'p':[[1,1],[-1,0]], 'b':[[0,1],[-1,-1]], 'q':[[0,-1],[1,-1]], 'd':[[1,-1],[1,0]], # Table 1
    'R':[[1,1],[0,1]], 'L':[[1,0],[1,1]], 'r':[[1,-1],[0,1]], 'l':[[1,0],[-1,1]], # Kin Sect.5
    'S':[[0,-1],[1,0]], 'T':[[1,1],[0,1]], 't':[[1,-1],[0,1]], 
    '1':[[1,0],[-1,1]], '-1':[[1,0],[1,1]], '2':[[1,1],[0,1]], '-2':[[1,-1],[0,1]], ' ':[[1,0],[0,1]], '':[[1,0],[0,1]], 
}

dictAB = {
    'A':'A', 'B':'B', 
    'R':'ABB', 'L':'AB', 'r':'BA', 'l':'BBA', 
    'b':'BB', 'd':'ABBA', 'p':'B', 'q':'ABA', 
    'S':'A', 'T':'ABB', 't':'BA', 
    '1':'BBA', '2':'ABB', '-1':'AB', '-2':'BA', 
}

dictST = {
    'A':'S', 'B':'STST', 
    'b':'ST', 'd':'TS', 'p':'STST', 'q':'TSTS', 
    'R':'T', 'L':'TST', 'r':'STSTS', 'l':'STS', 
    'S':'S', 'T':'T', 't':'STSTS', #'t':'t', 
    '1':'STS', '-1':'TST', '2':'T', '-2':'STSTS', 
}

dictRL = {
    'A':'LrL', 'B':'lR', # 'A':'RlR', 'B':'rLrL', 
    'b':'rL', 'd':'Lr', 'p':'lR', 'q':'Rl', 
    'S':'LrL', 'T':'R', 't':'r',
    'R':'R', 'L':'L', 'r':'r', 'l':'l', 
    '1':'l', '-1':'L', '2':'R', '-2':'r', 
    'db':'RR', 'dp':'RL', 'qb':'LR', 'qp':'LL', 
    'bd':'ll', 'bq':'lr', 'pd':'rl', 'pq':'rr', 
}

dictRLbdpq = {
    'RR':'db', 'RL':'dp', 'LR':'qb', 'LL':'qp', 
    'll':'bd', 'lr':'bq', 'rl':'pd', 'rr':'pq', 
    'R':'R', 'L':'L', 'r':'r', 'l':'l', 
    'db':'RR', 'dp':'RL', 'qb':'LR', 'qp':'LL', 
    'bd':'ll', 'bq':'lr', 'pd':'rl', 'pq':'rr', 
    'b':'b', 'd':'d', 'p':'p', 'q':'q', 
}

dictInv = {
    'A':'A', 'B':'BB', 
    'R':'r', 'L':'l', 'r':'R', 'l':'L', 
    'b':'p', 'd':'q', 'p':'b', 'q':'d', 
    'S':'S', 'T':'t', 't':'T', 
    '1':' -1', '2':' -2', '-1':' 1', '-2':' 2', 
    ' 1':' -1', ' 2':' -2', ' -1':' 1', ' -2':' 2', 
    'σ1':' -1', 'σ2': ' -2',
}

dictEPbdpq1 = {
    1:{'+':'p', '-':'d', '0':'p', '1':'d'},
    -1:{'+':'q', '-':'b', '0':'b', '1':'q'},
    0:{'+':'', '-':'', '0':'', '1':''},
}

dictEPbdpq = {
    '++':{'+':'p', '-':'d', '0':'d', '1':'p'},
    '+-':{'+':'p', '-':'d', '0':'p', '1':'d'},
    '-+':{'+':'q', '-':'b', '0':'q', '1':'b'},
    '--':{'+':'q', '-':'b', '0':'b', '1':'q'},
}

dictS3 = {
    '0+':(1), '1+':(1,2,3), '2+':(1,3,2),
    '0-':(2,3), '1-':(1,3), '2-':(1,2)
}

dictPDQ={
    'b':'q', 'q':'b', 'd':'p', 'p':'d'
}

dictTrFP = {
    'flipV' :{'p':'b', 'q':'d', 'b':'p', 'd':'q'},
    'flipH' :{'p':'q', 'q':'p', 'b':'d', 'd':'b'},
    'rotate':{'p':'d', 'q':'b', 'b':'q', 'd':'p'}
}

dictSgn = {1:'+', -1:'-', '+':1, '-':-1}

dictSgnMinus = {1:-1, -1:1, '+':'-', '-':'+'}


# 絡み目  端点処理
braidStyles = ['braid', 'circular', 'loop', 'loop3', 'loop6']
#braidStyles = ['braid', 'circular', 'closure', 'loop', 'loop3', 'loop6']

# 絡み目  表現形式
braidTypes = ['AB', 'Sigma', 'Syzygy', 'BDPQ', 'RL', 'ST', 'PSL2', 'Dilatation']


# plot options
options = {
    'ABrate':3,
    'figsize':(4,4), 
    'figsizeL':(10,3), 
    'linewidth':4, 
    'startArea':'0+', 
    'view':(-90, 0), 
    'viewSS':(-30, 20), 
    'T_interval':(0, 1), 
    'T_interval1':(0, 1), 
    'T_interval3':(0, 1/3), 
    'T_interval6':(0, 1/6), 
    #'color':(('red', 'green', 'blue'), ('yellow', 'gray', 'lime'), ('black', 'red', 'green', 'blue')), 
    'color':(('red', 'blue', 'yellow'), ('orange', 'gray', 'lime'), ('black', 'red', 'green', 'blue')), 
    'pathLims':4/(3-1)**(1/4)
}


#
def setOptions(**kwargs):
    if len(kwargs) > 0:
        print("以下の通り設定します。")
        for key in kwargs.keys():
            print(" ",key, ":", kwargs[key])
            options[key] = kwargs[key]
    else:
        print("設定支援メニュー　([]内は現在の値)\n\t変更の必要のないものは、何も入力せず、[return(enter)] キーを押してください。")
        #
        x = input("(1) 画像サイズ：１(小) 〜 ９(大) [{}] ".format(options['figsize'][0]))
        y = int(x) if '0' < x <= '9' else 3
        options['figsize'] = (y,y)
        options['figsizeL'] = (3*y,y)
        #
        x = input("(2) 線の太さ：１(細) 〜 9(太) [{}] ".format(options['linewidth']))
        if x != '':
            options['linewidth'] = int(x)
            if options['linewidth'] < 1: options['linewidth'] = 1
        #
        x = input("(3) 3-braid の Shape sphere 上での始点：(012)&(+-) [{}] ".format(options['startArea']))
        if x != '':
            sA = x[0]+x[-1]
            if sA[0] not in ('0', '1', '2'):
                sA = options['startArea'][0]+sA[-1]
            if sA[-1] not in ('+', '-'):
                sA = sA[0]+options['startArea'][-1]
            options['StartArea'] = sA
        #
        v00, v01 = options['view']
        x = input("(4) ３次元像の視点\n\   (4-1) braid : 視線 (水平方向) [{}] ".format(v00))
        if  x != '': v00 = int(x)
        x = input("   (4-2) braid : 仰角 (垂直方向) [{}] ".format(v01))
        if  x != '': v01 = int(x)
        options['View'] = v00, v01
        #
        x = input("(5) paramater t の変動域 : [{}] ".format(options['T_interval']))
        if len(x) > 0:
            z = tuple([int(y) for y in x.replace('(','').replace(')','').replace('[','').replace(']','').split(',')])
            if len(z) == 2:
                options['T_interval'] = z
                options['T_interval1'] = z
                options['T_interval3'] = (z[0], z[0]+(z[1]-z[0])/3)
                options['T_interval6'] = (z[0], z[0]+(z[1]-z[0])/6)
        #
        cBr, cTr, cP = options['color']
        x = input("(6) 色\n   (6-1) ３動点 [{},{},{}] ".format(*cBr))
        z = x.replace(' ', '').split(',')
        if len(z) == 3: cBr = tuple(z)
        x = input("   (6-2) 三角形の向き (正,朔,負) [{},{},{}] ".format(*cTr))
        z = x.replace(' ', '').split(',')
        if len(z) == 3: cTr = tuple(z)
        x = input("   (6-3) Shape Sphere (道,１,ω,極) [{},{},{},{}] ".format(*cP))
        z = x.replace(' ', '').split(',')
        if len(z) == 4: cP = z
        options['color'] = cBr, cTr, cP
        #
        x = input("(7) Lissajous 曲線の縦横比 (縦/横) [{}] ".format(options['ABrate']))
        if x != '':
            options['ABrate'] = float(x)
            options['pathLims'] = 4/(options['ABrate']-1)**(1/4)
        #
    #
    print("設定が完了しました")
    #


def margeOptions(kwargs):
    if 'color' in kwargs.keys():
        kwargs['color'] = [x if len(x) == len(y) else y for x,y in zip(kwargs['color'], options['color'])]
    return(options | kwargs)



