#########
# proc_Lin
#
# procedures for Linear Braid
#
# version info : 1.2.4  2022/10/24  H.Ogawa
#


import numpy as np
import math
import cmath
from fractions import Fraction

from . import globalParamaters as glbl
from . import subprocedures as sub
from . import translation as tr
from . import proc_TB as TB
#from . import proc_Lin as Lin
#from . import proc_Lis as Lis



# 初期化

def init(self) -> None:
    self.TBsw = False
    self.LinearParam = []
    self.LissajousType = []
    self.LissajousTypeNew = []
    self.CollisionFreeQ = True
    self.RawData = {'braidStyle':'Raw Data', 'inputData':['','']}
    self.LinearCancellation = {'braidStyle':'Linear Cancellation'}
    self.startArea = '0+'
    if glbl.support_plot_procedures:
        self.plotData = {'steps':0, 'LissajousType':{}, 'delta':[], 't':[], 'triangles':[], 'edges':[], 'trColors':[]}


#####

def title(self) -> str:
    return('Linear parameter: {}\nLissajous Type: {}\n  Collision-free?: {}\n'.format(sub.prettyPrint(self.LinearParam), sub.prettyPrint(self.LissajousType), self.CollisionFreeQ))


def values(self, **kwargs) -> str:
    return(
        'Raw Data\n' + TB.brValues(self.RawData, ['AB3', 'Syzygy3'])
        + '\nLinear cancellation\t' + TB.brValues(self.LinearCancellation, ['slope', 'AB3', 'Syzygy3'])
        + '\nReduced sequences   (1/3)' 
        + '\n' + TB.brValues(self.loop3, ['AB', 'Syzygy', 'BDPQ', 'PSL2', 'Dilatation'])
        #+ '\nReduced sequences   ' 
        #+ ('' if not self.TripleQ else ('(1/6)' if self.andHalfQ else '(1/3)'))
        #+ '\n' + TB.brValues(self.loop6 if self.andHalfQ else self.loop3, ['AB', 'Syzygy', 'BDPQ', 'PSL2', 'Dilatation'])
    )


#####

def set(self, brWord='', linear=[], lissajous=[], lissajousnew=[], startArea='0+', silent=False) -> None:
    self.clear()
    #
    self.inputArgs = [brWord, linear, lissajous, lissajousnew, startArea]
    linPara = lisType = lisTypeNew = []
    #
    if linear != []:
        if type(linear) in (list, tuple):
            if len(linear) > 2:
                linPara = [Fraction(linear[0]), Fraction(linear[1]), int(linear[2])]
            elif len(linear) == 2:
                linPara = [Fraction(linear[0]), Fraction(linear[1])]
            elif type(linear[0]) in (int, float, str):
                linPara = [Fraction(linear[0])]
            else:
                print('{} is not a Linear Paramater. The slope must be a ractional number.'.format(linear))
                return(False)
        elif type(linear) in (int, float, str):
            linPara = [Fraction(linear)]
        else:
            print('{} is not a Linear Paramater. The slope must be a ractional number.'.format(linear))
            return(False)
    #
    if lissajous != []:
        if (type(lissajous) in (list, tuple)) and len(lissajous) > 1:
            if len(lissajous) > 2:
                lisType = [math.floor(lissajous[0]), math.floor(lissajous[1]), Fraction(lissajous[2]) if type(lissajous[2]) == str else lissajous[2]]
            else:
                lisType = [math.floor(lissajous[0]), math.floor(lissajous[1]), 0]
        else:
            print('{} is not a Lissajous Type.'.format(lissajous))
            return(False)
    #
    if lissajousnew != []:
        if (type(lissajousnew) in (list, tuple)) and len(lissajousnew) > 1:
            if len(lissajousnew) > 2:
                lisTypeNew = [math.floor(lissajousnew[0]), math.floor(lissajousnew[1]), Fraction(lissajousnew[2]) if type(lissajousnew[2]) == str else lissajousnew[2]]
            else:
                lisTypeNew = [math.floor(lissajousnew[0]), math.floor(lissajousnew[1]), 0]
        else:
            print('{} is not a Lissajous Type.'.format(lissajousnew))
            return(False)
    #
    self.clear()
    #
    if lisType != []:
        self.RawData['inputData'] = ('Lissajous Type (old)', lisType)
        lisTypeNew = [-lisType[1], lisType[0], 2*math.pi*lisType[1]*lisType[2]]
    elif lisTypeNew != []:
        self.RawData['inputData'] = ('Lissajous Type (new)', lisTypeNew)
        lisType = [lisTypeNew[1], -lisTypeNew[0], lisTypeNew[2]/2/math.pi/lisTypeNew[1]]
    #
    if linPara != []:
        self.RawData['inputData'] = ('Linear Paramater', linPara)
        if len(linPara) >= 3:
            self.LinearParam = [linPara[0], linPara[1], int(linPara[2])]
        elif len(linPara) == 2:
            self.LinearParam = [linPara[0], linPara[1], 6*Fraction(linPara[0]).denominator]
        else:
            self.LinearParam = [linPara[0], Fraction(1, 2*(linPara[0].denominator)), 6*Fraction(linPara[0]).denominator]
        #
        self.proc_LissajousType(silent=silent)
        if glbl.support_plot_procedures:
            self.plotData['LissajousType'] = {'style':'old', 'new':self.LissajousTypeNew, 'old':self.LissajousType}
    #
    elif lisType != []:
        self.LissajousType = lisType
        self.LissajousTypeNew = lisTypeNew
        self.proc_LinearParamater(silent=silent)
        if glbl.support_plot_procedures:
            self.plotData['LissajousType'] = {'style':'old', 'new':self.LissajousTypeNew, 'old':self.LissajousType}
    else:
        self.TBsw = True
        self.brSet(brWord, startArea=startArea, silent=silent)


# Lissajous Curve and Lissajous Loop

def curve(t, lisType=[4,1,0]) -> complex:
    m,n,d = lisType
    return([math.sin(2*math.pi*(m*t+d)),math.sin(2*math.pi*n*t)])


def loop(t, lisType=[4,1,0]) -> complex:
    m,n,d = lisType
    nn = n if (m-n)%3 == 0 else -n
    el = (m-nn)/3
    e1 = 1 if m%3 == 1 else -1
    e0 = int(n/nn)/2
    return(-complex(math.cos(-4*math.pi*nn*t+4*math.pi/3), math.sin(-4*math.pi*nn*t+4*math.pi/3))
           *(1-e0*complex(math.cos(-e1*2*math.pi*(3*el*t+d)), math.sin(-e1*2*math.pi*(3*el*t+d)))*1j)
           /(1-e0*complex(math.cos( e1*2*math.pi*(3*el*t+d)), math.sin( e1*2*math.pi*(3*el*t+d)))*1j))


def domainOnShape(z, epsilon=0.001) -> list:
    r, s = cmath.polar(z)
    wk = str(int(s*3/2/math.pi)%3), sub.sign(r-1)
    if abs(wk[1]) < epsilon:
        return(wk[0]+' ')
    elif wk[1] > 0:
        return(wk[0]+'+')
    else:
        return(wk[0]+'-')


def startArea(LisType, t=0):
    dom = domainOnShape(loop(t, LisType))
    if dom[1] != ' ':
        return(dom)
    else:
        return(startArea(LisType, t=t-1/60/(abs(LisType[0])+abs(LisType[1]))))


# Collision-Free Test

def collisionFreeQ(Type, epsilon=0.0001) -> bool:
    m,n,d = Type
    el = (m-n)//3 if (m-n)%3 == 0 else (m+n)//3
    d0 = sub.gcd(m, n)
    return(abs(sub.fracPart((4*int(n)*d-el)/(2*d0)+Fraction('1/2'))
               -Fraction('1/2'))>epsilon)


# Lissajous Type ←→ Linear Paramater

def LisT_LinP(lisType: list) -> None:
    m,n,d = lisType
    if m*n%3 == 0: return(['error'])
    nn = n if (m-n)%3 ==0 else -n
    el = Fraction(m-nn, 3)
    return([sub.mod3m(Fraction(-nn, abs(el))), 
            sub.mod3m(Fraction(-nn, abs(el))*sub.fracPart(-2*sub.sign(el)*d)+Fraction('1/2')), 
            2*abs(m-nn)])


def LinP_LisT(linParam: list) -> None:
    if len(linParam) == 3 and linParam[2]%6 == 0:
        a,b,r = linParam
        while a <= 3: a += 3
        while b < Fraction('1/2'): b += 3
        while b >= Fraction('7/2'): b -= 3
        return([math.floor((r//6)*(3-a)), math.floor(-(r//6)*a), (1-2*b)/(4*a)])
    else:
        a = Fraction(linParam[0])
        return(LinP_LisT([a, linParam[1], 6*a.denominator]))


# Linear Syzygy Sequence and Lissajous Syzygy Sequence

def LinearSyzygySequence(linParam: list) -> str:
    if len(linParam) == 3 and linParam[2]%6 == 0:
        a,b,r = linParam
        return(''.join(str(sub.intMod3(a*t+b)) for t in range(r)))
    else:
        a = Fraction(linParam[0])
        return(LinearSyzygySequence([a, linParam[1], 6*a.denominator]))


def LissajousSyzygySequence(lisType: list) -> str:
    return(LinearSyzygySequence(LissajousType_LinearParam(lisType)))


# LinearCancellation of Slope

def LinearCancellationSlope(x: Fraction) -> Fraction:
    a = Fraction(x)
    cf32 = Fraction('3/2')
    cf12 = Fraction('1/2')
    while True:
        while a <= -cf32: a += 3
        while a > cf32: a -= 3
        if a == 0: return(0)
        if abs(a) == cf12: return(0)
        if abs(a) >= 1: return(a)
        a = a/abs(1-2*abs(a))



