#######
# proc_Lis
#
# Procedures for Lissajous Braid
#
# version info : 1.2.4  2022/10/24  H.Ogawa
#


import numpy as np
import math
import cmath
from fractions import Fraction

from . import globalParamaters as glbl
from . import subprocedures as sub
from . import translation as tr
from . import proc_TB as TB

#brType = sub.braidReprType
#pprint = sub.prettyPrint


# 初期化

def init(self) -> None:
    self.LissajousType = []
    self.LissajousTypeOld = []
    self.PrimitiveLissajousType = []
    self.KNOtype = 0
    self.CalcResultsKNO1 = {'braidStyle':'input data', 'concluded':False, 'primitive':False}
    self.CalcResultsKNO1Primitive = {'braidStyle':'primitive type', 'concluded':False, 'primitive':True}
    self.CalcResultsKNO2 = {'braidStyle':'input data', 'concluded':False, 'primitive':False}
    self.CalcResultsKNO2Primitive = {'braidStyle':'primitive type', 'concluded':False, 'primitive':True}


# show : KNO1 / KNO2

def KNO(self) -> str:
    return("Lissajous Type: {}\n    (primitive) {}\n".format(sub.prettyPrint(self.LissajousType), sub.prettyPrint(self.PrimitiveLissajousType))
           + "Lissajous Curve: sin({}πt) + i sin({}πt+{}) (t : a real number)\n".format(2*self.LissajousType[0], 2*self.LissajousType[1], self.LissajousType[2])
           ########
           + (
               ("\n"+glbl.red("[KNO1]")+"   Lissajous type: {}\n".format(self.LissajousType[:2])
                +"      BDPQ word (Frieze pattarn): {}\n".format(self.CalcResultsKNO1['Wbdpq'])
                ####
                +"  * "+glbl.blue("Prop 2.5 (Main Formula) / Lem 3.6 (BDPQ word)")+"\n"
                +"      ε': {}\n".format(self.CalcResultsKNO1["ε'"])
                +"      ε : {}\n".format(self.CalcResultsKNO1["ε"])
                +"      W({},{}): {} {}\n".format(*self.LissajousType[0:2], *self.CalcResultsKNO1["Wbdpq"])
                +"        (AB seq.)   = {}\n".format(self.CalcResultsKNO1["W"])
                +"        (reduction) = {}\n".format(self.CalcResultsKNO1["AB"])
                +"        (PSL2)      = {}\n".format(self.CalcResultsKNO1["PSL2"])
                +"      H({},{}): {}\n".format(*self.LissajousType[0:2], self.CalcResultsKNO1["Hbdpq"])
                +"        (PSL2)      = {} ({})\n".format(*self.CalcResultsKNO1["PSL2/H"])
                +"         H -> A H^(-1) A: {} -> {}\n".format(self.CalcResultsKNO1["Hbdpq"], tr.mirror(self.CalcResultsKNO1["Hbdpq"]))
                ####
                +"  * "+glbl.blue("Prop 2.8 (Double palindromicity)")+"\n"
                +"      ε': {} / {}\n".format(*sub.halfAll(self.CalcResultsKNO1["ε'"]))
                +"      ε : {} / {}\n".format(*sub.halfAll(self.CalcResultsKNO1["ε"]))
                +"             (reverse  order: "+(glbl.red("{}")+" / "+glbl.red("{}")+"    sign:"+glbl.red("{}")+")\n").format(*self.CalcResultsKNO1["ε' double palindromicity"])
                ####
           + "  * "+glbl.blue("Application of Boca-Merriman (odd continued fraction)")+"\n"
                +"      RL: {}\n".format(' '.join(sub.halfAll(self.CalcResultsKNO1['RL'])))
                +"      PSL2: {}\n".format(self.CalcResultsKNO1['PSL2'])
                +"      endpoint: {}\n".format(self.CalcResultsKNO1['EndPoint'][0])
                +"      period of odd continued fraction: {}\n".format(self.CalcResultsKNO1["oddCF"])
                +"     → RL: {}\n".format(self.CalcResultsKNO1["RL (oddCF)"])
                ####
                +"  * "+glbl.blue("Level and Slope")+"\n"
                +"      Primitive Lissajous type: {}\n".format(self.PrimitiveLissajousType)
                +"      slope: {1}/{0}   level: {2}\n".format(*self.CalcResultsKNO1Primitive["Slope Level"][0], self.CalcResultsKNO1Primitive["Slope Level"][1])
                ####
                + "  * "+glbl.blue("mod 2 difference of ε'")+"\n"
                + "      δ' = Δε': {}\n".format(self.CalcResultsKNO1Primitive["δ'"])
                + "     → ε':      {} {}\n".format(*sub.halfAll(self.CalcResultsKNO1Primitive["δ'->ε'"]))
                + "       H({},{}): {}\n".format(*self.PrimitiveLissajousType[0:2], self.CalcResultsKNO1Primitive["H (δ')"])
                + "  * "+glbl.blue("Christoffel word of slope and level lifting morphism")+"\n"
                + "      Christoffel word: cw({1}/{0}): {2}\n".format(*self.CalcResultsKNO1Primitive["Slope Level"][0], self.CalcResultsKNO1Primitive["Christoffel Word"])
                + "      δ' ~ Φ{0}(cw({2}/{1})): {3}\n".format(self.CalcResultsKNO1Primitive["Slope Level"][1], *self.CalcResultsKNO1Primitive["Slope Level"][0], self.CalcResultsKNO1Primitive["δ'(CW)"])
                + "     → ε': {} {}\n".format(*sub.halfAll(self.CalcResultsKNO1Primitive["δ'->ε'(CW)"][0]))
                + "       W({},{}): {} {}\n".format(*self.PrimitiveLissajousType[0:2], *self.CalcResultsKNO1Primitive["W (CW)"][0])
                + "       H({},{}): {}\n".format(*self.PrimitiveLissajousType[0:2], self.CalcResultsKNO1Primitive["H (CW)"][0])
                + "       {W} :  "+str((wk:=[" ".join(w) for w in self.CalcResultsKNO1Primitive["W (CW)"]])[0])+"\n"+sub.tinyFprint(wk[1:], n=1, head=' '*14)+"\n"
                + "  * "+glbl.blue("Palindromic Conjugate")+"\n"
                + "      Christoffel word: cw({1}/{0}): {2}\n".format(*self.CalcResultsKNO1Primitive["Slope Level"][0], self.CalcResultsKNO1Primitive["Christoffel Word"])
                +"      Palindromic conjugate: pw({1}/{0}): {2}\n".format(*self.CalcResultsKNO1Primitive["Slope Level"][0], self.CalcResultsKNO1Primitive["Palindromic Conjugate"])
                + "      δ' = Φ{0}(pw({2}/{1})): {3}\n".format(self.CalcResultsKNO1Primitive["Slope Level"][1], *self.CalcResultsKNO1Primitive["Slope Level"][0], self.CalcResultsKNO1Primitive["δ'(PC)"])
                + "     → ε': {} {}\n".format(*sub.halfAll(self.CalcResultsKNO1Primitive["δ'->ε'(PC)"]))
                + "       W({},{}): {} {}\n".format(*self.PrimitiveLissajousType[0:2], *self.CalcResultsKNO1Primitive["W (PC)"])
                + "       H({},{}): {}\n".format(*self.PrimitiveLissajousType[0:2], self.CalcResultsKNO1Primitive["H (PC)"])
                + "  * "+glbl.blue("Prop 4.11 (calculation from level and slope)")+"\n"
                +"      Christoffel word:      cw({1}/{0}): {2}\n".format(*self.CalcResultsKNO1Primitive["Slope Level"][0], self.CalcResultsKNO1Primitive["Christoffel Word"])
                +"      Palindromic conjugate: pw({1}/{0}): {2}\n".format(*self.CalcResultsKNO1Primitive["Slope Level"][0], self.CalcResultsKNO1Primitive["Palindromic Conjugate"])
                +"      r = φ{0}(pw({2}/{1})): {3}\n".format(self.CalcResultsKNO1Primitive["Slope Level"][1], *self.CalcResultsKNO1Primitive["Slope Level"][0], self.CalcResultsKNO1Primitive['r (4.11)'])
                +"     → H({},{}): {}\n".format(*self.PrimitiveLissajousType[0:2], self.CalcResultsKNO1Primitive['H (4.11)'])
                ####
                +"  * "+glbl.blue("Syzygy sequence")+"\n"
                +"      Ω: {}\n".format(self.CalcResultsKNO1Primitive["Ω"])
                +"     → Syzygy: {}\n".format(self.CalcResultsKNO1Primitive["Syzygy (PW)"])
                ####
                +"  * "+glbl.blue("Dilatation")+"\n"
                +"      RL: {}\n".format(self.CalcResultsKNO1["RL (oddCF)"])
                +"      PSL2: {}\n".format(self.CalcResultsKNO1["PSL2"])
                +"      Dilatation: "+("{}\n".format(self.CalcResultsKNO1["Dilatation"][1]) if self.CalcResultsKNO1["Dilatation"][2] else "{}  (value={})".format(*self.CalcResultsKNO1["Dilatation"][0:2]))
                +"\n\n"
               ) if self.KNOtype ==1 else '')
           ########
           + "\n"+glbl.red("[KNO2]")+"   Lissajous type: {}\n".format(self.LissajousType[:2])
           +"      BDPQ word (Frieze pattarn): {} {}\n".format(*self.CalcResultsKNO2['Wbdpq'][0])
           ####
           + "  * "+glbl.blue("Mirror companion class")+"\n"
           + "      # { Wκ | κ ∈ Z/"+str(2*abs(self.LissajousType[0]))+"Z } = "+str(len(self.CalcResultsKNO2["ε'(uniq)"]))+"\n"
           + "      ε': {}\n".format(removeSpace(withKappa(self.CalcResultsKNO2["ε'(uniq)"][0], self.CalcResultsKNO2["κ"][0])))
           + "      ε : {}\n".format(removeSpace(withKappa(self.CalcResultsKNO2["ε (uniq)"][0], self.CalcResultsKNO2["κ"][0])))
           + "      W({},{}): {}\n".format(*self.LissajousType[0:2], removeSpace(withKappa(' '.join(self.CalcResultsKNO2["Wbdpq"][0]), self.CalcResultsKNO2["κ"][0])))
           + "        (AB seq.)   = {}\n".format(self.CalcResultsKNO2["W"][0])
           + "        (reduction) = {}\n".format(self.CalcResultsKNO2["AB"][0])
           + "        (PSL2)      = {}\n".format(self.CalcResultsKNO2["PSL2"][0])
           + "      V({},{}): {}\n".format(*self.LissajousType[0:2], removeSpace(withKappa(self.CalcResultsKNO2["Vbdpq"][0], self.CalcResultsKNO2["κ"][0])))
           + "        (PSL2)      = {}\n".format(self.CalcResultsKNO2["PSL2/V"][0])
           ####
           + "      {ε'}:  "+str((wk:=withKappa(self.CalcResultsKNO2["ε'(uniq)"], self.CalcResultsKNO2["κ"]))[0])+"\n"+sub.tinyFprint(wk[1:], n=1, head=' '*13)+"\n"
           + "      {ε} :  "+str((wk:=withKappa(self.CalcResultsKNO2["ε (uniq)"], self.CalcResultsKNO2["κ"]))[0])+"\n"+sub.tinyFprint(wk[1:], n=1, head=' '*13)+"\n"
           + "      {V} :  "+str((wk:=withKappa(self.CalcResultsKNO2["Vbdpq"], self.CalcResultsKNO2["κ"]))[0])+"\n"+sub.tinyFprint(wk[1:], n=1, head=' '*13)+"\n"
           ####
           + "  * "+glbl.blue("Application of Boca-Merriman (odd continued fraction) (KNO1)")+"\n"
           + "      RL: {}\n".format(removeSpace(withKappa(' '.join(sub.halfAll(self.CalcResultsKNO2['RL'][0])), self.CalcResultsKNO2["κ"][0])))
           + "      PSL2: {}\n".format(removeSpace(withKappa(self.CalcResultsKNO2['PSL2'][0], self.CalcResultsKNO2["κ"][0])))
           + "      endpoint: {}\n".format(removeSpace(withKappa(self.CalcResultsKNO2['EndPoint'][0][0], self.CalcResultsKNO2["κ"][0])))
           + "      period of odd continued fraction: {}\n".format(withKappa(self.CalcResultsKNO2["oddCF"][0], self.CalcResultsKNO2["κ"][0]))
           + "     → RL: {}\n".format(removeSpace(withKappa(self.CalcResultsKNO2["RL (oddCF)"][0], self.CalcResultsKNO2["κ"][0])))
           ####
           +(
               ("      {PSL2}:  "+str((wk:=withKappa(self.CalcResultsKNO2["PSL2"], self.CalcResultsKNO2["κ"]))[0])+"\n"+sub.tinyFprint(wk[1:], n=1, head=' '*15)+"\n"
                +"      {endpoint}:  "+str((wk:=withKappa([x[0] for x in self.CalcResultsKNO2["EndPoint"]], self.CalcResultsKNO2["κ"]))[0])+"\n"+sub.tinyFprint(wk[1:], n=1, head=' '*19)+"\n"
                +"      {period}:  "+str((wk:=withKappa(self.CalcResultsKNO2["oddCF"], self.CalcResultsKNO2["κ"]))[0])+"\n"+sub.tinyFprint(wk[1:], n=1, head=' '*17)+"\n"
                +"      {RL}:  "+str((wk:=withKappa(self.CalcResultsKNO2["RL (oddCF)"], self.CalcResultsKNO2["κ"]))[0])+"\n"+sub.tinyFprint(wk[1:], n=1, head=' '*13)+"\n") if len(self.CalcResultsKNO2["PSL2"])>1 else ""
           )
           ####
           + "  * "+glbl.blue("Level and Slope")+"\n"
           + "      Primitive Lissajous type: {}\n".format(self.PrimitiveLissajousType)
           + "      slope: {1}/{0}   level: {2}\n".format(*self.CalcResultsKNO2Primitive["Slope Level"][0], self.CalcResultsKNO2Primitive["Slope Level"][1])
           ####
           + (
               ("  * "+glbl.blue("(KNO1)  mod 2 difference of ε'")+"\n"
                + "      δ' = Δε': {}\n".format(self.CalcResultsKNO2Primitive["δ'"])
                + "     → ε':      {} {}\n".format(*sub.halfAll(self.CalcResultsKNO2Primitive["δ'->ε'"]))
                + "       W({},{}): {} {}\n".format(*self.PrimitiveLissajousType[0:2], *self.CalcResultsKNO2Primitive["W (δ')"])
                + "       V({},{}): {}\n".format(*self.PrimitiveLissajousType[0:2], self.CalcResultsKNO2Primitive["V (δ')"])
                + "  * "+glbl.blue("(KNO1)  Christoffel word of slope and level lifting morphism")+"\n"
                + "      Christoffel word: cw({1}/{0}): {2}\n".format(*self.CalcResultsKNO2Primitive["Slope Level"][0], self.CalcResultsKNO2Primitive["Christoffel Word"])
                + "      δ' ~ Φ{0}(cw({2}/{1})): {3}\n".format(self.CalcResultsKNO2Primitive["Slope Level"][1], *self.CalcResultsKNO2Primitive["Slope Level"][0], self.CalcResultsKNO2Primitive["δ'(CW)"])
                + "     → ε': {} {}\n".format(*sub.halfAll(self.CalcResultsKNO2Primitive["δ'->ε'(CW)"][0]))
                + "       W({},{}): {} {}\n".format(*self.PrimitiveLissajousType[0:2], *self.CalcResultsKNO2Primitive["W (CW)"][0])
                + "       V({},{}): {}\n".format(*self.PrimitiveLissajousType[0:2], self.CalcResultsKNO2Primitive["V (CW)"][0])
                + "       {W} :  "+str((wk:=[" ".join(w) for w in self.CalcResultsKNO2Primitive["W (CW)"]])[0])+"\n"+sub.tinyFprint(wk[1:], n=1, head=' '*14)+"\n"
                #+ "       {V} :  "+str((wk:=self.CalcResultsKNO2Primitive["H (CW)"])[0])+"\n"+sub.tinyFprint(wk[1:], n=1, head=' '*13)+"\n"
                + "  * "+glbl.blue("Palindromic Conjugate")+"\n"
                + "      Christoffel word: cw({1}/{0}): {2}\n".format(*self.CalcResultsKNO2Primitive["Slope Level"][0], self.CalcResultsKNO2Primitive["Christoffel Word"])
                +"      Palindromic conjugate: pw({1}/{0}): {2}\n".format(*self.CalcResultsKNO2Primitive["Slope Level"][0], self.CalcResultsKNO2Primitive["Palindromic Conjugate"])
                + "      δ' = Φ{0}(pw({2}/{1})): {3}\n".format(self.CalcResultsKNO2Primitive["Slope Level"][1], *self.CalcResultsKNO2Primitive["Slope Level"][0], self.CalcResultsKNO2Primitive["δ'(PC)"])
                + "     → ε': {} {}\n".format(*sub.halfAll(self.CalcResultsKNO2Primitive["δ'->ε'(PC)"]))
                + "       W({},{}): {} {}\n".format(*self.PrimitiveLissajousType[0:2], *self.CalcResultsKNO2Primitive["W (PC)"])
                + "       V({},{}): {}\n".format(*self.PrimitiveLissajousType[0:2], self.CalcResultsKNO2Primitive["V (PC)"])
                + "  * "+glbl.blue("(KNO1)  Prop 4.11 (calculation from level and slope)")+"\n"
                + "      Christoffel word: cw({1}/{0}): {2}\n".format(*self.CalcResultsKNO2Primitive["Slope Level"][0], self.CalcResultsKNO2Primitive["Christoffel Word"])
                + "      r = φ{0}(cw({2}/{1})): {3}\n".format(self.CalcResultsKNO2Primitive["Slope Level"][1], *self.CalcResultsKNO2Primitive["Slope Level"][0], self.CalcResultsKNO2Primitive['r (4.11)'])
                + "     → V({},{}): {}\n".format(*self.PrimitiveLissajousType[0:2], self.CalcResultsKNO2Primitive['V (4.11)'])
                + "       R: {}\n".format(*self.PrimitiveLissajousType[0:2], self.CalcResultsKNO2Primitive['R (4.11)'])
               ) #if self.KNOtype==2 else ""
           )
           ####
           + "  * "+glbl.blue("Syzygy sequence")+"\n"
           + "      Ω: {}\n".format(self.CalcResultsKNO2Primitive["Ω"])
           + "     → Syzygy: {}\n".format(self.CalcResultsKNO2Primitive["Syzygy (PW)"])
           ####
           + "  * "+glbl.blue("Standard element in Mirror companion class")+"\n"
           + "      φ0 = {1} = {0}\n".format(*self.CalcResultsKNO2['phi0'])
           + "      ε': {} {}\n".format(*sub.halfAll(self.CalcResultsKNO2["st_ε'"]))
           + "      ε : {} {}\n".format(*sub.halfAll(self.CalcResultsKNO2["st_ε"]))
           + "     → R: {} ({})\n".format(*self.CalcResultsKNO2['Rbdpq'])
           + "        = {}\n".format(self.CalcResultsKNO2['R'])
           + "        = {}\n".format(tr.reduceAB(self.CalcResultsKNO2['R']))
           + "       V: {}\n".format(self.CalcResultsKNO2['st_Vbdpq'])
           + "        = {}\n".format(self.CalcResultsKNO2['st_V'])
           + "        = {}\n".format(tr.reduceAB(self.CalcResultsKNO2['st_V']))
           + "       RL: {} {}\n".format(*sub.halfAll(self.CalcResultsKNO2['st_RL']))
           ####
           + "  * "+glbl.blue("Dilatation")+"\n"
           + "      RL: {}\n".format(removeSpace(withKappa(self.CalcResultsKNO2["RL (oddCF)"][0], self.CalcResultsKNO2["κ"][0])))
           + "      PSL2: {}\n".format(removeSpace(withKappa(self.CalcResultsKNO2["PSL2"][0], self.CalcResultsKNO2["κ"][0])))
           + "      Dilatation: "+("{}".format(self.CalcResultsKNO2["Dilatation"][0][1]) if self.CalcResultsKNO2["Dilatation"][0][2] else "{}  (value={})".format(*self.CalcResultsKNO2["Dilatation"][0][0:2]))
           +(("\n       "+glbl.red("error: different dilatations appear.")) if len(self.CalcResultsKNO2["Dilatation"])>1 else "")
           ####
          )


def withKappa(word, kappa):
    if type(word) not in (tuple, list) or type(kappa) not in (tuple, list):
        if len(str(kappa))>2:
            return("κ={} {}".format(kappa, word))
        else:
            return("κ={} {}".format((str(kappa)+" ")[:2], word))
    else:
        return([withKappa(x,k) for x,k in zip(word, kappa)])


def removeSpace(wd):
    return(wd.replace(' )', ')'))



def showResults(self, brType='', **kwargs):
    title = "Lissajous Type: {}\n    (primitive) ({}, {}, {})\nLissajous Curve: sin({}πt) + i sin({}πt+{}) (t : a real number)\n".format(sub.prettyPrint(self.LissajousType), *self.PrimitiveLissajousType, self.CalcResultsKNO2Primitive['phi0'], 2*self.LissajousType[0], 2*self.LissajousType[1], self.LissajousType[2]) + "\n Slope {1}/{0}  /  Level: {2}".format(*self.CalcResultsKNO2Primitive['Slope Level'][0],self.CalcResultsKNO2Primitive['Slope Level'][1])
    #
    KNO2 = brValues(self.CalcResultsKNO2, ["-", "ε'", "ε", 'W', 'V', 'Wbdpq', 'Vbdpq', 'PSL2', 'PSL2/V', 'Dilatation', 'Dilatation/V'])
    #
    standard = brValues(self.CalcResultsKNO2, ["-", "phi0", "st_ε'", "st_ε", 'st_W', 'st_H', 'st_Wbdpq', 'st_Hbdpq', 'AB', 'R', 'Rbdpq', 'st_RL'])
    #
    KNO2Primitive = brValues(self.CalcResultsKNO2Primitive, ["-", 'Christoffel Word', 'Palindromic Conjugate', "δ'", "δ'->ε'", 'r (4.11)', 'V (4.11)', 'R (4.11)'])
    #
    #if self.KNOtype == 1:
    #    KNO1 = brValues(self.CalcResultsKNO1, ["-", "ε'", "ε", 'W', 'H', 'Wbdpq', 'Hbdpq', 'PSL2', 'PSL2/H', 'Dilatation', 'Dilatation/H', 'EndPoint', 'oddCF', 'BDPQ (oddCF)', 'RL (oddCF)'])
    #    #
    #    KNO1Primitive = brValues(self.CalcResultsKNO1Primitive, ["-", 'Slope Level', 'Christoffel Word', 'Palindromic Conjugate', "δ'", "δ'->ε'", 'r (4.11)', 'H (4.11)'])
    #
    return(title+"\n"+KNO2+"\n"+standard+"\n"+KNO2Primitive)


def brValues(brData, brtype='', **kwargs):    
    if 'concluded' in brData.keys() and not brData['concluded']:
        return('')
    elif type(brtype) == str:
        if brtype == '-':
            return('- '*20)
        elif brtype not in brData.keys():
            return('')
        elif 'PSL2' in brtype:
            if type(brData['PSL2SW']) in (tuple, list):
                return('  {}: {}'.format(brtype, '(too large components)' if brData['PSL2SW'][0] else brData[brtype][0]))
            else:
                return('  {}: {}'.format(brtype, '(too large components)' if brData['PSL2SW'] else brData[brtype]))
        elif 'Dilatation' in brtype:
            wk = brData[brtype]
            if type(wk[0]) != str:
                wk = wk[0]
            return('  {}: {}'.format(brtype, wk[1]) if wk[2] else '  {}: {}  (value={})'.format(brtype, wk[0], wk[1]))
        elif 'End' in brtype:
            wk = brData['EndPoint'][0]
            return('  EndPoint: {}'.format(wk[0] if type(wk)==str else wk[0]))
        else:
            wk = brData[brtype]
            if type(wk[0]) in (tuple, list):
                wk = wk[0]
            if type(wk) in (list, tuple) and type(wk[0]) == str:
                if len(wk) > 2:
                    wk = wk[0]
                else:
                    wk = ''.join(wk)
            return('  {}: {}'.format(brtype, sub.prettyPrint(wk)))
    elif type(brtype) in (list, tuple):
        return('\n'.join(sub.removeNull([brValues(brData, x) for x in brtype])))
    else:
        return('error: {}'.brtype)


# Procedures

def set(self, lisType, silent=False) -> None:
    self.clear()
    self.inputArgs = ['', [], [], lisType]
    if type(lisType) != list or len(lisType)<2 or lisType[0]%3 != 1 or lisType[1]%3 != 1:# or (lisType[0]+lisType[1])%2 == 0:
        print('error: {} is not of any Lissajous type.'.format(lisType))
        return(False)
    if len(lisType) == 2:
        self.LissajousType = tuple(lisType[0:2] + [0, (lisType[0]-lisType[1])//3])
    else:
        self.LissajousType = tuple(lisType[0:3] + [(lisType[0]-lisType[1])//3])
    self.PrimitiveLissajousType = reductionLissajousType(self.LissajousType)
    self.LissajousTypeOld = [self.LissajousType[1], -self.LissajousType[0], self.LissajousType[2]/2/math.pi/self.LissajousType[1], self.LissajousType[3]]
    self.KNOtype = 2-(self.LissajousType[3]%2)
    self.calc()
    self.lb.setAB(self.LissajousTypeOld[0:3], self.CalcResultsKNO2['W'][0]*3)
    self.lb.TBsw = False
    if glbl.support_plot_procedures:
        self.lb.plotData['LissajousType'] = {'style':'new', 'new':self.LissajousType, 'old':self.LissajousTypeOld}



# == Main Procedures

def calc(self) -> None:
    calculateKNO2(self.CalcResultsKNO2, self.LissajousType)
    calculateKNO2(self.CalcResultsKNO2Primitive, self.PrimitiveLissajousType)
    self.bdpq = [
        ('KNO2', self.CalcResultsKNO2["Wbdpq"][0], self.LissajousType),
        ('KNO2primitive', self.CalcResultsKNO2Primitive["Wbdpq"][0], self.PrimitiveLissajousType)]
    if self.KNOtype == 1:
        calculateKNO1(self.CalcResultsKNO1, self.LissajousType[:2])
        calculateKNO1(self.CalcResultsKNO1Primitive, self.PrimitiveLissajousType[:2])
        self.bdpq += [('KNO1', self.CalcResultsKNO1["Wbdpq"], self.LissajousType[:2]), 
                      ('KNO1primitive', self.CalcResultsKNO2Primitive["Wbdpq"], self.PrimitiveLissajousType[:2])]
        #


def calculateKNO1(calcResults, lisType) -> None:
    m, n = lisType[:2]
    ell = (m-n)//3
    KNO1_step1_EpsilonPrime(calcResults, lisType)
    KNO1_step2_W(calcResults, m, ell)
    KNO1_step3_H(calcResults, m)
    KNO1_step6_SlopeLevel(calcResults, m, ell)
    KNO1_step7_PSL2(calcResults, m)
    #
    KNO1_step9_Syzygy(calcResults, m)
    calcResults['concluded'] = True


def calculateKNO2(calcResults, lisType) -> None:
    m, n = lisType[0:2]
    ell = (m-n)//3
    KNO2_step1_EpsilonPrime(calcResults, lisType)
    KNO2_step2_WV(calcResults, m, ell)
    #
    KNO1_step6_SlopeLevel(calcResults, m, ell)
    KNO2_step7_PSL2(calcResults, m)
    KNO2_step8_R(calcResults, m, ell, lisType)
    KNO1_step9_Syzygy(calcResults, m)
    calcResults['concluded'] = True



######## KNO1

def KNO1_step1_EpsilonPrime(calcResults, lisType) -> None:
    m, n = lisType[0:2]
    ell = (m-n)//3
    calcResults["ε'"] = KNO1_EpsilonPrime(lisType)
    calcResults["ε"] = ''.join(['+--+'[1+sub.sign(m*ell)+int(ep)] for ep in calcResults["ε'"]])
    calcResults["ε' half"] = sub.halfAll(calcResults["ε'"])
    calcResults["ε' double palindromicity"] = (calcResults["ε' half"][0]==tr.ABrev(calcResults["ε' half"][0]), calcResults["ε' half"][1]==tr.ABrev(calcResults["ε' half"][1]), calcResults["ε' half"][0]==tr.εmir(calcResults["ε' half"][1]))



def KNO2_step1_EpsilonPrime(calcResults, lisType) -> None:
    m, n = lisType[0:2]
    ell = (m-n)//3
    calcResults["ε'"] = KNO2_EpsilonPrime(lisType)
    calcResults["ε"] = [''.join(['+--+'[1+sub.sign(m)+int(ep)] for ep in epL]) for epL in calcResults["ε'"]]
    wk = sub.uniqNum(calcResults["ε'"])
    calcResults["κ"] = [k for ep,k in wk]
    calcResults["ε'(uniq)"] = [ep for ep,k in wk]
    calcResults["ε (uniq)"] = [calcResults["ε"][k] for k in calcResults["κ"]]

    

def KNO1_step2_W(calcResults, m, ell) -> None:
    epL = [glbl.dictSgn[ep] for ep in calcResults['ε']]
    wk = [['A', (1-sub.sign(m)*epL[0])//2]]
    for i in range(2*abs(m)-1):
        wk += [['B', epL[i]], ['A', (epL[i]-epL[i+1])//2]]
    wk += [['B', epL[-1]], ['A', (1-sub.sign(m)*epL[-1])//2]]
    #calcResults['W_org'] = wk
    calcResults['W'] = ''.join([c*(e%2 if c == 'A' else e%3) for c,e in wk])
    calcResults["Wbdpq0"] = tr.toBDPQ(calcResults["W"])
    calcResults["Wbdpq"] = sub.halfAll(tr.reduceBDPQ(tr.ep_bdpq_1(calcResults["ε'"], m, ell)))
    calcResults["Hbdpq"] = calcResults["Wbdpq"][0]
    calcResults["RL"] = tr.bdpq_RL("".join(calcResults["Wbdpq"]))
    calcResults['AB'] = tr.reduceAB(calcResults['W'])


def KNO2_step2_WV(calcResults, m, ell) -> None:
    calcResults['W'] = [''.join(['A'*(1-int(ep))+('B' if e=='+' else 'BB')+'A'*(1-int(ep)) for ep,e in zip(epL, eL)]) for epL,eL in zip(calcResults["ε'(uniq)"], calcResults["ε (uniq)"])]
    calcResults['V'] = [''.join(['A'*(1-int(ep))+('B' if e=='+' else 'BB')+'A'*(1-int(ep)) for ep,e in list(zip(epL, eL))[:m]]) for epL,eL in zip(calcResults["ε'(uniq)"], calcResults["ε (uniq)"])]
    calcResults["Wbdpq0"] = [tr.toBDPQ(w) for w in calcResults["W"]]
    calcResults["Wbdpq"] = [sub.halfAll(tr.reduceBDPQ(tr.ep_bdpq_2(ep, m, ell))) for ep in calcResults["ε'(uniq)"]]
    calcResults["Vbdpq"] = [x[0] for x in calcResults["Wbdpq"]]
    calcResults["RL"] = [tr.bdpq_RL(''.join(w)) for w in calcResults["Wbdpq"]]
    calcResults['AB']  = [tr.reduceAB(w) for w in calcResults['W']]



def KNO1_step3_H(calcResults, m) -> None:
    epL = [glbl.dictSgn[ep] for ep in calcResults['ε']]
    wk = [['A', (1-sub.sign(m)*epL[0])//2]]
    for i in range(abs(m)-1):
        wk += [['B', epL[i]], ['A', (epL[i]-epL[i+1])//2]]
    wk += [['B', epL[abs(m)-1]], ['A', (1-sub.sign(m)*epL[0])//2]]
    calcResults['H'] = ''.join([c*(e%2 if c == 'A' else e%3) for c,e in wk])


def KNO1_step4_AB(calcResults) -> None:
    calcResults['ABH'] = tr.reduceAB(calcResults['H'])


def KNO2_step4_AB(calcResults) -> None:
    calcResults['ABV'] = [tr.reduceAB(v) for v in calcResults['V']]
    #calcResults['RL'] = [tr.toRL(AB, circular=True) for AB in calcResults['AB']]
    #calcResults['Syzygy'] = [tr.AB_Syzygy(AB) for AB in calcResults['AB']]


def KNO1_step6_SlopeLevel(calcResults, m, ell) -> None:
    if calcResults['primitive']:
        calcResults['Slope Level'] = slopeLevel(abs(m), abs(ell))
        calcResults['Christoffel Word'] = cw(calcResults['Slope Level'][0])
        calcResults['Palindromic Conjugate'] = pw(calcResults['Slope Level'][0])
        calcResults["δ'"] = ''.join([str((math.floor(abs(ell/m)*(k+1/2))-math.floor(abs(ell/m)*(k-1/2)))%2) for k in range(1, abs(m)+1)])        
        calcResults["δ'->ε'"] = tr.intΔ(calcResults["δ'"]*2)
        calcResults["W (δ')"] = sub.halfAll(tr.reduceBDPQ(tr.ep_bdpq_1(calcResults["δ'->ε'"], m, ell)))
        calcResults["H (δ')"] = calcResults["V (δ')"] = calcResults["W (δ')"][0]
        calcResults["δ'(CW)"] = wk = tr.phi(calcResults['Christoffel Word'], calcResults['Slope Level'][1])
        calcResults["δ'->ε'(CW)"] = [tr.intΔ((wk:=sub.shift(wk))*2) for k in range(abs(m))]
        calcResults["W (CW)"] = [sub.halfAll(tr.reduceBDPQ(tr.ep_bdpq_1(ep, m, ell))) for ep in calcResults["δ'->ε'(CW)"]]
        calcResults["H (CW)"] = calcResults["V (CW)"] = [w[0] for w in calcResults["W (CW)"]]
        calcResults["δ'(PC)"] = tr.phi(calcResults['Palindromic Conjugate'], calcResults['Slope Level'][1])
        calcResults["δ'->ε'(PC)"] = tr.intΔ(calcResults["δ'(PC)"]*2)
        calcResults["W (PC)"] = sub.halfAll(tr.reduceBDPQ(tr.ep_bdpq_1(calcResults["δ'->ε'(PC)"], m, ell)))
        calcResults["H (PC)"] = calcResults["V (PC)"] = calcResults["W (PC)"][0]
        calcResults['r (4.11)'] = tr.Phi(calcResults['Palindromic Conjugate'], calcResults['Slope Level'][1])
        calcResults['H (4.11)'] = calcResults["V (4.11)"] = tr.r_bdpq(calcResults['r (4.11)'], m)
        calcResults['R (4.11)'] = calcResults['V (4.11)']+(calcResults['V (4.11)'] if ell%2 else tr.mirror(calcResults['V (4.11)']))


def KNO1_step7_PSL2(calcResults, m, overFlow=200) -> None:
    calcResults['PSL2'] = tr.toPSL2(''.join(calcResults['Wbdpq']))
    calcResults['PSL2/H'] = tr.toPSL2(calcResults['Hbdpq'])
    calcResults['PSL2SW'] = len(calcResults['Wbdpq']) > overFlow
    calcResults['EndPoint'] = TB.EndPoint(calcResults['PSL2'])
    calcResults['Dilatation'] = TB.Dilatation(calcResults['PSL2'])
    calcResults['Dilatation/H'] = TB.Dilatation(calcResults['PSL2/H'])
    calcResults['oddCF'] = tr.CF(calcResults['EndPoint'][-1])[1]
    calcResults['r (oddCF)'] = [(x+1)//2 for x in calcResults['oddCF']]
    calcResults['BDPQ (oddCF)'] = tr.CF_bdpq(calcResults['oddCF'], m)
    calcResults['RL (oddCF)'] = ' '.join(sub.halfAll(tr.CF_RL(calcResults['oddCF'])))


def KNO2_step7_PSL2(calcResults, m, overFlow=200) -> None:
    calcResults['PSL2'] = [tr.toPSL2(''.join(w)) for w in calcResults['Wbdpq']]
    calcResults['PSL2/V'] = [tr.toPSL2(v) for v in calcResults['Vbdpq']]
    calcResults['PSL2SW'] = [len(w)>overFlow for w in calcResults['Wbdpq']]
    calcResults['EndPoint'] = [TB.EndPoint(PSL2) for PSL2 in calcResults['PSL2']]
    calcResults['Dilatation'] = sub.uniqAt([TB.Dilatation(PSL2)+[k] for PSL2,k in zip(calcResults['PSL2'], calcResults["κ"])])
    calcResults['Dilatation/V'] = sub.uniqAt([tuple(TB.Dilatation(PSL2)+[k]) for PSL2,k in zip(calcResults['PSL2/V'], calcResults["κ"])])
    calcResults['oddCF'] = [tr.CF(ep[-1])[1] for ep in calcResults['EndPoint']]
    calcResults['r (oddCF)'] = [[(x+1)//2 for x in cf] for cf in calcResults['oddCF']]
    calcResults['BDPQ (oddCF)'] = [tr.CF_bdpq(cf, m) for cf in calcResults['oddCF']]
    calcResults['RL (oddCF)'] = [' '.join(sub.halfAll(tr.CF_RL(cf))) for cf in calcResults['oddCF']]


def KNO2_step8_R(calcResults, m, ell, lisType):
    phi = lisType[2] if len(lisType)>2 else 0
    wk = [sub.sign(m*ell)*(sub.sign(m)-3*ell), 2*abs(m)]
    d = sub.gcd(*wk)
    calcResults['phi0'] = wk[0]/wk[1]*math.pi, "{}/{} π".format(wk[0]//d, wk[1]//d)
    calcResults["st_ε'"] = canEp = KNO2_EpsilonPrime_aux(abs(m), ell, calcResults['phi0'][0], 0)
    calcResults["st_ε"] = canE = ''.join(['+--+'[1+sub.sign(m*ell)+int(ep)] for ep in calcResults["st_ε'"]])
    calcResults['st_W'] = ''.join(['A'*(1-int(ep))+('B' if e=='+' else 'BB')+'A'*(1-int(ep)) for ep,e in zip(canEp, canE)])
    calcResults['st_V'] = ''.join(['A'*(1-int(ep))+('B' if e=='+' else 'BB')+'A'*(1-int(ep)) for ep,e in zip(sub.half(canEp), sub.half(canE))])
    calcResults['st_Wbdpq'] = sub.halfAll((canW:=tr.reduceBDPQ(tr.ep_bdpq_2(canEp, m, ell))))
    calcResults['st_Vbdpq'] = calcResults['st_Wbdpq'][0]
    calcResults['st_AB'] = tr.reduceAB(calcResults['st_W'])
    calcResults['st_ABV'] = tr.reduceAB(calcResults['st_V'])
    calcResults['st_RL'] = tr.toRL(calcResults['st_AB'])
    #calcResults['st_Syzygy'] = tr.AB_Syzygy(calcResults['st_AB'])
    calcResults['R'] = calcResults['st_W']
    calcResults['Rbdpq'] = calcResults['st_Wbdpq']
    #
    calcResults["ε'(uniq)"] = [ep+('' if sub.compaireCircularWords(canEp, ep) else glbl.red(' (mirror)')) for ep in calcResults["ε'(uniq)"]]
    calcResults["ε (uniq)"] = [ep+('' if sub.compaireCircularWords(canE, ep) else glbl.red(' (mirror)')) for ep in calcResults["ε (uniq)"]]
    calcResults["Vbdpq"] = [v if sub.inCircularWords(v, canW) else v+glbl.red('  (mirror)') for v in calcResults["Vbdpq"]]





def KNO1_step9_Syzygy(calcResults, m):
    if calcResults['primitive']:
        calcResults['Ω'] = tr.PhiSZ(calcResults["Palindromic Conjugate"], calcResults["Slope Level"][1], m)
        calcResults['Syzygy (PW)'] = tr.omegaToSZ(calcResults['Ω'])


# Reduction of Lissajous Type
def reductionLissajousType(lisType: list) -> list:
    m,n = lisType[:2]
    while abs(m)>=abs(n) or abs(n)>2*abs(m):
        if abs(m)>abs(n):
            n,m = m,n
        else:
            n = sub.min_abs((n+3*m)%(6*m)-3*m, (2*m-n+3*m)%(6*m)-3*m)
    return(m, n)


def primitiveQ(m, n):
    return(sub.gcd(m, n)==1 and m%3==1 and n%3==1 and m*n<0 and abs(m)<abs(n)<2*abs(m))


def KNO1_EpsilonPrime(lisType) -> str:
    if len(lisType) > 1:
        m, n = lisType[:2]
        ell = abs((m-n)//3)
    else:
        return('error')
    ma = abs(m)
    return(''.join([str(math.floor(ell/ma*k-ell/2/ma)%2) for k in range(1, 2*abs(ma)+1)]))


def KNO2_EpsilonPrime(lisType) -> str:
    if len(lisType) >= 4:
        m, n, phi, ell = lisType[0:4]
    elif len(lisType) == 3:
        m, n, phi = lisType
        ell = (m-n)//3
    elif len(lisType) == 2:
        m, n = lisType
        ell = (m-n)//3
        phi = 0
    else:
        return('error')
    return([KNO2_EpsilonPrime_aux(abs(m), ell, phi, kappa) for kappa in range(0, 2*abs(m))])


def KNO2_EpsilonPrime_aux(m, ell, phi, kappa):
    return(''.join([str(math.floor(ell*(k+kappa/(2*m))/m - ell/2/m - phi/math.pi)%2) for k in range(1,2*m+1)]))


# Slope and Level
def slopeLevel(m, ell) -> list:
    lvl = level(m, ell)
    return(slope(m, ell, lvl), lvl)

def level(m, ell) -> int:
    return(math.floor((2*ell-m)/(3*ell-2*m)))

def level_wk(m, ell) -> list:
    return([(2*ell-m)/(3*ell-2*m), (ell-m)/(-3*ell+2*m)])

def slope(m, ell, level) -> list:
    return([(3*level+1)*ell-(2*level+1)*m, -(3*level-2)*ell+(2*level-1)*m])


def fractSlope(m, ell, level) -> Fraction:
    wk = slope(m, ell, level)
    return(Fraction(wk[1], wk[0]))


# == Christoffel Word
def ChristoffelWord(p, q) -> str:
    return(''.join([str(math.floor((k+1)*q/(p+q))-math.floor(k*q/(p+q))) for k in range(p+q)]))


def cw(pq) -> str:
    return(ChristoffelWord(*pq))


# Palindromic Conjugate of Christoffel Word
def pw(pq) -> str:
    c = cw(pq)
    p,q = pq
    k,l = sub.extGcd(p, q)[1:]
    j = q-k
    i = p+l
    if (i+j)%2 == 1:
        return(c[(k-l)//2:]+c[:(k-l)//2])
    else:
        return(c[k-l+(i+j)//2:]+c[:k-l+(i+j)//2])


# Φ-作用素、φ-作用素　
def phi(s, level) -> str:
    return(''.join(['101'*(level-1+int(c))+'1' for c in s]))


def Phi(s, level) -> list:
    return([level+int(c) for c in s])



# Christoffel Word cw(q/p) and Palindromic Conjugate pw(q/p)
def ChristoffelWord_NG(m, ell) -> list:
    p, q = slopeLevel(m, ell)[0]
    return(cw(p, q), pw(p, q))


# standard W

def Wstandard(arg1, *arg2):
    type1 = type(arg1)
    if type1 == int:
        if len(arg2) < 1 or type(arg2[0]) != int:
            return(False)
        else:
            return(Wstandard(slopeLevel(abs(arg1), abs((arg1-arg2[0])//3))))
    elif type1 == str:
        return(Wstandard((arg1, 1), *arg2))
    elif type1 in (list, tuple):
        if len(arg1) < 2:
            return(False)
        elif type(arg1[0]) == str:
            if '/' in arg1[0]:
                return(Wstandard(([arg1[0].split('/')[1], arg1[0].split('/')[0]], arg1[1]), *arg2))
            else:
                return(False)
        elif type(arg1[0]) == int and type(arg1[1]) == int:
            if len(arg2) < 1:
                return(Wstandard((arg1, 1)))
            else:
                return(Wstandard((arg1, arg2[0]), *arg2[1:]))
        elif len(arg2) < 1 or type(arg2[0]) != str:
            return(Wstandard(arg1, tr.cw(arg1[0])))
        else:
            cwo = tr.PhiOdd(arg2[0], arg1[1])
            cwo2 = cwo*2
            w = ''.join(['R'*cwo2[2*j]+'L'*cwo2[2*j+1] for j in range(len(cwo))])
            return(w, (arg1, arg2, cwo))
    else:
        return(False)

def Wstandard(arg1, *arg2):
    type1 = type(arg1)
    if type1 == int:
        if len(arg2) < 1 or type(arg2[0]) != int:
            return(False)
        else:
            ell = (arg1-arg2[0])//3
            sl = Lis.slopeLevel(abs(arg1), abs(ell))
            cw = tr.cw(sl[0])
            return(Wstandard(sl, cw))
    elif type1 == str:
        return(Wstandard((arg1, 1), *arg2))
    elif type1 in (list, tuple):
        if len(arg1) < 2:
            return(False)
        elif type(arg1[0]) == str:
            if '/' in arg1[0]:
                return(Wstandard(([arg1[0].split('/')[0], arg1[0].split('/')[1]], arg1[1]), *arg2))
            else:
                return(False)
        elif type(arg1[0]) == int and type(arg1[1]) == int:
            if len(arg2) < 1:
                return(Wstandard((arg1, 1), tr.cw(arg1)))
            else:
                return(Wstandard((arg1, arg2[0]), *arg2[1:]))
        elif len(arg2) < 1:
            return(Wstandard(arg1, tr.cw(arg1[0])))
        elif type(arg2[0]) != str:
            return(Wstandard(arg1))
        else:
            cwo = tb.PhiOdd(arg2[0], arg1[1])
            cwo2 = cwo*2
            w = ''.join(['R'*cwo2[2*j]+'L'*cwo2[2*j+1] for j in range(len(cwo))])
            return(w)
    else:
        return(False)


##


