####
# proc_TB
#
# procedures for ThreeBraid class
#
# version info : 1.2.4  2022/10/24  H.Ogawa
#


import numpy as np
import math
import cmath
from . import globalParamaters as glbl
from . import subprocedures as sub
from . import translation as tr


#brType = sub.braidReprType
#pprint = sub.prettyPrint


# Initialize

def init(self) -> None:
    self.inputData = ''
    self.braid = {'braidStyle':'braid', 'startArea':'0+', 'circular':False, 'concluded':False}
    self.circular = {'braidStyle':'circular word', 'startArea':'0+', 'circular':True, 'concluded':False}
    self.loop = {'braidStyle':'loop on shape sphere', 'startArea':'0+', 'circular':True, 'concluded':False}
    self.loop3 = {'braidStyle':'loop/3 on shape sphere', 'startArea':'0+', 'circular':True, 'concluded':False}
    self.loop6 = {'braidStyle':'loop/6 on shape sphere', 'startArea':'0+', 'circular':True, 'concluded':False}
    self.TripleQ = False
    self.Fexp = []


# set

def set(self, dataStr, silent=False, startArea='0+', overFlow=200, **kwargs) -> None:
    init(self)
    self.inputData = dataStr
    self.startArea = startArea
    if sub.braidReprType(dataStr)=='Syzygy':
        self.startArea = dataStr[0]+startArea[-1]
    for bs in glbl.braidStyles[0:3]:
        update(self.ref(bs), dataStr, startArea=self.startArea)
    self.TripleQ = sub.tripleQ(self.loop['AB'])
    self.andHalfQ = False
    if self.TripleQ:
        for ky in glbl.braidTypes[:6]+['ABex', 'Frieze']:
            self.loop3[ky] = sub.oneThird(self.loop[ky])
        self.loop3['firstEntry'] = self.loop3['AB']
        for ky in ('startArea', 'circular', 'T_interval'):
            self.loop3[ky] = self.loop[ky]
        self.loop3['forPlot'] = tr.toClosureSigma(self.loop3['Sigma'], **kwargs)
        self.loop3['pdConj'] = tr.palindromicQ(self.loop3['BDPQ'])
        self.loop3['PSL2'] = tr.toPSL2(self.loop3['AB'])
        self.loop3['PSL2SW'] = len(self.loop3['RL']) > overFlow
        self.loop3['EndPoint'] = EndPoint(self.loop3['PSL2'])
        self.loop3['Dilatation'] = Dilatation(self.loop3['PSL2'])
        #
        self.loop3['concluded'] = True
        #
        self.andHalfQ = sub.halfABQ(self.loop3['AB'])
        if self.andHalfQ:
            for ky in glbl.braidTypes[:6]+['ABex', 'Frieze']:
                self.loop6[ky] = sub.halfBR(self.loop3[ky])
            self.loop6['firstEntry'] = self.loop6['AB']
            for ky in ('startArea', 'circular', 'T_interval'):
                self.loop6[ky] = self.loop3[ky]
            self.loop6['forPlot'] = tr.toClosureSigma(self.loop6['Sigma'], **kwargs)
            self.loop6['pdConj'] = tr.palindromicQ(self.loop6['BDPQ'])
            self.loop6['PSL2'] = tr.toPSL2(self.loop6['AB'])
            self.loop6['PSL2SW'] = len(self.loop6['RL']) > overFlow
            #self.loop6['EndPoint'] = EndPoint(self.loop6['PSL2'])
            #self.loop6['Dilatation'] = Dilatation(self.loop6['PSL2'])
            #
            self.loop6['concluded'] = True
            #
    if not silent: print('The process completed successfully.')


# Display Values for show()

def values(brData) -> str:
    if brData['concluded']:
        return(
            '- '*20
            + '\nGiven: {}'.format(brData['firstEntry'])
            + '\n   style: {}\n'.format(brData['braidStyle'])
            + brValues(brData, braidTypes)
            + '\n'+'- '*20+'\n'
            + brValues(brData, 'Dilatation')
        )
    else:
        return('')


def brValues(brData, brtype, **kwargs):
    if 'concluded' in brData.keys() and not brData['concluded']:
        return('')
    elif type(brtype) == str:
        if 'tyle' in brtype and 'braidStyle' in brData.keys():
            return('Braid Style: {}'.format(brData['braidStyle']))
        elif 'type' in brtype and 'type' in brData.keys():
            return('Lissajous Type: {}'.format(brData['type']))
        elif 'paramater' in brtype and 'paramater' in brData.keys():
            return('Linear Paramater: {}'.format(brData['paramater']))
        elif '-' in brtype:
            return('- '*20)
        elif 'first' in brtype or 'given' in brtype or 'input' in brtype or 'raw' in brtype and 'firstEntry' in brData.keys():
            return('  raw data: {}'.format(brData['firstEntry'] if brData['firstEntry']!='' else "''"))
        elif brtype not in brData.keys():
            return('')
        elif brtype == 'PSL2':
            return('  PSL2: {}'.format('(too large components)' if brData['PSL2SW'] else brData['PSL2']))
        elif brtype in ('Dilatation', 'EndPoint'):
            return('  {}: {}'.format(brtype, brData[brtype][1]) if brData[brtype][2] else '  {}: {}  (value={})'.format(brtype, brData[brtype][0], brData[brtype][1]))
        elif brtype in ('BDPQ', 'Frieze'):
            return('  BDPQ (Frieze): {}'.format(sub.prettyPrint(brData[brtype], **kwargs)))
        else:
            return('  {}: {}'.format(brtype, sub.prettyPrint(brData[brtype], **kwargs)))
    elif type(brtype) == list:
        return('\n'.join(sub.removeNull([brValues(brData, x, **kwargs) 
                                     for x in brtype])))
    else:
        return('error: {}'.brtype)


# update ThreeBraid data

def update(brData, wd, startArea='0+', overFlow=200) -> None:
    brtype = sub.braidReprType(wd)
    if 'error' in brtype:
        brData['concluded'] = False
    else:
        kwargs = {'circular':brData['circular'], 'startArea':startArea}
        brData['firstEntry'] = wd if brtype != 'PSL2' else tr.PSL2_ST(wd)
        brData['multiple'] = 1 if 'loop' not in brData['braidStyle'] or '3' in brData['braidStyle'] or '6' in brData['braidStyle'] else tr.toS3(tr.AB_ABex(tr.toAB(brData['firstEntry'], **kwargs), **kwargs))
        brData['T_interval'] = (0, 1/brData['multiple'])
        if brData['multiple'] > 1:
            if brtype == 'Sigma':
                brData['firstEntry'] = ' '.join([brData['firstEntry'] for i in range(brData['multiple'])])
            else:
                brData['firstEntry'] = brData['firstEntry']*brData['multiple']
        #
        wk = tr.toAB(brData['firstEntry'], **kwargs)
        if 'circular' in brData['braidStyle'] and brtype == 'Syzygy':
            wk = tr.toAB(brData['firstEntry'])
        brData['ABex'] = tr.reduceABex(tr.AB_ABex(wk, **kwargs), **kwargs)
        brData['startArea'] = tr.ABex_startArea(brData['ABex'], **kwargs)
        kwargs['startArea'] = brData['startArea']
        #brData['AB'] = tr.reduceAB(wk, **kwargs)
        brData['AB'] = tr.ABex_AB(brData['ABex'], **kwargs)
        brData['Sigma'] = tr.toSigma(brData['AB'], **kwargs)
        brData['Syzygy'] = tr.ABex_Syzygy(brData['ABex'], **kwargs)
        brData['forPlot'] = tr.toClosureSigma(brData['Sigma'], **kwargs)
        brData['BDPQ'] = tr.toBDPQ(brData['AB'], **kwargs)
        brData['Frieze'] = brData['BDPQ']
        brData['pdConj'] = tr.palindromicQ(brData['BDPQ'])
        brData['RL'] = tr.toRL(brData['AB'], **kwargs)
        brData['ST'] = tr.toST(brData['AB'], **kwargs)
        #
        brData['PSL2'] = tr.toPSL2(brData['AB'])
        brData['PSL2SW'] = len(brData['RL']) > overFlow
        brData['EndPoint'] = EndPoint(brData['PSL2'])
        brData['Dilatation'] = Dilatation(brData['PSL2'])
        #
        brData['concluded'] = True
    #


# EndPoint

def EndPoint(a: list, overFlow=1.0e+30) -> list:
    if a[1][0] == 0:
        return(['---', '---', False])
    else:
        wk = [a[1][0], a[1][1]-a[0][0], -a[0][1]]
        d = sub.sign(wk[0])*sub.gcd(*wk)
        wk = [x//d for x in wk]
        return([evaluateSym(wk), evaluate(wk), abs(wk[1])>overFlow, wk])


# Dilatation

def Dilatation(a: list, overFlow=1.0e+30) -> list:
    wk = [1, -abs(sub.tr2(a)), sub.det2(a)]
    return([evaluateSym(wk), evaluate(wk), -wk[1]>overFlow])


# evaluations

def evaluate(a: list) -> float:
    if a[1]*a[1]-4*a[0]*a[2] > 1.0e+300:
        return('overFlow')
    else:
        return((-a[1]+sub.sign(a[0])*sub.sqrtChop(a[1]*a[1]-4*a[0]*a[2]))/(2*a[0]))


def evaluateSym(a: list) -> str:
    d = sub.sign(a[0])*sub.gcd(*a)
    wk = -a[1]//d, (a[1]*a[1]-4*a[0]*a[2])//d//d, 2*a[0]//d
    return('({} + sqrt({})) / {}'.format(wk[0], wk[1], wk[2]))


# equality and inequality

def eq(self, other):
    #if isinstance(other, self.__class__) or isinstance(self, other.__class__):
    return({'braid':self.braid['AB'] == other.braid['AB'], 
            'circular':sub.compaireCircularWords(self.circular['AB'], other.circular['AB'])})
    #else:
    #    raise NotImplementedError()

def ne(self, other):
    #if isinstance(other, self.__class__) or isinstance(self, other.__class__):
    return({'braid':self.braid['AB'] != other.braid['AB'],
            'circular':not sub.compaireCircularWords(self.circular['AB'], other.circular['AB'])})
    #else:
    #    raise NotImplementedError()

def le(self, other):
    #if isinstance(other, self.__class__) or isinstance(self, other.__class__):
    return({'braid':self.braid['AB'] in other.braid['AB'],
            'circular':sub.inCircularWords(self.circular['AB'], other.circular['AB'])})
    #else:
    #    raise NotImplementedError()

def ge(self, other):
    #if isinstance(other, self.__class__) or isinstance(self, other.__class__):
    return({'braid':other.braid['AB'] in self.braid['AB'],
            'circular':sub.inCircularWords(other.circular['AB'], self.circular['AB'])})
    #else:
    #    raise NotImplementedError()

def lt(self, other):
    #if isinstance(other, self.__class__) or isinstance(self, other.__class__):
    return({'braid':(self.braid['AB'] in other.braid['AB']) and (self.braid['AB'] != other.braid['AB']),
            'circular':sub.inCircularWords(self.circular['AB'], other.circular['AB']) and (not sub.compaireCircularWords(self.circular['AB'], other.circular['AB']))})
    #else:
    #    raise NotImplementedError()

def gt(self, other):
    #if isinstance(other, self.__class__) or isinstance(self, other.__class__):
    return({'braid':(other.braid['AB'] in self.braid['AB']) and (self.braid['AB'] != other.braid['AB']),
            'circular':sub.inCircularWords(other.circular['AB'], self.circular['AB']) and (not sub.compaireCircularWords(self.circular['AB'], other.circular['AB']))})
    #else:
    #    raise NotImplementedError()




