################
# subprocedures.py
#
# Sub-Procedures
#
#    sub.braidReprType : repr. types
#    sub.compCW : compaire of circular words
#    sub.inCW   : subset of circular words
#
# version info : 1.2.4  2022/10/24  H.Ogawa
#


import math
import cmath
import numpy as np
from fractions import Fraction

from . import globalParamaters as glbl


# 表現型

def braidReprType(wd, combined=''):
    w = chop(wd)
    if w == '' or w == []:
        return('empty word')
    elif type(w) not in [list, str]:
        return(type(w))
    elif combined == '' and type(w[0]) == list and len(w[0]) == 2 and type(w[0][0]) == str:
            return(braidReprType(w[0][0], combined=' comb'))
    elif 'S' in w or 'T' in w or 's' in w or 't' in w:
        return('ST'+combined)
    elif 'R' in w or 'L' in w or 'r' in w or 'l' in w:
        return('RL'+combined)
    elif 'p' in w or 'q' in w or 'b' in w or 'd' in w:
        return('BDPQ'+combined)
    elif type(w) == list and len(w) == 2 and type(w[0]) == list and len(w[0]) == 2 and type(w[1]) == list and len(w[1]) == 2 and type(w[0][0]) != str and type(w[1][0]) != str:
        if det2(w) != 1:
            return('error: {} is not in PSL2.'.format(w))
        else:
            return('PSL2')
    elif (' ' in w and ('1' in w or '2' in w)) or 'σ' in w or '-1' in w or '-2' in w:
        if 'A' in w:
            return('ABex'+combined)
        else:
            return('Sigma'+combined)
    elif '0' in w or '1' in w or '2' in w:
        return('Syzygy'+combined)
    elif 'A' in w or 'B' in w:
        if '+' in w or '-' in w:
            return('ABex'+combined)
        else:
            return('AB'+combined)
    elif type(w) == list and len(w) <= 3 and type(w[0]) != list:
        return('Linear Paramater / Lissajous Type')
    elif type(w) == list and set(w) == {1, -1}:
        return('epsilon')
    else:
        return('error: {} is not supported.'.format(w))


# 基本関数  若干の修正など

#  符号関数
def sign(c: float) -> int:
    return(-1 if c<0 else 1)

#  平方根 （若干のオーバーフロー対策付き）
def sqrtChop(x: float, overFlow=1.0e+300) -> complex:
    if x < 0:
        return(sqrtChop(-x, overFlow=overFlow)*1j)
    if x > overFlow:
        return(math.exp(math.log(x)/2))
    else:
        return(math.sqrt(x))

#  拡張GCD
def extGcd(a: int, b: int) -> list:
    if b:
        d, y, x = extGcd(b, a % b)
        y -= (a // b)*x
        return d, x, y
    return a, 1, 0

def gcd(a, *args: list) -> int:
    ans = a
    for x in args:
        ans = math.gcd(ans, x)
    return(ans)


#  modulo 3
def mod3(x):
    return(x%3)

def mod3m(x):
    y = mod3(x)
    #return(y if 2*y < 3 else y-3)
    return(y if 2*y > 3 else y-3)

#  floor + mod 3
def intMod3(x):
    return(mod3(floor(x)))

def floor(x):
    return(math.floor(x))

# 小数部分
def fracPart(x):
    return(x%1)

#  実数切り詰め
def chop(wd, prec=10**5):
    if type(wd) in [list, str]:
        if ':' in wd:
            return(wd[:wd.index(':')])
        else:
            return(wd)
    elif 'float' in str(type(wd)):
        return(int(prec*wd)/prec)
    else:
        return(wd)

# 最小絶対値
def min_abs(m: float, n: float) -> float:
    return(m if abs(m)<abs(n) else n)


# 範囲内？
def inInterval(x, ab):
    return(min(max(x, ab[0]), ab[1]))

# ２次正方行列
def matProd(a, *args):
    ans = a
    for x in args:
        ans = matProd2(ans, x)
    return(ans)

def matProd2(a, b):
    return([[a[0][0]*b[0][0]+a[0][1]*b[1][0], 
             a[0][0]*b[0][1]+a[0][1]*b[1][1]], 
            [a[1][0]*b[0][0]+a[1][1]*b[1][0], 
             a[1][0]*b[0][1]+a[1][1]*b[1][1]]])

def matPow(a, n):
    if n == 0:
        return([[1, 0], [0, 1]])
    elif n < 0:
        return(matPow(matInv(a), -n))
    else:
        ans = a
        for i in range(n-1):
            ans = matProd2(ans, a)
        return(ans)

def matInv(a):
    d = det2(a)
    if d == 0:
        return('not regular')
    elif abs(d) == 1:
        return([[d*a[1][1], -d*a[0][1]], 
                [-d*a[1][0], d*a[0][0]]])
    else:
        return([[a[1][1]/d, -a[0][1]/d], 
                [-a[1][0]/d, a[0][0]/d]])

def matConj(a, b):
    return(matProd(matInv(b), a, b))

def matTrans(a):
    return([[a[0][0], a[1][0]], [a[0][1], a[1][1]]])

def mm(a):
    return([[-x for x in y] for y in a])

def det2(a: list) -> float:
    return(a[0][0]*a[1][1]-a[0][1]*a[1][0])

def tr2(a: list) -> float:
    return(a[0][0]+a[1][1])

#  PSL2 としての比較
def PSLeq(a: list, b: list) -> bool:
    return(a == b or a == PSLminus(b))

def PSLminus(a: list) -> list:
    if type(a) != list:
        return(-a)
    else:
        return([PSLminus(x) for x in a])


# 文字列
#  巡回文字列比較
def compaireCircularWords(a: list, b: list, simple=True) -> bool:
    if len(a) == len(b):
        wk = b
        for k in range(len(b)):
            if a == wk:
                if simple:
                    return(True)
                else:
                    return((True, k))
            wk = shift(wk)
    if simple:
        return(False)
    else:
        return((False, ''))


def inCircularWords(a, b, simple=True):
    if len(a) <= len(b):
        wk = b
        for k in range(len(b)):
            if wk.startswith(a):
                if simple:
                    return(True)
                else:
                    return((True, k))
            wk = shift(wk)
    if simple:
        return(False)
    else:
        return((False, ''))


def shift(word, s=1):
    return(word[s:]+word[:s])


# tuple の or
def orExt(ex, *args):
    if ex:
        return(True)
    elif len(args) > 1:
        return(ex or orExt(*args))
    else:
        return(ex)


#  ２分割/３分割
def half(a: str) -> str:
    return(a[:len(a)//2])

def halfAll(a: str) -> list:
    return([a[:len(a)//2], a[len(a)//2:]])

def halfQ(a: str) -> list:
    wk = halfAll(a)
    return(wk[0] == wk[1])

def oneThird(a: str) -> str:
    return(a[:len(a)//3])

def oneThirdAll(a: str) -> list:
    return([a[:len(a)//3], a[len(a)//3:2*len(a)//3], a[2*len(a)//3:]])

def tripleQ(a: str) -> bool:
    wk = oneThirdAll(a)
    return(wk[0] == wk[1] and wk[1] == wk[2])

def doubleQ(a: str) -> bool:
    wk = halfAll(a)
    return(wk[0] == wk[1])

def halfAllAB(wd):
    return(tuple([''.join(x) for x in halfAll(wd.replace('A', ' A').split())]))

def halfAB(wd):
    return(''.join(half(wd.replace('A', ' A').replace('S', ' S').split())))

def halfABQ(wd):
    wdn = wd.replace('BB', '2').replace('B', '1')
    a,b1 = halfAllAB(wdn)
    b2 = b1.replace('2', '3').replace('1','2').replace('3', '1')
    return(a==b1 or a==b2)

def halfBR(wd):
    brt = braidReprType(wd)
    if 'AB' in brt or brt == 'ST':
        return(halfAB(wd))
    elif brt == 'Syzygy':
        return(half(wd))
    elif brt == 'Sigma':
        return(' '.join(half(wd.split())))
    else:
        return(half(wd))


# == 文字列 <-> 重複表示
def toSimple(w):
    wd = sub.chop(w)
    if braidReprType(wd) == 'PSL2':
        return(wd)
    else:
        return(''.join([toSimplesub(x) for x in wd]))

def toSimplesub(x):
    if type(x) != list:
        return(str(x))
    else:
        y = str(x[0]).replace('σ', ' ')
        if x[1]>0:
            return(y*x[1])
        else:
            return(glbl.dictInv[y]*(-x[1]))

def toMultiple(wd):
    brtype = braidReprType(wd)
    if brtype == 'PSL2':
        return(wd)
    elif brtype == 'Sigma':
        return([['σ'+x[0], x[1]] if x[0][0] != '-' else ['σ'+x[0][1:], -x[1]] for x in splitSame(wd.split())])
    else:
        return(splitSame(wd))

def splitSame(st):
    x = st[0]
    c = 1
    ans = []
    for i in range(1, len(st)):
        if x != st[i]:
            ans += [[x, c]]
            x,c = st[i],1
        else:
            c += 1
    return(ans + [[x, c]])


#  文字列/リスト  取り除き
def removeFrom(wds, x):
    if type(wds) == list:
        return([s for s in wds if s != x])
    elif type(wds) == str:
        return(''.join([s for s in wds if s != x]))
    else:
        return(wds)

def removeNull(wds):
    return(removeFrom(wds, ''))


# リスト
#  重複数え上げ
def gatherList(lst: list) -> list:
    if len(lst) == 0: return('')
    ans = []
    c = 1
    now = lst[0]
    for s in lst[1:]:
        if s == now:
            c += 1
        else:
            ans += [[now, c]]
            now = s
            c = 1
    ans += [[now, c]]
    return(ans)

def uniq(lst):
    wk = []
    return([x for x in lst if x not in wk and not wk.append(x)])

def uniqNum(lst):
    wk = []
    return([(x,e) for e,x in enumerate(lst) if x not in wk and not wk.append(x)])

def uniqAt(lst, col=0):
    wk = []
    return([x for x in lst if x[col] not in wk and not wk.append(x[col])])


# indexAll
def indexAll(lst, word):
    if type(lst) in (list, tuple):
        return(tuple([k for k,w in enumerate(lst) if w==word]))
    elif type(lst) == str:
        return(tuple([k for k in range(len(lst)) if lst[k:].startswith(word)]))
    else:
        return([])

def find(lst, word):
    return(lst.index(word) if word in lst else False)


# Pretty Print for Braid
def prettyPrint(brData, SimpleSigma=False, **kwargs):
    brType = braidReprType(brData)
    if brType == 'Sigma':
        if SimpleSigma:
            ans = brData
        else:
            ans = ' '.join(['σ'+s[-1]+(('^'+str(-c)) if len(s)==2 else ('' if c<2 else ('^'+str(c)))) for s,c in gatherList(brData.split())])
    elif brType == 'RL':
        ans = ' '.join([s.upper()+(('^{}' if s.isupper() else '^(-{})').format(c) if c>1 or s.islower() else '') for s,c in gatherList(brData)])
    elif brType == 'ST':
        ans = ''.join([(' S' if c=='S' else 'T')+('' if n==1 else str(n)) for c,n in gatherList(brData)])
        if ans[0] == ' ':
            ans = ans[1:]
    elif brType == 'ABex':
        ans = brData.replace('A', 'A(').replace('+', '+)').replace('-', '-)')
    elif brType == Fraction:
        ans = str(brData)
    elif brType == 'Paramater':
        ans = ', '.join([str(c) for c in brData])
    elif brType == 'null':
        return('')
    elif type(brType) != str:
        ans = str(brData)
    elif 'Li' in brType:
        ans = '['+(', '.join([str(s) for s in brData]))+']'
    elif brType == 'epsilon':
        ans = ' '+(''.join(['- +'[s+1] for s in brData]))
    else:
        ans = str(brData)
    if ans == '' or ans == ' ':
        return("''")
    else:
        return(ans)


#  簡易プリント. 分数、実数切り詰め
def pprintR(*args, **kwargs):
    if len(args) > 1:
        xs = tuple(args)
    else:
        xs = args[0]
    if type(xs) == list:
        return([pprintR(x, **kwargs) for x in xs])
    elif type(xs) == tuple:
        return(tuple([pprintR(x, **kwargs) for x in xs]))
    elif 'array' in str(type(xs)):
        return(np.array([pprintR(x, **kwargs) for x in xs]))
    elif 'float' in str(type(xs)):
        return(chop(xs, **kwargs))
    else:
        return(str(xs))


# 簡易整形 print
def tinyFprint(lst, n=5, head=''):
    if type(lst) not in (list, tuple):
        return(lst)
    else:
        r = len(lst)
        return(('\n'.join([head+('\t'.join(['{}']*n))]*(r//n)+([head+('\t'.join(['{}']*(r%n)))] if r%n else []))).format(*lst))


#  str -> int list
def strToIntList(word):
    if type(word) == str:
        return([int(x) for x in word])
    else:
        return(word)

#  int list -> str
def toStr(word):
    if type(word) in (list, tuple):
        return(''.join(toStr(x) for x in word))
    else:
        return(str(word))


#  変換 : R2 <-> C <-> 極座標
def R2toC(xy, ABrate=glbl.options['ABrate'], **kwargs):
    return(complex(ABrate*xy[0], xy[1]))

def CtoR2(z):
    return(z.real, z.imag)

def CtoPolar(z):
    return(cmath.polar(z))

def PolartoC(pol):
    return(cmath.rect(*pol))


# reduction : Riemann sphere -> C
def cutoff(x):
    return(math.atan(x)*4/math.pi)

def cutoffC(z):
    r, s = CtoPolar(z)
    return(PolartoC((cutoff(r), s)))

def cutoffC2(z):
    r, s = CtoPolar(z)
    return(PolartoC((cutoff(r), 2*s)))


# 変換 : R2 <-> Sphere (in R3)
def R2toSphere(xy):
    x, y = xy
    z2 = x**2+y**2
    k = 2/(1+z2)
    return(k*x, k*y, 1-k)

def PhaseonSphere(xyz):
    x, y, z = xyz
    return(cmath.phase(R2toC((x, y))), math.asin(z))

def CtoSphere(z):
    return(R2toSphere(CtoR2(z)))

def C2toSS(z1, z2):
    r1,t1 = cmath.polar(z1)
    r2,t2 = cmath.polar(z2)
    t = t1-t2
    z = (-r1**2+r2**2)/(r1**2+r2**2)
    z2 = math.sqrt(1-z**2)
    x, y = z2*math.cos(t), z2*math.sin(t)
    return(x, y, z)

def R2toSS(z1, z2):
    r1 = np.sqrt(z1[0]**2+z1[1]**2)
    r2 = np.sqrt(z2[0]**2+z2[1]**2)
    x1, y1 = z1[0]/r1, z1[1]/r1
    x2, y2 = z2[0]/r2, z2[1]/r2
    z = (-r1**2+r2**2)/(r1**2+r2**2)
    x = np.sqrt(1 - z**2)*(x1*x2 - y1*y2)
    y = np.sqrt(1 - z**2)*(x2*y1 - x1*y2)
    return(x, y, z)


