#####
# translation.py
#
# procedures for various types to represent 3-braids
#
# version info : 1.2.4  2022/10/24  H.Ogawa
#

from . import globalParamaters as glbl
from . import subprocedures as sub

import math
import cmath
import numpy as np



def referBraidStyle(self, brType):
    wk = brType.lower()
    if 'can' in wk: # linear cancellation
        return(self.LinearCancellation)
    elif wk.startswith('b'): # braid
        return(self.braid)
    elif wk.startswith('c'): # circular word
        return(self.circular)
    elif wk.startswith('lo'): # loop / loop3 / loop6
        if wk.endswith('3'): # loop3
            return(self.loop3)
        elif wk.endswith('6'): # loop6
            return(self.loop6)
        else:                     # loop
            return(self.loop)
    elif wk.startswith('r'): # raw data
        return(self.RawData)
    elif wk.startswith('lis'): # Lissajous
        return(self.Lissajous)
    elif wk.startswith('lin'): # linear
        return(self.Linear)
    else:
        return({})


# PSL2 -> ST
def PSL2_ST(a, **kwargs):
    if sub.det2(a) != 1:
        print('error: {} is not in PSL2.'.format(a))
        return('')
    wk = a
    ans = ''
    while wk[1][0] != 0 or wk[0][1] != 0:
        if abs(wk[0][0]) < abs(wk[1][0]):
            wk = sub.matProd2(glbl.dictPSL2['S'], wk)
            ans += 'S'
        if wk[1][0] != 0:
            q = wk[0][0]//wk[1][0]
        else:
            q = wk[0][1]//wk[0][0]
        if q != 0:
            wk = sub.matProd2(sub.matPow(glbl.dictPSL2['T'], -q), wk)
            ans += 'T'*q if q>0 else 't'*(-q)
    return(ans)


# toPSL2
def toPSL2(st, **kwargs):
    ans = [[1,0],[0,1]]
    if ' ' in st:
        for x in st.split(' '):
            ans = sub.matProd2(ans, glbl.dictPSL2[x])
    else:
        for x in st:
            if len(x) == 1:
                ans = sub.matProd2(ans, glbl.dictPSL2[x])
            else:
                ans = sub.matProd2(ans, sub.matPow(glbl.dictPSL2[x[0]], x[1]))
    return(ans)


# toAB
def toAB(st, **kwargs):
    brtype = sub.braidReprType(st)
    if 'comb' in brtype:
        return(toAB(sub.toSimple(st), **kwargs))
    elif brtype == 'AB':
        ans = st
    elif brtype == 'PSL2':
        ans = toAB(PSL2_ST(st, **kwargs), **kwargs)
    elif brtype == 'Syzygy':
        ans = Syzygy_AB(st, **kwargs)
    elif brtype == 'Sigma':
        ans = ''.join([toABsub(x) for x in st.split()])
    else:
        ans = ''.join([toABsub(x) for x in st])
    return(ans)


def toABsub(x):
    if len(x) == 1 or type(x) == str:
        return(glbl.dictAB[x])
    elif x[1] > 0:
        return(glbl.dictAB[x[0]]*x[1])
    else:
        if x[0] == 'T':
            return(glbl.dictAB['t']*(-x[1]))
        elif x[0] == 't':
            return(glbl.dictAB['T']*(-x[1]))
        else:
            return('?')


# startArea / endArea
def ABex_startArea(wd, startArea='0+', **kwargs):
    p = wd.find('A')
    return(startArea if p < 0 else str((int(wd[p+1]) - p*glbl.dictSgn[wd[p+2]])%3)+glbl.dictSgnMinus[wd[p+2]])

def ABex_endArea(wd, startArea='0+', **kwargs):
    q = int(startArea[0])
    if 'A' not in wd:
        return(str((int(startArea[0]) + len(wd)*glbl.dictSgn[startArea[-1]])%3)+startArea[-1])
    else:
        p = list(reversed(wd)).index('A')
        if p > 1:
            return(str((int(wd[-p]) - (p-2)*glbl.dictSgn[wd[1-p]])%3)+wd[1-p])
        else:
            return(str((int(startArea[0]) + (len(wd)-1)*glbl.dictSgn[startArea[-1]])%3)+glbl.dictSgnMinus[startArea[-1]])

def reversedABex(abex):
    return(''.join(list(reversed(abex.replace('A', ' A').replace('B', ' B').split()))))

def SyzygyEx_startArea(szex, startArea='0+', **kwargs):
    if len(szex) < 2:
        return(startArea)
    else:
        return(szex[:2])

def SyzygyEx_endArea(szex, startArea='0+', **kwargs):
    if len(szex) < 2:
        return(startArea)
    else:
        return(szex[-2:])


#
#  AB / ABex / Syzygy / SyzygyEx

# AB <-> ABex
def AB_ABex(ab, **kwargs):
    c = 2*ab.count('A')
    #kwargs['circular'] = True
    return(ab.replace('A', 'A{}{}').format(*(AB_SyzygyEx(ab, **kwargs)[-c-2:-2])))

def ABex_AB(abex, **kwargs):
    return(abex.replace('0','').replace('1','').replace('2','').replace('+','').replace('-',''))


# AB <-> Syzygy
def AB_Syzygy(ab, **kwargs):
    return(SyzygyEx_Syzygy(AB_SyzygyEx(ab, **kwargs), **kwargs))

def Syzygy_AB(sz, circular=False, startArea='0+', **kwargs):
    if sz == '':
        return('')
    sA = (startArea*2)[:2] if type(startArea) == str and len(startArea)>0 else '0+'
    if sA[-1] not in '+-': sA = sA[0]+'+'
    if sA[0] not in '012': sA = str(sz[0])+sA[-1]
    e = e0 = glbl.dictSgn[sA[-1]]
    p = p0 = int(sA[0])
    ans = []
    for x in sz + (sz[0] if circular else ''):
        ans += ['B'*((e*(int(p)-int(x)))%3)]
        p = x
        e = -e
    return('A'.join(ans))


# AB <-> SyzygyEx
def AB_SyzygyEx(aborg, startArea='0+', circular=False, debug=False, **kwargs):
    if debug: print(startArea, circular, kwargs)
    ab = aborg
    sA = startArea[:2] if len(startArea) > 1 else ('0'+startArea if len(startArea) == 1 else '0+')
    if sA[0] not in '012': sA = '0'+sA[-1]
    if sA[-1] not in '+-': sA = sA[0]+'+'
    s = glbl.dictSgn[sA[-1]]
    p = int(sA[0])
    if 'A' not in ab:
        return(sA+str((p - s*len(ab))%3)+sA[-1])
    if circular:
        while ab[0] == 'B':
            p -= s
            ab = ab[1:]+ab[0]
        syzygySeq = ''
    else:
        syzygySeq = sA
    p0 = p
    for c in ab:
        x = str(p%3)+glbl.dictSgn[s]
        if c == 'A':
            p = p%3
            s = -s
            syzygySeq += str(p)+glbl.dictSgn[s]
        elif c == 'B':
            p -= s
        if debug: print(c, ':', x, '->', str(p%3)+glbl.dictSgn[s], ':', syzygySeq)
    return(syzygySeq + str(p%3)+glbl.dictSgn[s])

def SyzygyEx_AB(abex, **kwargs):
    kwargs['startArea'] = abex[:2]
    return(Syzygy_AB(SyzygyEx_Syzygy(abex, **kwargs), **kwargs))


# ABex <-> Syzygy
def ABex_Syzygy(abex, circular=False, **kwargs):
    return(abex.replace('A', '').replace('B', '').replace('+', '').replace('-', '')+('' if circular else ABex_endArea(abex, **kwargs)[0]))

def Syzygy_ABex(sz, **kwargs):
    return(AB_ABex(Syzygy_AB(sz, **kwargs), **kwargs))


# ABex <-> SyzygyEx
def ABex_SyzygyEx(abex, **kwargs):
    return(ABex_startArea(abex, **kwargs)+abex.replace('A', '').replace('B', '')+ABex_endArea(abex, **kwargs))

def SyzygyEx_ABex(szex, **kwargs):
    return(AB_ABex(SyzygyEx_AB(szex, **kwargs), **kwargs))


# Syzygy <-> SyzygyEx
def Syzygy_SyzygyEx(sz, startArea='0+', circular=False, **kwargs):
    sA = (startArea*2)[:2] if type(startArea) == str and len(startArea)>0 else '+'
    if sA[-1] not in '+-': sA = sA[0]+'+'
    if sA[0] not in '012': sA = (sz[0] if len(sz) > 0 else '0')+sA[-1]
    if len(sz) == 0:
        return(sA)
    if circular:
        wk = sz
        s = (('' if sA[-1] == '-' else '-')+('+-'*(len(wk)//2+1)))[:len(wk)]
    else:
        wk = sA[0]+sz
        s = (('' if sA[-1] == '+' else '-')+('+-'*(len(wk)//2+1)))[:len(wk)-1]
        s += s[-1]
    return(('{}'.join(list(wk))+'{}').format(*s))

def SyzygyEx_Syzygy(wd, circular=False, **kwargs):
    return((wd.replace('+', '').replace('-', ''))[0 if circular else 1:])


# toST
def toST(wd, **kwargs):
    brtype = sub.braidReprType(wd)
    if 'comb' in brtype:
        return(toST(sub.toSimple(wd), **kwargs))
    elif brtype == 'PSL2':
        return(toST(PSL2_ST(wd), **kwargs))
    else:
        return(reduceST(''.join([glbl.dictST[x] for x in wd]), **kwargs))


# toRL
def toRL(wd, **kwargs):
    return(''.join([glbl.dictRL[x] for x in toSigma(wd, **kwargs).split()]))


# toBDPQ / toFrieze
def toBDPQ(wd, **kwargs):
    return(reduceBDPQ(AB_bdpq(toAB(wd, **kwargs)), **kwargs))

toFrieze = toBDPQ

def AB_bdpq(ab, **kwargs):
    wk = ab
    ans = ''
    while len(wk)>0:
        if wk=='B':
            ans += 'p'
            wk = ''
        elif wk=='BB':
            ans += 'b'
            wk = ''
        elif wk=='A':
            ans += 'A'
            wk = ''
        elif wk[:2]=='AA':
            wk = wk[2:]
        elif wk[:3]=='ABB':
            ans += 'd'
            wk = 'A'+wk[3:]
        elif wk[:2]=='AB':
            ans += 'q'
            wk = 'A'+wk[2:]
        elif wk[:2]=='BB':
            ans += 'b'
            wk = wk[2:]
        else:
            ans += 'p'
            wk = wk[1:]
    return(ans)

def ep_bdpq_1(ep, m, ell):
    trdict = glbl.dictEPbdpq[('+' if m>0 else '-')+('+' if ell>0 else '-')]
    return(''.join(trdict[x] for x in ep))

def ep_bdpq_2(ep, m, ell):
    trdict = glbl.dictEPbdpq[('++' if m>0 else '-+')]
    return(''.join(trdict[x] for x in ep))

def reduceBDPQ(bdpq, circular=True, **kwargs):
    wk = bdpq
    wk0 = ''
    while wk0!=wk:
        wk0 = wk
        wk = wk.replace('bp', '').replace('pb', '').replace('dq', '').replace('qd', '')
        wk = wk.replace('bb', 'p').replace('pp', 'b').replace('dd', 'q').replace('qq', 'd')
        if circular:
            if wk[0]+wk[-1] in ('bp', 'pb', 'dq', 'qd'):
                wk = wk[1:-1]
            elif len(wk)>1 and wk[0]==wk[-1]:
                if wk[0]=='b':
                    wk = 'p'+wk[1:-1]
                elif wk[0]=='p':
                    wk = 'b'+wk[1:-1]
                elif wk[0]=='d':
                    wk = 'q'+wk[1:-1]
                else:
                    wk = 'd'+wk[1:-1]
    return(wk)

reduceFrieze = reduceBDPQ

def reduceEpsilon(ep, circular=True):
    wk0 = ''
    wk = ep
    while wk0!=wk:
        wk0 = wk
        wk = wk.replace('---', '').replace('+++', '')
        if circular and len(wk)>2:
            if wk[0]==wk[1]==wk[-1]: wk = wk[2:-1]
            if wk[0]==wk[-2]==wk[-1]: wk = wk[1:-2]
    return(wk)


# toSyzygy
def toSyzygy(w, **kwargs):
    return(AB_Syzygy(toAB(w, **kwargs), **kwargs))

def toSyzygyEx(w, **kwargs):
    return(AB_SyzygyEx(toAB(w, **kwargs), **kwargs))

def Sigma_Syzygy(si, startArea='0+', debug=False, **kwargs):
    a,b,c = ['102', '012', '021'][int(startArea[0])]
    siL = si.split()
    sz = startArea[0]
    if len(siL) == 0:
        return(sz, startArea)
    dr = 1 if int(siL[0])%3==1 else -1
    sA = startArea[0]+glbl.dictSgn[dr]
    if debug: print('SS  : {}{}{} : {} : {}'.format(a,b,c,glbl.dictSgn[dr],sz))
    for s in si.split():
        if (int(s)-dr)%3 != 0:
            sz += sz[-1]
            dr = -dr
        if abs(int(s)) == 1:
            a, b = b, a
        else:
            c, b = b, c
        sz += b
        if debug: print(' {} : {}{}{} : {} : {}'.format((' '+s)[-2:],a,b,c,glbl.dictSgn[dr],sz))
    if (int(siL[0])-dr)%3 != 0:
        sz += sz[-1]
        dr = -dr
    return(sz, sA)

def loopOnSSQ(wd, startArea='0+'):
    return(startArea == motionOnSS(wd, startArea=startArea))

def motionOnSS(wd, startArea='0+'):
    wdAB = toAB(wd, circular=False, startArea=startArea)
    s = 1 if startArea[-1] == '+' else -1
    p = int(startArea[0]) if len(startArea) > 2 else 0
    for c in wdAB:
        if c == 'A':
            s = -s
        elif c == 'B':
            p -= s
    return(str(p%3) + ('+' if s > 0 else '-'))

def toS3(wd, **kwargs):
    sA = ABex_startArea(wd, **kwargs)
    eA = ABex_endArea(wd, **kwargs)
    if sA == eA:
        return(1)
    elif sA[1] == eA[1]:
        return(3)
    else:
        return(2)


# toSigma
def toSigmaA(wd, **kwargs):
    ab = reduceAB(toAB(wd, **kwargs), **kwargs)
    if ab[-1] != 'A' and 'A' in ab:
        ab += 'A'
    return(toSigma(ab, **kwargs))

def toSigma(wd, **kwargs):
    return(AB_Sigma(toAB(wd, **kwargs), **kwargs))

def AB_Sigma(ab, **kwargs):
    wk = reduceAB(ab, **kwargs)
    if wk == '':
        return('')
    elif wk == 'A':
        return('1 2 1')
        #return('1 2 1')
    elif wk == 'B':
        return('1 2')
    elif wk == 'BB':
        return('-2 -1')
    else:
        s = wk[0]+wk[-1]
        if s == 'AB':
            return(wk.replace('ABB', '2 ').replace('AB', '-1 '))
        elif s == 'BA':
            return(wk.replace('BBA', '1 ').replace('BA', '-2 '))
        elif s == 'AA':
            if wk[-3] == 'A': #ABA = ABB BBA = 2 1
                return(toSigma(wk[:-3])+'2 1')
            else: #ABBA = AB BA = -1 -2
                return(toSigma(wk[:-4])+'-1 -2')
        elif s == 'BB':
            if wk[1] == 'B':
                return('-2 '+toSigma('A'+wk[1:], **kwargs))
            else:
                return('1 '+toSigma('AB'+wk, **kwargs))
        else:
            return(False)


# 簡約化
def reduce(w, brtype='', normal=False, **kwargs):
    brt = brtype.upper()
    if brt == '':
        brt = sub.braidReprType(w).upper()
    wk = reduceAB(toAB(w, **kwargs), normal=normal, **kwargs)
    if brt == 'AB':
        return(wk)
    elif brt == 'ST':
        return(reduceST(toST(reduceRL(toRL(wk, **kwargs), **kwargs), **kwargs), **kwargs))
    elif brt == 'RL':
        return(reduceRL(toRL(wk, **kwargs), **kwargs))
    elif brt in ('BDPQ', 'FRIEZE'):
        return(toBDPQ(wk, **kwargs))
    elif brtype == 'PSL2':
        return(toPSL2(wk, **kwargs))
    elif brtype == 'SIGMA':
        return(toSigma(wk, **kwargs))
    elif brtype == 'SYZYGY':
        return(reduceSyzygy(w, **kwargs))
    else:
        return(w)


# reduction for words
def reduceAB(w, normal=True, circular=False, **kwargs):
    wk = w
    wk0 = ''
    while wk != wk0:
        wk0 = wk
        wk = wk.replace('AA', '').replace('BBB', '')
        if circular:
            if len(wk) > 1 and wk[0] == wk[-1]:
                if wk[0] == 'A':
                    wk = wk[1:-1]
                elif wk[0] == 'B' and len(wk) > 2:
                    if wk[1] == 'B':
                        wk = wk[2:-1]
                    elif wk[-2] == 'B':
                        wk = wk[1:-2]
            if normal:
                if 'A' in wk:
                    while wk[0] != 'A':
                        wk = wk[-1]+wk[:-1]
                        #wk = wk[1:]+wk[0]
    return(wk)

def reduceABex(w, normal=True, circular=False, **kwargs):
    wk = w.replace('BBB', '')
    wk0 = ''
    while wk0 != wk:
        wk0 = wk.replace('BBB', '')
        wkL = (wk.replace('A', ' A').replace('B', ' B')).split(' ')[1:]
        wk = ''
        while len(wkL)>1:
            s = wkL.pop(0)
            if s == 'B':
                wk += 'B'
            else:
                t = wkL.pop(0)
                if t == 'B':
                    wk += s
                    wkL.insert(0, t)
        if len(wkL) == 1:
            wk += wkL[0]
        wk = wk.replace('BBB', '')
        wkL = (wk.replace('A', ' A').replace('B', ' B')).split(' ')[1:]
        if circular and len(wkL)>1 and wkL[0][0] == wkL[-1][0]:
            if wkL[0][0] == 'A':
                wkL = wkL[1:-1]
            elif len(wkL)>2:
                if wkL[1] == 'B':
                    wkL = wkL[2:-1]
                elif wkL[-2] == 'B':
                    wkL = wkL[1:-2]
        wk = ''.join(wkL)
    if circular and normal and 'A' in wk:
        s = wk.find('A')
        wk = wk[s:] + wk[:s]
    return(wk)

reduceSigma = toSigma

def reduceST(st, circular=False, **kwargs):
    wk = st.replace('t', 'STSTS')
    wk0 = ''
    while wk != wk0:
        wk0 = wk
        wk = wk.replace('TSTST', 'S')
        wk = wk.replace('SS', '')
        if circular:
            if len(wk)>1 and 'S' in wk and 'T' in wk:
                while wk[0] != 'S' or wk[-1] != 'T':
                    wk = wk[1:]+wk[0]
    return(wk)

def reduceSTnormal(w):
    if 'S' in w and 'T' in w:
        while w[0] != 'S' or w[-1] != 'T':
            w = w[1:]+w[0]
    return(w)

def reduceSyzygy(w, circular=False, **kwargs):
    wk = w
    wk0 = ''
    while wk != wk0:
        wk0 = wk
        wk = wk.replace('00', '').replace('11', '').replace('22', '')
        if circular:
            if len(wk)>1 and wk[0] == wk[-1]:
                wk = wk[1:-1]
    return(wk)

def reduceRL(w, circular=False, **kwargs):
    wk = w
    wk0 = ''
    while wk != wk0:
        wk0 = wk
        wk = wk.replace('Rr', '').replace('rR', '').replace('Ll', '').replace('lL', '')
        wk = wk.replace('rLr', 'lRl').replace('RlR', 'LrL')
        wk = wk.replace('Rr', '').replace('rR', '').replace('Ll', '').replace('lL', '')
        wk = wk.replace('lRl', 'rLr').replace('LrL', 'RlR')
        if circular:
            if ('R' in wk or 'r' in wk) and ('L' in wk or 'l' in wk):
                while (wk[0] not in 'Rr') or (wk[-1] not in 'Ll'):
                    wk = wk[1:]+wk[0]
    return(wk)



# transforms on words
def transform(wd, funcAB, **kwargs):
    brtype = sub.braidReprType(wd)
    ans = reduceAB(funcAB(toAB(wd, **kwargs)), **kwargs)
    if brtype == 'Syzygy':
        return(toSyzygy(ans, **kwargs))
    elif brtype == 'Sigma':
        return(toSugma(ans, **kwargs))
    elif brtype in ('BDPQ', 'Frieze'):
        return(toBDPQ(ans, **kwargs))
    elif brtype == 'RL':
        return(toRL(ans, **kwargs))
    elif brtype == 'ST':
        return(toST(ans, **kwargs))
    elif brtype == 'PSL2':
        return(toPSL2(ans))
    else:
        return(ans)

# Involutions on AB
def ABid(wd):
    return(toAB(wd))

def ABinvA(wd):
    return('A'+ABinv(wd)+'A')

def ABrevA(wd):
    return('A'+ABrev(wd)+'A')

def ABmirA(wd):
    return('A'+ABmir(wd)+'A')

def ABinv(wd):
    return(ABmir(ABrev(wd)))

def ABrev(wd):
    return(wd[::-1])

def ABmir(wd):
    return(wd.replace('B', 'b').replace('bb', 'B').replace('b', 'BB'))

def εmir(wd):
    return(wd.replace('0', 'x').replace('1', '0').replace('x', '1'))


# Involutions
def invA(wd):
    return(transform(wd, ABinvA))

def revA(wd):
    return(transform(wd, ABrevA))

def mirA(wd):
    return(transform(wd, ABmirA))

def inv(wd):
    return(transform(wd, ABinv))

def rev(wd):
    return(transform(wd, ABrev))

def mir(wd):
    return(transform(wd, ABmir))

def idf(wd):
    return(wd)


# Palindromic
def palindromicQ(wd):
    if 'A' in wd or sub.braidReprType(wd) not in ('BDPQ', 'Frieze'):
        return(False,'')
    else:
        wk = palindromic(wd)
        return(sub.compaireCircularWords(wd, wk), wk)

def palindromic(wd):
    if 'A' in wd or sub.braidReprType(wd) not in ('BDPQ', 'Frieze'):
        return(False)
    else:
        return(''.join([glbl.dictPDQ[x] for x in ABrev(wd)]))


#####

def toClosureAB(ab, **kwargs):
    return(toClosureSigma(toSigma(ab), **kwargs))


def toClosureSigma(si, **kwargs):
    opts = glbl.margeOptions(kwargs)
    abex = AB_ABex(toAB(si, **opts), **opts)
    m = toS3(abex)
    opts['startArea'] = sA = ABex_startArea(abex, **opts)
    sz, sA = Sigma_Syzygy(' '.join([si]*m), **opts)
    opts['startArea'] = sA
    return(sz, sA, (0, 1/m), m)


def mirrorSZ(word, l=0):
    return(''.join([str((-l-int(s))%3) for s in word]))

def mirrorSI(word):
    return(' '.join([str(-int(s)) for s in word.split()]))


# oddCF and BDPQ word
def CF_r(oddcf):
    return([(x+1)//2 for x in oddcf])

def CF_bdpq(oddcf, m):
    return(r_bdpq(CF_r(oddcf), m))

def r_bdpq(rLst, m):
    wd = ('db', 'pq') if m>0 else ('bd', 'qp')
    return(''.join([(wd[j%2]*r)[:-1] for j,r in enumerate(rLst)]))

def CW_bdpq(cw, m, lvl):
    return(r_bdpq(sub.Phi(cw, lvl), m))

def CF_RL(oddcf):
    c = 1
    return(''.join(['RL'[(c:=1-c)]*x for x in oddcf*2]))

def bdpq_RL(word):
    return(''.join([glbl.dictRLbdpq[word[k:k+2]] for k in list(range(0,len(word),2))]))

def RL_bdpq(word):
    return(''.join([glbl.dictRLbdpq[word[k:k+2]] for k in list(range(0,len(word),2))]))



####################
####################

#  BDPQ words の 比較
def compaireBDPQwords(a: str, b: str, simple=True, **kwargs) -> str:
    if len(a) != len(b):
        return(False, 'defferent lengths')
    ans = compaireBDPQwordsSingle(a, b, simple=simple, **kwargs) + compaireBDPQwordsSingle(a, reverse(b), simple=simple, **kwargs)
    if simple:
        return(orExt(*ans))
    else:
        return(("正順: {} {}\t[回転]{} {}\t[上下]{} {}\t[左右]{} {}\n逆順: {} {}\t[回転]{} {}\t[上下]{} {}\t[左右]{} {}".format(*sum(ans, ()))).replace('True', glbl._ColorForPrint['RED']+'True'+glbl._ColorForPrint['END']).replace('False 0', 'False'))

def compaireBDPQwordsSingle(a, b, **kwargs):
    return(subcompaireCircularWords(a, b, **kwargs), 
           subcompaireCircularWords(a, rotate(b), **kwargs),
           subcompaireCircularWords(a, flipV(b), **kwargs), 
           subcompaireCircularWords(a, flipH(b), **kwargs))

compaireFriezePattern = compaireBDPQwords

def flipV(brBDPQ: str) -> str:
    return(''.join([glbl.dictTrFP['flipV'][c] for c in brBDPQ]))

def flipH(brBDPQ: str) -> str:
    return(''.join([glbl.dictTrFP['flipH'][c] for c in brBDPQ]))

def rotate(brBDPQ: str) -> str:
    return(''.join([glbl.dictTrFP['rotate'][c] for c in brBDPQ]))

def reverse(brBDPQ: str) -> str:
    return(brBDPQ[::-1])

def mirror(brBDPQ: str) -> str:
    return(rotate(reverse(brBDPQ)))


## Christoffel Word
##  Christoffel Word
#def ChristoffelWord(p, q) -> str:
#    return(''.join([str(math.floor((k+1)*q/(p+q))-math.floor(k*q/(p+q))) for k in range(p+q)]))
#
#def cw(pq) -> str:
#    #return(ChristoffelWord(pq[0], pq[1]))
#    return(ChristoffelWord(*pq))
#
#
##  Palindromic Conjugate of Christoffel Word
#def pw(pq) -> str:
#    c = cw(pq)
#    p,q = pq
#    k,l = sub.extGcd(p, q)[1:]
#    j = q-k
#    i = p+l
#    if (i+j)%2 == 1:
#        return(c[(k-l)//2:]+c[:(k-l)//2])
#    else:
#        return(c[k-l+(i+j)//2:]+c[:k-l+(i+j)//2])
#


#  Φ-作用素、φ-作用素　
def phi(s, level) -> str:
    return(''.join(['101'*(level-1+int(c))+'1' for c in s]))

def Phi(s, level) -> list:
    return(tuple([level+int(c) for c in s]))

def PhiOdd(s, level) -> list:
    return(tuple([2*level+2*int(c)-1 for c in s]))

def intΔ(s):
    c = 0
    ans = ['0'] + [str(c) if x=='0' else str((c:=1-c)) for x in s]
    return(''.join(ans[:-1]))

def PhiSZ(s, level, m):
    szc = ('-+' if m>0 else '+-+')+'-+'*level
    r = 2*level-1
    return(''.join([szc[:r+2*int(c)] for c in s]))

def omegaToSZ(s):
    p = -glbl.dictSgn[s[0]]%3
    return('.'.join([''.join([str((p:=(p+glbl.dictSgn[x])%3)+1) for x in s]) for k in range(6)]))

##  Christoffel Word cw(q/p) and Palindromic Conjugate pw(q/p)
#def LissajousBraid_ChristoffelWord(m, ell) -> list:
#    p, q = LissajousBraid_SlopeLevel(m, ell)[0]
#    return(cw(p, q), pw(p, q))


# 描画用 subprocedures
#  orientation
def orientation(abc):
    [ax,ay],[bx,by],[cx,cy] = abc
    d = ax*by+bx*cy+cx*ay -bx*ay-cx*by-ax*cy
    if d>0:
        return(1)
    elif d<0:
        return(-1)
    else:
        return(0)

#  有限 Fourier 係数 (real)
sqrt3 = float(math.sqrt(3))

def psi0(delta):
    (ax,ay), (bx,by), (cx,cy) = delta
    return((ax+bx+cx)/3, 
           (ay+by+cy)/3)

def psi1(delta):
    (ax,ay), (bx,by), (cx,cy) = delta
    return((ax+(-bx+sqrt3*by)/2+(-cx-sqrt3*cy)/2)/3, 
           (ay+(-sqrt3*bx-by)/2+(sqrt3*cx-cy)/2)/3)

def psi2(delta):
    return(psi1((delta[0],delta[2],delta[1])))

def psi3(delta):
    (ax,ay), (bx,by), (cx,cy) = delta
    return((ax+(-bx-sqrt3*by)/2+(-cx+sqrt3*cy)/2)/3, 
           (ay+(sqrt3*bx-by)/2+(-sqrt3*cx-cy)/2)/3)

#  有限 Fourier 係数 (complex)
omega = (-1+cmath.sqrt(-3))/2
omega2 = (-1-cmath.sqrt(-3))/2

def cpsi0(delta):
    a, b, c = delta
    return((a+b+c)/3)

def cpsi1(delta):
    a, b, c = delta
    return((a+omega2*b+omega*c)/3)

def cpsi2(delta):
    return(cpsi1((delta[0],delta[2],delta[1])))

def cpsi3(delta):
    a, b, c = delta
    return((a+omega*b+omega2*c)/3)

def cPsi(delta):
    return(cpsi2(delta)/cpsi1(delta))

def cPsiT(delta):
    return(cpsi2(delta), cpsi1(delta))

#  Lissajous curve (real, c:complex)
def cLissajous(t, lisType):
    return(R2toC(Lissajous(t, lisType)))

def Lissajous(t, lisType, type='old'):
    m,n,d = lisType[0:3]
    if type == 'old':
        return(np.sin(2*np.pi*m*(t+d)), np.sin(2*np.pi*n*t))
    else:
        return(np.sin(2*np.pi*m*t), math.sin(2*np.pi*n*t+d))

def cLissajousDelta(t, lisType):
    return(cLissajous(t-1/3, lisType), 
           cLissajous(t,     lisType), 
           cLissajous(t+1/3, lisType))

def LissajousDelta(t, lisType):
    return(Lissajous(t-1/3, lisType), 
           Lissajous(t,     lisType), 
           Lissajous(t+1/3, lisType))


#  Syzygy 列の持ち上げ
def lift(word, type='inc'):
    if type == 'inc':
        return(lift_inc(word))
    elif type == 'nb':
        return(lift_nb(word))
    elif type == 'red':
        return(redMod3(word))
    else:
        return(word)

#  単調増加
def lift_inc(word):
    s = 0
    ans = []
    for x in word:
        s += (x-s)%3
        ans += [s]
    return(ans)

#  最近値
def lift_nb(word):
    nb = {0:0, 1:1, 2:-1}
    s = word[0]
    ans = [s]
    for x in word[1:]:
        s += nb[(x-s)%3]
        ans += [s]
    return(ans)

#  reduction mod 3
def redMod3(word):
    return([w%3 for w in word])

def strToSyzygy(word):
    if type(word) == str:
        return([int(x) for x in word])
    else:
        return(word)

def linearSyzygy(alpha, beta):
    a = alpha
    if type(alpha) == str:
        a = fractions.Fraction(alpha)
    if not(type(a) in [int, fractions.Fraction]):
        return(False)
    if type(a) == int:
        r = 6
    else:
        r = 2*(a.denominator)
        if r%3 != 0: r *= 3
    return([intMod3(a*j+beta) for j in range(r)])

#  線形 Syzygy 列
def linearSyzygy(alpha, beta):
    a = alpha if type(alpha) != str else Fraction(alpha)
    b = beta if type(beta) != str else Fraction(beta)
    if type(alpha) == str:
        a = Fraction(alpha)
    if not(type(a) in [int, Fraction]):
        return(False)
    if type(a) == int:
        r = 6
    else:
        r = 2*(a.denominator)
        if r%3 != 0: r *= 3
    return([intMod3(a*j+b) for j in range(r)])


# 連分数展開 ２次無理数

# qNum : (a, b, c) : large root of aX^2-bX+c=0  =(b + √b^2-4ac)/2a = ((b, b^2-4ac), a)
#        ((r, d), a) : (r + √d)/2a   = (a, r, (b^2-d)/4a)

def trQNum(qNum):
    if len(qNum)==2:
        if len(qNum[0])==2:
            return(qNum[1], -qNum[0][0], (qNum[0][0]**2-qNum[0][1])//4//qNum[1])
        elif len(qNum[0])==3:
            return(trQNum(((qNum[0][0], qNum[0][1]**2*qNum[0][2]), qNum[1])))
        else:
            return(False)
    elif len(qNum)!=3:
        return(False)
    else:
        return((-qNum[1], qNum[1]**2-4*qNum[0]*qNum[2]), qNum[0])

def continuedFractionExpansion(qNum, cftype='normal', **kwargs):
    if "odd" in cftype:
        return(CF_odd(qNum, **kwargs))
    elif "ne" in cftype:
        return(CF_negative(qNum, **kwargs))
    elif "no" in cftype:
        return(CF_normal(qNum, **kwargs))
    else:
        return(False)

CF = continuedFractionExpansion

def CF_normal(qNum, debug=False):
    a,b,c = qNum if len(qNum)==3 else trQNum(qNum)
    if a<0: a,b,c = -a,-b,-c
    D = b**2-4*a*c
    if D < 1: return(False, "the discriminant {} is not positive.".format(D))
    sqD = math.isqrt(D)
    if D == sqD**2: return(False, "the discriminant {} is a square.".format(D))
    rhoData = []
    cf = []
    if debug: print("[{}, {}, {}]  D={} ({})".format(a,b,c, D, sqD))
    while (a,b,c) not in rhoData:
        rhoData += [(a,b,c)]
        q = math.floor((-b+sqD)/2/a)
        if debug: print("  {} : {}".format(q, (-b+math.sqrt(D))/2/a))
        cf += [q]
        a,b,c = -a*q*q-b*q-c, -2*a*q-b, -a
        if debug:
            print(" q={}  [{}, {}, {}]  D={}".format(q, a,b,c, b**2-4*a*c))
            wk = input()
            if len(wk)>0: return(cf)
    j = rhoData.index((a,b,c))
    return(tuple(cf[:j]),tuple(cf[j:]))

def CF_negative(qNum, debug=False):
    a,b,c = qNum if len(qNum)==3 else trQNum(qNum)
    if a<0: a,b,c = -a,-b,-c
    D = b**2-4*a*c
    if D < 1: return(False, "the discriminant {} is not positive.".format(D))
    sqD = math.isqrt(D)
    if D == sqD**2: return(False, "the discriminant {} is a square.".format(D))
    rhoData = []
    cf = []
    if debug: print("[{}, {}, {}]  D={} ({})".format(a,b,c, D, sqD))
    while (a,b,c) not in rhoData:
        rhoData += [(a,b,c)]
        q = math.floor((-b+sqD)/2/a)+1
        if debug: print("  {} : {}".format(q, (-b+math.sqrt(D))/2/a))
        cf += [q]
        a,b,c = a*q*q+b*q+c, -2*a*q-b, a
        if debug:
            print(" q={}  [{}, {}, {}]  D={}".format(q, a,b,c, b**2-4*a*c))
            wk = input()
            if len(wk)>0: return(cf)
    j = rhoData.index((a,b,c))
    return(tuple(cf[:j]),tuple(cf[j:]))

def CF_odd(qNum, debug=False):
    a,b,c = qNum if len(qNum)==3 else trQNum(qNum)
    if a<0: a,b,c = -a,-b,-c
    D = b**2-4*a*c
    if D < 1: return(False, "the discriminant {} is not positive.".format(D))
    sqD = math.isqrt(D)
    if D == sqD**2: return(False, "the discriminant {} is a square.".format(D))
    rhoData = []
    cf = []
    if debug: print("[{}, {}, {}]  D={} ({})".format(a,b,c, D, sqD))
    while (a,b,c) not in rhoData:
        rhoData += [(a,b,c)]
        q = math.floor((-b+sqD)/2/a)
        if debug: print("  {} : {}".format(q, (-b+math.sqrt(D))/2/a))
        if q%2 == 0:
            q+=1
            e = -1
            a,b,c = a*q*q+b*q+c, -2*a*q-b, a
        else:
            e = 1
            a,b,c = -a*q*q-b*q-c, -2*a*q-b, -a
        cf += [(q, e)]
        if debug:
            print(" q={} ({}) [{}, {}, {}]  D={}".format(q, "- +"[e+1], a,b,c, b**2-4*a*c))
            wk = input()
            if len(wk)>0: return(cf)
    j = rhoData.index((a,b,c))
    return(tuple(cf[:j]),tuple(cf[j:]))


####
##### syzygy
def SZadd(sz, b=1):
    return(''.join([str((int(x)+b)%3) for x in sz]))

def SZminus(sz):
    return(''.join([str(3-int(x)) for x in sz]))

def SZto(sz, b):
    return(SZadd(sz, (b-int(sz[0]))%3))

def SZmTo(sz, b):
    if sz == '':
        return('')
    else:
        b0 = b-int(sz[0])
        return(''.join([str((b0-int(x))%3) for x in sz]))




