# ThreeBraidClass 実例による利用の手引き
#
#
#  version 1.0.0  date : 2022/07/08
#  version 1.0.1  date : 2022/07/14
#        文章の軽微な修正
#        この手引きを実行した結果において、動画生成手続きの出力が表示されないので、動画生成手続きは表記のみで実行しない形に変更（コメントアウトした）
#
#  version 1.1.0  date : 2022/08/31
#        ThreeBraidClass ver.1.1.0 に対応
#        グラフ関係の大幅な変更（ばらばらだった描画ルーチンとユーザインターフェイスを整理統合した）に伴い、記述を改めた。また、動画生成は一旦取りやめた
#
#  version 1.1.1  date : 2022/09/03
#        ThreeBraidClass ver.1.1.1 に対応
#        スライダーなどの付与されたグラフが、ノートブックの保存などの後に、グラフとしては表示さず、スクリプト文になってしまうことから、import 文で実行した場合はグラフを表示しない形に変更した
#
#  version 1.1.2  date : 2022/09/07
#        ThreeBraidClass ver.1.1.2 に対応
#
#  version 1.2.0  date : 2022/09/12
#        ThreeBraidClass ver.1.2.0 に対応
#
#  version 1.2.3  date : 2022/10/04
#        ThreeBraidClass ver.1.2.3 に対応
#
#  version 1.2.4  date : 2022/10/24
#        ThreeBraidClass ver.1.2.4 に対応
#
#   Hiroyuki OGAWA
#
#

if __name__ == "__main__":
    showGraphs = True
else:
    showGraphs = False


#
# print文用 色指定
#
def black(s): return(BLACK+ s+ END)
def red(s): return(RED+ s+ END)
def green(s): return(GREEN+ s+ END)
def yellow(s): return(YELLOW+ s+ END)
def blue(s): return(BLUE+ s+ END)
def magenta(s): return(MAGENTA+ s+ END)
def cyan(s): return(CYAN+ s+ END)
def white(s): return(WHITE+ s+ END)
def reverse(s): return(REVERSE+ s+ END)
def bold(s): return(BOLD+ s+ END)
def underline(s): return(UNDERLINE+ s+ END)
BLACK = '\033[30m'
RED = '\033[31m'
GREEN = '\033[32m'
YELLOW = '\033[33m'
BLUE = '\033[34m'
MAGENTA = '\033[35m'
CYAN = '\033[36m'
WHITE = '\033[37m'
REVERSE = '\033[7m'
BOLD = '\033[1m'
UNDERLINE = '\033[4m'
END = '\033[0m'


def Title(x): return("{}\n\n".format(bold(red(x))))
def Sect(x): return(bold(blue("[{}]".format(x))))
def SubSect(x): return(bold(green("[{}]".format(x))))
def Command(x): return("\t>>> {}\n".format(InlineCmd(x)))
def CommandNext(x): return("\t    {}\n".format(InlineCmd(x)))
def InlineCmd(x): return(bold(x))
def Method(x): return(bold(red(x)))
def Keyword(x): return(bold(blue(x)))
def Function(x): return(bold(magenta(x)))
def Module(x): return(bold(cyan(x)))


#
#
#
print(
    Title("ThreeBraidClass　モジュール 利用の手引き")
    + Sect("概要") + "\n"
    + Module("ThreeBraidClass") + " モジュールは、3-braid の様々な表示形式とそれらの間の変換や、3-braid の視覚化のための python モジュールです。３つのクラスと、3-braid の表示形式の間の変換関数、それらを扱うための関数などが定義されています。３つのクラスは、様々な表示形式を内包し一般的な 3-braid を扱うための　" + Function("ThreeBraid") + " クラス、論文 [Og] をもとに linear 3-braid（Lissajous 3-braid を含む）を扱うための " + Function("LinearBraid") + " クラス、論文 [KNO1]/[KNO2] に基づき Lissajous 3-braid を扱うための " + Function("LissajousBraid") + " クラスです。" 
    + Module("ThreeBraidClass") + " モジュールの利用のために他のモジュールを明示的に組み込む必要はありませんが、python に標準モジュールとして用意されている " + Module("math") + ", " + Module("cmath") + ", " + Module("fractions") + " を組み込むこと（" + InlineCmd("import") + "）を推奨します。また、様々な表現形式で表された 3-braid の視覚化のためには、" + Module("numpy") + ", " + Module("matplotlib") + ", " + Module("ipywidgets") + ", " + Module("IPython") + " モジュールがインストールされ、" + Module("jupyter") + " などのグラフ表示可能な端末上で実行される必要があります。\n\n"
)
#
print(
    Sect("準備")
    + "\npython 標準モジュール " + Module("math") + ", " + Module("cmath") + ", " + Module("fractions") + " と、" + Module("ThreeBraidClass") + " モジュールを組み込んでください。ただし、最初の３つの標準モジュールは組み込まなくても動作します。\n\n"
    + Command("import math\t\t\# 標準的な関数")
    + Command("import cmath\t\# 標準的な複素関数")
    + Command("from fractions import Fraction\t\# 分数環境")
   + "\n" + Module("ThreeBraidClass") + " モジュールを組み込む。\n\n"
    + Command("import ThreeBraidClass as tb\t\# 3-braid 計算環境")
   + "\npython では組み込んだモジュールで定義された関数などを利用する際に、頭にモジュール名をつけます。" + Module("ThreeBraidClass") + " モジュールの名称は少し長いので省略形 " + bold("tb") + " で利用できるように " + InlineCmd("import") + " 文に " + InlineCmd("as tb") + " をつけています。\n"
)
#
import math
import cmath
from fractions import Fraction
#
import ThreeBraidClass as tb
#
#
#################################################
#
# ThreeBraid class
#
#################################################
#
print(
    "\n\n" + Sect("ThreeBraid クラス") + "\n"
    + Module("ThreeBraid") + " クラスは、さまざまな形で表された 3-braid （upto full-twist） について、別の形での表示や、簡約形を得るための計算手続きをまとめたものです。使利用手順は、" + SubSect("インスタンスの生成") + "、" + SubSect("3-braid 入力") + "、" + SubSect("結果出力") + " となります。また、" + SubSect("3-braid の連結（加法演算）") + "や " + SubSect("3-braid の比較") + " もできます。\n\n"
)
#
print(
    SubSect("インスタンスの生成") + "\t"
   + "インスタンスとは、クラス型オブジェクトのこのとです。" + Function("ThreeBraid") + " クラスを利用するために、" + Function("ThreeBraid") + " クラス型オブジェクトを参照する変数、例えば　" + InlineCmd("br") + " を用意します。これをインスタンスの生成と言います。変数の名称は任意です。\n\n"
    + Command("br = tb.ThreeBraid()")
   + "\n" + InlineCmd("tb") + " は " + Module("ThreeBraidClass") + " モジュールの略称で、" + Module("ThreeBraidClass") + " モジュールの中の " + Function("ThreeBraid") + " クラス定義（" + InlineCmd("tb.ThreeBraid()") + " のこと）を呼び出して " + Function("ThreeBraid") + " クラス型オブジェクトを生成し、変数　" + InlineCmd("br") + " に割り当てると言う意味です。後述しますが、" + Function("ThreeBraid") + " クラスに引数を指定することもできます。引数を指定しすることで、" + SubSect("インスタンスの生成") + " と次の " + SubSect("3-braid 入力") + " を同時に行えます。\n\n"
)
#
br = tb.ThreeBraid()
#
print(
    SubSect("3-braid 入力") + "\t"
    + Function("ThreeBraid") + " クラス型オブジェクト変数 " + InlineCmd("br") + " への 3-braid の入力は、メソッド " + Method("set()") + " を使います。クラス型オブジェクト変数へメソッドを適用ためには、変数にメソッドをドット \".\" でつなぎます。つまり、" + InlineCmd("br.set()") + " と記述します。"
    + Method("set()") + " メソッドの引数には、3-braid の表現形式を表す文字列またはリストを与えます。3-braid を特定するための表現形式には " + InlineCmd("AB") + "、" + InlineCmd("Sigma") + "、" + InlineCmd("BDPQ") + "(or " + InlineCmd("Frieze") + ")、" + InlineCmd("Syzygy") + "、" + InlineCmd("RL") + "、" + InlineCmd("ST") + "、" + InlineCmd("PSL2") + " の７つを採用しています。" + InlineCmd("PSL2") + " 型表現形式は行列（長さ２のリストを成分とする長さ２のリスト））で与え、他は文字列 ('' もしくは \"\" で括られた文字の並び) で与えます。入力された文字列がどの表現形式であるか、自動的に判別します。次の例は、" + InlineCmd("AB") + " 型表現形式で与えたものです。入力が正常に終了したとき、" + InlineCmd("The process completed successfully.") + " と表示されます。\n\n"
    + Command("br.set('BBABBBBAAABBABABB')")
   + "\n\n計算終了のメッセージ出力を必要としないとき、" + Method("set()") + " メソッドのキーワード引数 " + Keyword("silinet") + " に真偽値 " + Keyword("True") + " を指定してください。\n\n"
    + Command("br.set('BBABBBBAAABBABABB', silent=True)") 
   + "\n\n" + Function("ThreeBraid") + " クラスで扱う 3-braid は、上端下端のそれぞれが一直線上に並んだものを想定しています。上端下端を並びの順につないで得られる閉じた 3-braid、同じ 3-braid を幾つかつないで shape sphere 上の閉路としたものも、同時に扱えるようにしています。" + Function("ThreeBraid") + " クラスでは、上端下端のあるものを " + Keyword("braid") + "、上端下端をつないだものを " + Keyword("closure") + "、shape sphere の閉路に対応するように伸ばして上端下端をつないだ " + Keyword("loop") + " の３種を " + Method("braid style") + " と呼び、" + Function("ThreeBraid") + " クラスの内部に、同じ名称のインスタンス変数をもち、それぞれの形に応じた７つの表現形式における簡約形などが格納されます。" + Method("set()") + " メソッドでは、これらのことが全て自動的に実行されます。"
    + Keyword("closure") + " は、表現形式を巡回語 (circular word) とみること、つまり、braid 群における共役類を考えたものにあたります。" + Keyword("loop") + "は、与えられた 3-braid を高々３つつなげることで、shape sphere 上の対応する道が閉路になるようにしたもので、表現形式の文字列を高々３つつないが語の属する巡回語で与えられます。3-braid は上端３点から下端３点への置換を引き起こすので、高々３つつなげば、下端の点の配置が上端の点の配置にもどります。3-braid を水平方向の面で切った断面における３点の配置が、頂点の順序を指定した相似三角形の moduli 空間の shape sphere 上の点に対応するので、切る面を上端から下端に連続的に動かして、shape sphere 上の道が定まります。上端と下端の端点の配置を対応する点の順序も含めて同じになるようにしたので、その道は閉路になります。" 
    + Keyword("braid") + " の表す 3-braid は shape sphere 上の道に対応しますが、" + Keyword("loop") + " の表す shape sphere 上の閉路の一部になっています。" + Keyword("braid") + " の上端下端における端点は一直線上にあるとしましたので、" + Module("ThreeBraidClass") + " モジュールでは、Shape sphere 上の道は基本的に、赤道 (朔 syzygy の状態) 上のある点を始点とし、赤道上の (別の) 点を終点と考えます。表現形式によっては、端点の並び順などを指定する必要があります。これは、shape sphere 上の道の始点を指定することに対応します。始点の指定にはキーワード引数 " + Keyword("startArea") + "を使います。"
    + "shape sphere の赤道上の３点 １, ω, ω^2 (ωは１の原始３乗根) は３本の紐のうち２本が交わる (collision) 状態をを意味するので、道 (path) はその３点を通りません。道の始点、終点として、その３点で切り分けられた赤道上の３つの区画 (syzygy の状態) を考えればよい。また、道が赤道上を通り抜ける向き (３点が動き始めた瞬間における３点の配置の向き) も考えれば、" + Keyword("startArea") + " には、区画を表す三つの文字 (0,1,2) に向きを表す符号 (+ ,-) を添えた六種のうちの一つを指定することになります。"
    + Method("set()") + " メソッドで実行後、" + Keyword("braid") + "、" + Keyword("closure") + "、" + Keyword("loop") + " の３種それぞれに対しする７つの表現形式が、自動的に計算され " + Module("ThreeBraid") + " クラス型オブジェクト変数の内部インスタンス変数に格納されます。"
)
#
print(
    "\n\n" + Function("ThreeBraid") + " クラス型オブジェクト変数を生成する際に、" + Method("set()") + " メソッドに受け渡す引数を指定し、" + SubSect("インスタンスの生成") + " と " + SubSect("3-braid の入力") + " を同時に行うことができます。ただし、このとき silent 引数のデフォルト値は True に設定されています。"
    + Command("br = tb.ThreeBraid()") + "\n"
    + Command("br.set('BBABBBBAAABBABABB', silent=True)") + "\n"
   + "と\n"
    + Command("br = tb.ThreeBraid('BBABBBBAAABBABABB')")
   + "\n\nは、同じ意味になります。後述しますが、後者を利用することで、変数に割り当てずに " + Function("ThreeBraid") + " クラスを利用することもできます。"
)
#
print(
    "\n\n" + SubSect("出力") + "\t"
    + Function("ThreeBraid") + " クラス型オブジェクト変数 " + InlineCmd("br") + " に " + Method("set()") + " メソッドで 3-braid を入力すると、" + Keyword("braid") + "、" + Keyword("closure") + "、" + Keyword("loop") + " の３種それぞれに対して、前述の７つの表現形式と dilatation などが計算され、変数 " + InlineCmd("br") + " の内部インスタンス変数 " + InlineCmd("braid") + "、" + InlineCmd("closure") + "、" + InlineCmd("loop") + " に格納されます。 " + InlineCmd("br.braid") + " などとして計算結果を参照することもできますが、モジュール内での計算に使われている情報なども表示されるので、やや冗長なものになります。内部インスタンス変数は、python における辞書型オブジェクトなので、" + InlineCmd("br.braid['AB']") + " のように、必要な情報を直接引き出すこともできます。一般的な用途における計算結果の参照には、" + Method("show()") + " メソッドを利用してください。\n\n"
    + Command("br.show()")
)
#
print(
    "\n\n実際に計算してみます。AB 型表現形式 " + InlineCmd("'BBABBBBAAABBABABB'") + " で与えられた 3-braid を入力し、" + Method("show()") + " メソッドで結果を出力します。\n\n"
    + Command("br.set('BBABBBBAAABBABABB')") + "\n"
)
#
br.set('BBABBBBAAABBABABB')
#
print(
    "\n\n" + Command("br.show()") + "\n"
)
#
br.show()
#
print(
    "\n\nこのように " + Method("show()") + " メソッドで、すべての計算結果が出力されます。" + Keyword("braid") + "、" + Keyword("closure") + "、" + Keyword("loop") + " の３種に対して、７つの表現形式での表示と dilatation が出力されます。、また、" + Keyword("loop") + " が同じ 3-braid ３つに分割できるときは " + Keyword("loop") + " の 1/3 長 (" + Keyword("loop3") + ") についても計算され、出力されます。さらにもう半分 (同一ではなく鏡映に場合も含めて) に分けられるときは 1/6 長 (" + Keyword("loop6") + ") についても計算され、出力されます。" 
        + Method("show()") + " メソッドに引数を与えて、必要なものだけを出力させることができます。例えば " + Keyword("closure") + " に関する情報のみを出力したい場合は、第１引数に " + InlineCmd("'closure'") + " と書く方法と、キーワード引数 (" + Keyword("brStyle") + ") に " + InlineCmd("'closure'") + " を割り当てる方法があります。\n\n"
    + Command("br.show('closure')") + "\n\n"
    + Command("br.show(brStyle='closure')") + "\n"
)
#
br.show(brStyle='closure')
#
print(
    "\n\n" + Method("show()") + " メソッドのキーワード引数 (" + Keyword("brStyle") + ") には、" + InlineCmd("'braid'") + ", " + InlineCmd("'closure'") + ", " + InlineCmd("'loop'") + ", " + InlineCmd("'loop3'") + ", " + InlineCmd("'loop6'") + " と、これらの任意の組 (" + InlineCmd("list []") + ", " + InlineCmd("tuple ()") + ") を指定できます。すべてを指定する代わりに " + InlineCmd("'all'") + " や " + InlineCmd("''") + " を使うことができます。" + Keyword("brStyle") + " 引数の記述を省略した場合は " + Keyword("brStyle") + " に " + InlineCmd("'all'") + " が割り当てられます。"
)
#
print(
    "\n\n特定の表現形式のみを表示させたい場合は、" + Method("show()") + " メソッドの第２引数、あるいはキーワード引数 （" + Keyword("brType") + "） を使います。" + Keyword("brType") + " 引数には、７つの表現形式の他に、内部インスタンス変数の辞書キー (" + Method("keys()") + " で調べられます) を指定できます。" + Keyword("brStyle") + " と同様に組にして複数を指定することもできます。\n\n"
    + Command("br.show(brType=['AB', 'Syzygy'])") + "\n"
)
#
br.show(brType=['AB', 'Syzygy'])
#
print(
    "\n\nこの例では " + Keyword("brStyle") + " 引数が省略されているので、" + Keyword("brStyle='all'") + " が指定されたことになります。"
   + "キーワード引数は、値を割り当てる変数名が書かれているので、任意の順序で引数を記述できます。" + Keyword("brStyle") + " 引数と " + Keyword("brType") + " 引数の順序はどちらが先でもかまいません。引数名 " + Keyword("brStyle") + ", " + Keyword("brType") + " を省略する場合は、" + Keyword("brStyle") + "に割り当てる値を第１引数に、" + Keyword("brType") + " に割り当てる値を第２引数に書かなければなりません。\n\n"
    + Command("br.show(brStyle=['braid', 'closure', 'loop'], brType=['AB', 'Syzygy'])") + "\n"
)
#
br.show(brStyle=['braid', 'closure', 'loop'], brType=['AB', 'Syzygy'])
#
print(
    "\n\n上端下端のある " + Keyword("braid") + " の表示式は始点終点のある文字列で表します。" + Keyword("closure") + "、" + Keyword("loop") + "、" + Keyword("loop3") + "、" + Keyword("loop6") + " の表示式は巡回語 (circular words) として扱われます。" + Keyword("braid") + " は " + Method("set()") + " メソッドで与えられた文字列をそのまま用います。" + Keyword("closure") + " は、与えられた文字列を巡回語として扱います。" + Keyword("loop") + " は、shape sphere 上の閉路に対応するように、与えられた文字列を２つか３つつないだものを巡回語にして扱います。"
   + "braid に対応する shape sphere 上の道を、次のように作ることができます。１、ω、ω^2と北極、南極を結ぶ子午線と赤道で、shape sphere を６つの領域に分割します。６つの領域の任意の一つの中の点を始点とし、braid の文字列（具体的には AB 型表現形式での A や B）に従って、南北や東西の隣の領域に移動する道が braid の対応する道（の属するホモトピー類の代表）となります。その道の始点と終点はそれぞれ6つの領域のいずれかに属します。" + Module("ThreeBraidClass") + " モジュールでは、始点と終点を赤道上の３つの区画のうちのひとつにとりその区画を表す文字 (0,1,2) と、始点が北極側にあるときは " + InlineCmd(" + ") + "、南極側にあるときは " + InlineCmd("-") + "を添えた、６種で表せます。Shape sphere 上の閉路になるためには、始点と終点の属する領域が同じでなければなりません。"
    + Keyword("loop") + " は、同じ 3-braid を何回か繰り返して、始点終点が赤道上の同じ区画の同じ向きになるようにしたものです。例えば、始点が " + InlineCmd("'0+ '") + " で、終点が " + InlineCmd("'1-'") + " の 3-braid を２つつないだとき、２つめの 3-braid は始点が " + InlineCmd("'1-'") + " になりますが、南半球で道の進む方角 (東西方向) は、北半球で進む方向と逆向きになるので、最初の道と赤道に対して対称な道をたどることになり、終点が " + InlineCmd("'0+ '") + " になります。始点終点が赤道上の同じ区画で同じ向きになったので、" + Keyword("loop") + " は、3-braid を２倍に伸ばして、始点終点をつないだ circular word とみたものになります。" 
    + Keyword("loop") + " にするためにくり返した回数は " + Method("show()") + " メソッドの引数 " + Keyword("brType") + " に " + InlineCmd("'multiple'") + " を指定することでわかります。" + InlineCmd("'multiple'") + " は " + Keyword("loop") + " に対してのみ意味があり、" + Keyword("braid") + ", " + Keyword("closure") + ", " + Keyword("loop3") + ", " + Keyword("loop6") + " では常に１です。\n\n"
    + Command("br.show(brType='multiple')") + "\n"
)
#
br.show(brType='multiple')
#
print(
    "\n\n" + Function("ThreeBraid") + " クラス型オブジェクト生成時に適当な表現形式を与え、続けて " + bold("show()") + " メソッドを適用し、結果を表示させることができます。変数に割り当てずに " + Function("ThreeBraid") + " クラスに組み込まれた計算アルゴリズムを利用できます。\n\n"
    + Command("tb.ThreeBraid('BBABBBBAAABBABABB').show()") + "\n"
)
#
tb.ThreeBraid('BBABBBBAAABBABABB').show()
#
print(
    "\nより電卓的 (?) に、必要な答えだけ引き出すには、\n\n"
    + Command("tb.ThreeBraid('BBABBBBAAABBABABB').show('', 'Syzygy')") + "\n"
)
#
tb.ThreeBraid('BBABBBBAAABBABABB').show('', 'Syzygy')
#
print(
    "\n\n"
    + SubSect("使用例") + "\t" + Function("ThreeBraid") + " クラス型オブジェクト変数に再度 " + Method("set()") + " メソッドを呼び出すことで、別の 3-braid を入力できます。新たに ThreeBraid クラス型オブジェクトを生成し別の変数に割り当てることもできます。上と同じ " + InlineCmd("br") + " に、Sigma 表現形式で　3-braid を与えてみます。"
   + "Sigma 表現形式は σ1、σ2 のべき乗の積の形で与えるのですが、その形の代用として、σ1 を " + InlineCmd("'1'") + " に、σ2 を " + InlineCmd("'2'") + " に、σ1^(-1) を " + InlineCmd("'-1'") + " に、σ2^(-1) を " + InlineCmd("'-2'") + " に置き換え、空白で区切り、冪指数の分だけ並べた文字列で表します。"
   + "例えば、σ1^(-3) σ2^2 σ1^(-1) σ2^(-2) σ1 は、" + InlineCmd("'-1 -1 -1 2 2 -1 -2 1'") + " で与えます。\n\n"
    + Command("br.set('-1 -1 -1 2 2 -1 -2 1')") + "\n"
)
#
br.set(' -1 -1 -1 2 2 -1 -2 1')
#
print(
    "\n" + Command("br.show()") + "\n"
)
#
br.show()
#
print(
    "\n\n上の計算結果を見ると、" + Keyword("braid") + " の表現形式には何らかの文字列が表示されていますが、" + Keyword("loop") + " などでは空の文字列 ('') になっています。その理由を調べるために、計算の最初の素データ (raw data) を表示させてみます。" + Function("ThreeBraid") + " クラスでは、それぞれの " + Keyword("braid style") + " ごとに表現形式 " + Method("braid type") + " を計算していますが、そのための最初のデータが " + InlineCmd("raw") + " に格納されています。\n\n"
    + Command("br.show(brType=['raw', 'AB', 'multiple'])") + "\n"
)
#
br.show(brType=['raw', 'AB', 'multiple'])
#
print(
    "\ncircular word をとった " + Keyword("closure") + " の AB 型表現形式は " + InlineCmd("'BB'") + " です。従って、与えられた 3-braid を３つつなぐことで shape sphere の閉路になります。" + Keyword("loop") + " は、Sigma 表現形式で与えた文字列を３つつないだものが " + InlineCmd('raw') + " データとなり、これを AB 型表現形式に変換すると " + InlineCmd("''") + " (自明な 3-braid) になります。これは " + Keyword("closure") + " の AB 型表現形式を３倍にのばした " + InlineCmd("'BBBBBB' = ''") + " とも矛盾しません。" + Keyword("loop3") + " の有無は、自明になった " + Keyword("loop") + " の AB 型表現形式が３分割できるかどうかで判断され、AB 型表現形式を３分割したものを " + InlineCmd("'raw'") + " データとしています。" + Keyword("loop6") + " は、" + Keyword("loop3") + " の AB 型表現形式をもとにしているので、" + InlineCmd("'raw'") + " データは " + Keyword('loop3') + " の AB 型表現形式の半分になります。以上のことから " + Keyword("loop") + ", " + Keyword("loop3") + ", " + Keyword("loop6") + " のほとんどの項目が " + InlineCmd("''") + " になったのです。"
)
#
print(
    "\n\nFrieze pattern (BDPQ word) は4つの文字 p, b, q, d からなる文字列で、Syzygy 列は3つの文字 0, 1, 2 からなる文字列で与えます。RL 表現形式は R, L と r, l からなる文字列で、小文字 r, l はそれぞれ R と L の inverse を表します。これらの表現形式では、空白などの区切り文字は入れません。それぞれについて、'AB' 型表現形式に変換してみましょう。\n\n"
    + Command("br.set('pqdpdb', silent=True)") + "\n\n"
    + Command("br.show(brType=['raw', 'BDPQ', 'AB'])") + "\n"
)
#
br.set('pqdpdb', silent=True)
br.show(brType=['raw', 'BDPQ', 'AB'])
#
print(
    "\n\n" + Command("br.set('000112220102011', silent=True)") + "\n\n"
    + Command("br.show(brType=['raw', 'Syzygy', 'AB'])") + "\n"
)
#
br.set('000112220102011', silent=True)
br.show(brType=['raw', 'Syzygy', 'AB'])
#
print(
    "\n\n" + Command("br.set('rLLRLrLRlrL', silent=True)") + "\n\n"
    + Command("br.show(brType=['raw', 'RL', 'AB'])") + "\n"
)
#
br.set('rLLRLrLRlrL', silent=True)
br.show(brType=['raw', 'RL', 'AB'])
#
print(
    "\n\n" + Function("ThreeBraid") + " クラスにおける 3-braid の表現形式の間の変換は、多くの場合 AB 型表現形式を経由しています。一旦, AB 型表現形式に変換し、AB 型表現形式で簡約化します。変換規則に則って、簡約化された AB 型表現形式から目的の表現形式に変換します。ただし、Syzygy 列への変換に対してのみ、簡約化する前の AB 型表現形式から変換します。AB 型表現形式の簡約化は、簡約則 AA=BBB=id を適応することです。" + Keyword("closure") + ", " + Keyword("loop") + " など circular word の場合は、共役類の中で最小語数のものとし、文字 A を含む場合は A を先頭語に、文字 B を含む場合は B を末尾語になるように正規化しています。"
)
#
print(
    "\n\nLissajous 3-braid などのように、" + Keyword("loop") + " が同じ形の３つの部分に分けられるときには、自動的に３分割したものを計算します。さらに３分割したものが更にふたつのブロックに分けられる時は、６分割したものを計算します。ただし、後の方は同じ形の２つとは限りません。鏡映などで対応する場合もあります。" + Method("show()") + " メソッドでそれらを表示させるには、引数 " + Keyword("brStyle") + " に " + InlineCmd("'loop3'") + " や " + InlineCmd("'loop6'") + " を指定します。" + Method("show(brStyle='all')") + " などでの " + InlineCmd('Braid Style: loop /3') + " や " + InlineCmd('Braid Style: loop /6') + " がそれにあたります。\n\n"
    + Command("br.set('012012', silent=True) \t# (figure-8)") + "\n\n"
    + Command("br.show(brStyle=['loop', 'loop3'])") + "\n"
)
#
br.set('012012', silent=True)
br.show(brStyle=['loop', 'loop3'])
#
print(
    "\n\n" + Command("br.set('qbqbqbbbb', silent=True) \t# (これも、figure-8)") + "\n\n"
    + Command("br.set(brType=['raw', 'Frieze', 'AB'])") + "\n"
)
#
br.set('qbqbqbbbb', silent=True)
br.show(brType=['raw', 'Frieze', 'AB'])
#
print(
    "\n\nPSL2 型表現形式で与えられた場合、基本変形により　S、T （PSL2(Z)=<S,　T>）で表し、ST 型表現形式から AB 型表現形式に変換し、他の表現形式に変換します。\n\n"
    + Command("br.set([[17,4],[4,1]], silent=True)") + "\n\n"
    + Command("br.show(brType=['raw', 'PSL2', 'AB'])") + "\n"
)
#
br.set([[17,4],[4,1]], silent=True)
br.show(brType=['raw', 'PSL2', 'AB'])
#
print(
    "\n\nPSL2(Z) に属さない２次正方行列が与えられた場合は、error メッセージが返されます。\n\n"
    + Command("br.set([[15,4],[4,1]])") + "\n"
)
#
print(br.set([[15,4],[4,1]]))
#
print(
    "\n\n" + Method("show()") + " メソッドの紹介の最初で、" + Function("ThreeBraid") + " クラス型オブジェクトの内部インスタンス変数を直接呼び出すことで、必要なデータを文字列として抽出することができます。内部インスタンス変数の名称は、" + Keyword("brStyle") + " の指定に対応した " + Keyword("braid") + ", " + Keyword("closure") + ", " + Keyword("loop") + ", " + Keyword("loop3") + ", " + Keyword("loop6") + " です。それらは辞書型オブジェクトで、3-braid の表現形式に対応した " + InlineCmd("'AB'") + ", " + InlineCmd("'Sigma'") + ", " + InlineCmd("BDPQ") + ", " + InlineCmd("'Frieze'") + ", " + InlineCmd("'Syzygy'") + ", " + InlineCmd("'RL'") + ", " + InlineCmd("'ST'") + ", " + InlineCmd("'PSL2'") + " と、" + InlineCmd("'Dilatation'") + " などのキーワードに対する値として収められています。"
   + "内部インスタンス変数における 3-braid の表示はやや可視性に劣ります。" + Function("ThreeBraid") + " クラスの " + Method("show()") + " メソッドなどでは、" + InlineCmd("tb.pprint") + " 関数を表示に利用しています。"
   + "これらを使った例は次章 " + Sect("LinearBraid クラス") + " の中で紹介します。"
)
#
print(
    "\n\n" + SubSect("コピー") + "\t"
    + Method("copy()") + " メソッドを使って、" + Function("ThreeBraid") + " クラス型オブジェクトの複製を生成することができます。python において、変数への代入は、新たなオブジェクトを生成して代入するのではなく、同一オブジェクトへのアクセスポインタの代入なので、一方の変数へ加えられた操作は別の変数にも作用します。計算途中の状態を保持したままで、新たに何らかの操作を加えた結果も参照したい場合など、" + Method("copy()") + " メソッドにより " + Function("ThreeBraid") + " クラス型オブジェクト変数の複製を生成し、新たな変数に割り当てます。この場合、それぞれの変数には異なるオブジェクトが割り当てられ、それらの一方に加えられた変更は他方に影響を与えません。具体的に説明します。\n\n"
    + Command("br1 = tb.ThreeBraid('ABABB')") + "\n"
    + Command("br2 = br1")
   + "\n\n最初のコマンドで、初期値 " + InlineCmd("'ABABB'") + " を与えて " + Function("ThreeBraid") + " クラス型オブジェクトを生成し変数 " + InlineCmd("br1") + " に割り当てています。次のコマンドでは " + InlineCmd("br1") + " 変数と同じオブジェクトを別の変数 " + InlineCmd("br2") + " に割り当てます。" + Method("show()") + " メソッドでそれぞれの変数に代入された 3-braid をみてみます。\n\n"
    + Command("br1.show('braid','AB')") + "\n"
)
#
br1 = tb.ThreeBraid('ABABB')
br2 = br1
br1.show('braid','AB')
#
print(
    "\n" + Command("br2.show('braid','AB')") + "\n"
)
#
br2.show('braid','AB')
#
print(
    "\nどちらもも同じ 3-braid が代入されています。最初の " + InlineCmd("br1") + "に別の 3-braid を代入してみます。\n\n"
    + Command("br1.set('BABA', silent=True)") + "\n\n"
    + Command("br1.show('braid','AB')") + "\n"
)
#
br1.set('BABA', silent=True)
br1.show('braid','AB')
#
print(
    "\nもう一方の変数をみて見ると、\n\n"
    + Command("br2.show('braid','AB')") + "\n"
)
#
br2.show('braid','AB')
#
print(
    "\n" + InlineCmd("br1") + " だけでなく " + InlineCmd("br2") + " にも新たな値が代入されてしまいました。複製を作る " + Method("copy()") + " メソッドを利用してみます。\n\n"
    + Command("br3 = br1.copy()") + "\n\n"
    + Command("br1.show('braid','AB')") + "\n"
)
#
br3 = br1.copy()
br1.show('braid','AB')
#
print(
    "\n" + Command("br3.show('braid','AB')") + "\n"
)
#
br3.show('braid','AB')
#
print(
    "\n" + InlineCmd("br3") + " に " + InlineCmd("br1") + " と同じ 3-braid が代入されています。"
    + Command("br1.set('ABBAB', silent=True)")
   + "\n\n最初の " + InlineCmd("br1") + "  に新たな値 " + InlineCmd("'ABBAB'") + " を与えます。\n\n"
    + Command("br1.show('braid','AB')") + "\n"
)
#
br1.set('ABBAB', silent=True)
br1.show('braid','AB')
#
print(
    "\n" + Command("br3.show('braid','AB')") + "\n"
)
#
br3.show('braid','AB')
#
print(
    "\n" + InlineCmd("br1") + " には新たな値が代入されましたが、" + InlineCmd("br3") + " はもとのままです。"
   + "ところで、変数 " + InlineCmd("br1") + ", " + InlineCmd("br2") + ", " + InlineCmd("br3") + " を呼び出すと、その変数に割り当てられたオブジェクトへのアクセスポインタが返されます。\n\n"
    + Command("br1")
   + "\n\n<ThreeBraid." + ("{}".format(str(br1.__repr__).split(" of ")[-1])) + "\n\n"
    + Command("br2")
   + "\n\n<ThreeBraid." + ("{}".format(str(br2.__repr__).split(" of ")[-1])) + "\n\n"
    + Command("br3")
   + "\n\n<ThreeBraid." + ("{}".format(str(br3.__repr__).split(" of ")[-1]))
   + "\n\n" + InlineCmd("br1") + " と " + InlineCmd("br2") + " が同一オブジェクトを指し、" + InlineCmd("br3") + " は " + InlineCmd("br1") + ", " + InlineCmd("br2") + " とは異なるオブジェクトを指していることがわかります。"
   + "\n\nちなみに、" + InlineCmd("print") + " 文や " + InlineCmd("str") + " 文の中で呼び出すと、クラス名と 3-braid を定義したときの表現形式が返されます。\n\n" 
    + Command("print(br1)") + "\n\n" + ("{}".format(br1))
)
#
print(
    "\n\n" + SubSect("3-braid の連結（加法演算）") + "\t"
   + "いくつかの 3-braid をつなぐ、加法演算もサポートしています。通常の数や式と同じ様に、" + InlineCmd("'+ '") + " 記号でつなぐだけです。予め、" + Function("Threebraid") + " クラス型オブジェクト変数を用意し " + InlineCmd("'+ '") + " でつないぐこともできますが、変数に割り当てずに、引数を与えて生成した " + Function("ThreeBraid") + " クラス型オブジェクトに対して加法を行うこともできます。"
)
#
print(
    "\n\n例えば、AB 型表現形式が " + InlineCmd("'ABBABBABBABBABABABAB'") + " の 3-braid に " + InlineCmd("'BBABABABA'") + " の 3-braid をつないでみます。ぞれぞれを変数に割り当てて、\n\n"
    + Command("br1 = tb.ThreeBraid('ABBABBABBABBABABABAB')")
   + "\n" + Command("br2 = tb.ThreeBraid('BBABABABA')")
   + "\n" + Command("br3 = br1 + br2")
   + "\n" + Command("br3.show()")
   + "\n\nにより、" + InlineCmd("br1") + " に " + InlineCmd("br2") + " をつないだ 3-braid が " + InlineCmd("br3") + " に割り当てられます。ここでの " + Keyword("show()") + " メソッドの出力は省きます。変数に割り当てる必要がなく、計算結果のみを知りたい場合は、次でじゅうｂんです。\n\n"
    + Command("(tb.ThreeBraid('ABBABBABBABBABABABAB') + tb.ThreeBraid('BBABABABA')).show(brType='AB')")
)
#
#(tb.ThreeBraid('ABBABBABBABBABABABAB') + tb.ThreeBraid('BBABABABA')).show(brType='AB')
#
print(
    "\n\n異なる表現形式で与えられた 3-braid に対しても、連結が計算できます。AB 型表現形式にしてから計算しています。\n\n"
)
#
print(
    SubSect("3-braid の比較") + "\t"
   + "3-braid を比較するとき、上端下端のある " + Keyword("braid") + " として比較するか、上端下端を結んだ " + Keyword("closure") + " として比較するかで、意味が変わります。" + Keyword("braid") + " としての比較結果と " + Keyword("closure") + " としての比較結果の組 (tuple) を、3-braid の比較の戻り値としています。比較は、等号 (==)、不等号 (!=)、大小関係 (<, >, <=, >=) からなります。等号、不等号は AB 型表現形式が一致するかどうか、大小関係は AB 型表現形式における包含関係の有無で判定しています。" + Keyword("closure") + " の場合は巡回語として比較しています。"
   + "\n\n\t>>> " + bold("tb.ThreeBraid('ABABB') == tb.ThreeBraid('qb')") + "\n"
)
#
print(tb.ThreeBraid('ABABB') == tb.ThreeBraid('qb'))
#
print(
    "\n\t>>> " + bold("tb.ThreeBraid('ABABB') == tb.ThreeBraid('BBABA')") + "\n"
)
#
print(tb.ThreeBraid('ABABB') == tb.ThreeBraid('BBABA'))
#
print(
    "\n\t>>> " + bold("tb.ThreeBraid('ABABB') != tb.ThreeBraid('BBABA')") + "\n"
)
#
print(tb.ThreeBraid('ABABB') != tb.ThreeBraid('BBABA'))
#
print(
    "\n\t>>> " + bold("tb.ThreeBraid('BABABB') < tb.ThreeBraid('ABABB')") + "\n"
)
#
print(tb.ThreeBraid('BABABB') < tb.ThreeBraid('ABABB'))
#
print(
    "\n\t>>> " + bold("tb.ThreeBraid('BABABB') > tb.ThreeBraid('ABABB')") + "\n"
)
#
print(tb.ThreeBraid('BABABB') > tb.ThreeBraid('ABABB'))
#
#
#
#
#
#################################################
#
# LinearBraid class
#
#################################################
#
print(
    "\n\n\n\n"
    + Sect("LinearBraid クラス") + "\n"
   + "線形な有限長の syzygy 列をもつ 3-braid で shape sphere 上の loop に対応するものを linear 3-braid といいます。Lissajous 3-braid は linear 3-braid で、linear 3-braid は Lissajous 3-braid で表せるか、周期部分の長さを２倍に伸ばすことで Lissajous 3-braid で表せます。" + Function("LinearBraid") + " クラスは、linear 3-braid, Lissajous 3-braid を扱うための環境で、" + Function("ThreeBraid") + " クラスを拡張したクラスです。" + Function("LinearBraid") + " クラスは Syzygy 列を計算の基本にしています。linear 3-braid を特徴付ける linear paramater か、Lissajous 3-braid を特徴付ける Lissajous type から Syzygy 列を生成し、3-braid を定めます。他の表現形式への変換などに " + Function("ThreeBraid") + " クラスを利用します。" + Function("LinearBraid") + " クラスの利用手順も、" + Function("ThreeBraid") + " クラスと同じ３ステップ " + SubSect("インスタンスの生成") + "、" + SubSect("パラメーターの入力") + "、" + SubSect("結果出力") + " です。\n\n"
)
#
print(
    SubSect("インスタンスの生成") + "\t"
    + Function("LinearBraid") + " クラス型オブジェクトを生成し、変数 " + InlineCmd("lb") + " に割り当てます。"
   + "\n\n\t>>> " + bold("lb = tb.LinearBraid()") + "\n"
)
#
lb = tb.LinearBraid()
#
print(
    "\n\n" + SubSect("パラメーターの入力") + "\t"
   + "Linear 3-braid は、linear paramater で特徴付けられます。linear paramater は傾き、切片、長さの３つ組です。傾きは有理数、切片は実数、長さは非負偶数で、傾きと長さの積は３の倍数です。Lissajous 3-braid は、Lissajous type　で特徴付けられます。Lissajous type は２つの振動数と、フェイズの３つ組です。振動数は３と素な整数で、フェイズは実数です。"
   + "" + Function("LinearBraid") + " クラス型オブジェクトへのパラメーターの入力は、" + Method("set()") + " メソッドのキーワード引数 (" + Keyword("linear") + ", " + Keyword("lissajous") + ", " + Keyword("lissajousnew") + ") への割り当てで行います。例えば、linear paramater として傾き１、切片1/2、語長６を指定する場合、\n"
    + Command("lb.set(linear=[1, '1/2', 6])")
   + "\n\nとします。Lissajous type " + InlineCmd("[4, -5, 0]") + " (振動数 4, -5, フェイズパラメータ 0) を指定する場合は、\n"
    + Command("lb.set(lissajous=[4, -5, 0])")
   + "\n\nとします。"
   + "\n\n（注意） " + Function("LinearBraid") + " クラスでは、Lissajous 曲線として"
   + "\n\n\t L(t) = sin(2π m(t + δ)) + i sin(2π n t)\t\t0≦t≦1"
   + "\n\nで与えられる閉曲線をとり、三つ組 " + InlineCmd("[m, n, δ]") + " を Lissajous Type としています。後で具体的に定義式を与えますが、[KNO2] とは Lissajous 曲線のパラメーターが少し変違っています。上記閉曲線では実部に置かれたフェイズパラメータですが、[KNO2] では虚部に移しています。変更後の三つ組で計算する場合のために、" + Method("set()") + " メソッドのキーワード引数 " + Keyword("lissajousnew") + " を用意しています。\n"
    + Command("lb.set(lissajousnew=[4, -5, 0])")
   + "\n\nのように指定します。"
   + "上記 Lissajous 曲線の型 " + InlineCmd("[m, n, δ]") + " に対応する、（変更後の）Lissajous 曲線の型は " + InlineCmd("[-n, m, 2π mδ]") + " で、上記 Lissajous 曲線を π/2 回転したものになっています。"
   + "\n\nLinear 3-braid のlinear paramater において、３つ目の語長が略された場合は、傾きの分母の６倍で補われます。さらに２つ目も略されて、傾きのみが長さが1のリストか有理数で与えられた場合、２つ目の切片には傾きの分母の２倍を分母とする単位分数が補われ、３つ目の語長には傾きの分母の６倍が指定されます。この意味で次はすべて同じパラメーター指定です。\n"
    + Command("lb.set(linear=[1, '1/2', 6])")
    + Command("lb.set(linear=[1, '1/2'])")
    + Command("lb.set(linear=[1])")
    + Command("lb.set(linear=1)")
   + "\n\n切片が指定されなかったときに傾きの分母の２倍を分母とする単位分数を補うのは、collision-free にするためです。Linear paramater における傾き a, 切片 b の Linear braid が collision-free であるための必要十分条件は、直線 y=ax+ b が格子点を通らないことです。"
   + "\n\nところで、すでに使っている表記ですが、引数に有理数を指定するときに、" + InlineCmd("'1/2'") + " のように文字列を使います。プログラム内部で有理数を扱うときに、" + Module("fractions") + " モジュールの " + Function("Frantion") + " クラスを利用しています。有理数 " + InlineCmd("q/p") + " は、" + InlineCmd("Fraction(q, p)") + ", " + InlineCmd("Fraction('q/p')") + " で表されます。" + Module("ThreeBraidClass") + " モジュールでの引数などの入力に分数を使う場合、" + Module("fractions") + " モジュールを組み込んで " + Function("Fraction") + " クラスを使うこともできますが、文字列 " + InlineCmd("'q/p'") + " で受け渡すこともできます。" + Module("ThreeBraidClass") + " モジュールでは、後者の文字列表記を推奨します。"
   + "\n\n" + Function("LinearBraid") + " クラスは " + Function("ThreeBraid") + " クラスを拡張したものなので、" + Function("LinearBraid") + " クラスで " + Function("ThreeBraid") + " クラスを代用することができます。" + Method("set()") + " メソッドで、キーワード引数 (" + bold("linear") + ", " + bold("lissajous") + ", " + bold("lissajousnew") + ") に値を割り当てず、引数を与えて呼び出したとき、" + Function("ThreeBraid") + " クラスを代用していると解釈し、" + Function("ThreeBraid") + " クラスにおける " + Method("set()") + " メソッドが呼び出され、新たに " + Method("set()") + " メソッドで linear 3-braid が代入されるまで " + Function("ThreeBraid") + " クラスとして機能します。"
   + "\n\n" + Function("ThreeBraid") + " クラスと同様に、" + Function("LinearBraid") + " クラスもインスタンス生成の際に引数を指定することで、インスタンス生成と入力を一度に行うことができます。"
)
#
print(
    "\n\n" + SubSect("出力") + "\t"
    + Function("LinearBraid") + " クラス型オブジェクト変数の内容を表示するために、" + Method("show()") + " メソッドを使います。" + Method("show()") + " メソッドは、linear 3-braid としてのパラメータ（linear paramater と Lissajous type）と、パラメータから生成された AB 型表現形式と Syzygy 列、それらの linear cancellation、簡約形を出力します。" + Method("show()") + " メソッドは、" + Function("ThreeBraid") + " クラスの " + Method("shoq()") + " メソッドと同じキーワード引数　( " + Keyword("brStyle") + ", " + Keyword("brType") + ") をもち、それらの少なくとも一つを指定することで、" + Function("LinearBraid") + " クラスに埋め込まれた " + Function("ThreeBraid") + " クラスの　" + Method("show()") + " メソッドが呼び出されます。キーワード引数 ( " + Method("brStyle") + ", " + Method("brType") + ") の指定方法も、表示される内容も、" + Function("ThreeBraid") + " クラスの " + Method("show()") + " メソッドと同じです。\n"
    + Command("lb.set(linear=[1, '1/2', 6], silent=True)")
    + Command("lb.show()")
)
#
lb.set(linear=[1, '1/2', 6], silent=True)
lb.show()
#
print(
    Command("lb.show(brStyle='loop3', brType=['AB', 'Syzygy', 'RL', 'PSL2', 'Dilatation'])")
)
#
lb.show(brStyle='loop3', brType=['AB', 'Syzygy', 'RL', 'PSL2', 'Dilatation'])
#
print(
    "\nlinear paramater として " + InlineCmd("[1, '1/2', 6]") + " ではなく、" + InlineCmd("[1, '1/2', 12]") + " とすると、syzygy 列の語長が２倍になるので、3-braid も２倍になり、PSL2 表示と dilatation は２乗になります。\n"
    + Command("lb.set(linear=[1, '1/2', 12], silent=True)")
    + Command("lb.show(brStyle='loop3', brType=['AB', 'Syzygy', 'RL', 'PSL2', 'Dilatation'])")
)
#
lb.set(linear=[1, '1/2', 12], silent=True)
lb.show(brStyle='loop3', brType=['AB', 'Syzygy', 'RL', 'PSL2', 'Dilatation'])
#
print(
    "\nLinear 3-braid, Lissajous 3-braid は必ず同じ形の３つの部分に分けられるので、" + Method("show()") + " メソッドでは３分割したものを出力しています。" + Function("ThreeBraid") + " クラスにおける " + Keyword("loop3") + " に対応するものです。" + Method("show()") + " メソッドでの出力は、" + Keyword("'Raw data'") + ", " + Keyword("'linear cancellation'") + ", " + Keyword("'reduced sequences'") + " の３項目に分かれています。" + Keyword("'Raw data'") + " には " + Method("set()") + " メソッドで入力された linear paramater や Lissajous type から生成された素データが、" + Keyword("'linear cancellation'") + " には linear cancellation で得られた傾きをもとに生成されたものが、" + Keyword("'reduced sequences'") + " には素データを簡約化したものが表示されます。\n"
    + Command("lb.set(linear=['1/5'], silent=True)")
    + Command("lb.show()")
)
#
lb.set(linear=['1/5'], silent=True)
lb.show()
#
print(
    "\nキーワード引数 (" + Keyword("linear") + ", " + Keyword("lissajous") + ", " + Keyword("lissajousnew") + ") を指定せずに " + Method("set()") + " メソッドを呼び出したとき、" + Function("LinearBraid") + " クラスは " + Function("ThreeBraid") + " クラスを代用していると解釈し、" + Function("ThreeBraid") + " クラスとして扱われます。このとき、出力メソッド " + Method("show()") + " は " + Function("ThreeBraid") + " クラスの " + Method("show()") + " メソッドが使われます。\n"
    + Command("lb.set('ABBABABBABABBAB', silent=True)")
    + Command("lb.show()")
)
#
lb.set('ABBABABBABABBAB', silent=True)
lb.show()
#
print(
    "\n\n" + SubSect("Lissajous 3-braid as linear 3-braid") + "\t"
   + "Lissajous 3-braid は linear 3-braid です。Lissajous 3-braid を " + Function("LinearBraid") + " クラスで計算する場合も、" + Method("set()") + " メソッドで Lissajous type を入力し、" + Method("show()") + " メソッドで出力するだけです。Lissajous type は " + InlineCmd("[整数, 整数, 実数]") + " の３つ組で与えます。\n"
    + Command("lb.set(lissajous=[-1,4,Fraction('-5/16')], silent=True)")
    + Command("lb.show()")
)
#
lb.set(lissajous=[-1,4,Fraction('-5/16')], silent=True)
lb.show()
#
print(
    "\n計算例を少し。\n"
    + Command("lb.set(lissajous=[4,-11,0], silent=True)")
    + Command("lb.show()")
)
#
lb.set(lissajous=[4,-11,0], silent=True)
lb.show()
#
print(
    "\n" + Command("lb.set(lissajous=[4,19,0], silent=True)")
    + Command("lb.show()")
)
#
lb.set(lissajous=[4,19,0], silent=True)
lb.show()
#
print(
    "\n" + Command("lb.set(lissajous=[4,-23,0], silent=True)")
    + Command("lb.show()")
)
#
lb.set(lissajous=[4,-23,0], silent=True)
lb.show()
#
print(
    "\n\nLissajous 3-braid において、振動数を固定し、フェイズパラメータを変化させて、Syzygy 列の移り変わりをみてみましょう。\n"
    + Command("for d in range(10):")
    + CommandNext("    delta = (' '+ str(d) + '/10')[-5:]")
    + CommandNext("    lb.set(lissajous=[4,-5,delta], silent=True)")
    + CommandNext("    print('\t{} [{}] {}'.format(delta, lb.startArea, lb.RawData['Syzygy']))")
)
#
for d in range(10):
    delta = (' '+ str(d) + '/10')[-5:]
    lb.set(lissajous=[4,-5,delta], silent=True)
    print('\t{} [{}] {}'.format(delta, lb.startArea, lb.RawData['Syzygy']))
#
print(
    "\n\nフェイズパラメータをもっと細かく変化させて、Syzygy 列が変わったところのみを表示させてみましょう。\n"
    + Command("wk = ''")
    + Command("for d in range(-5, 410):")
    + CommandNext("    delta = ('  '+ str(d) + '/400')[-7:]")
    + CommandNext("    lb.set(lissajous=[4,-5,delta], silent=True)")
    + CommandNext("    if wk == '': wk = lb.RawData['Syzygy']")
    + CommandNext("    if wk != lb.RawData['Syzygy']:")
    + CommandNext("        print('\t{} [{}] {}:{}'")
    + CommandNext("              .format(")
    + CommandNext("                  delta, lb.startArea, ")
    + CommandNext("                  lb.RawData['Syzygy'], ")
    + CommandNext("                  tb.compCW(lb.RawData['Syzygy'], wk, simple=False)[-1]")
    + CommandNext("              )")
    + CommandNext("             )")
    + CommandNext("        wk = lb.RawData['Syzygy']")
)
#
wk = ''
for d in range(-5, 410):
    delta = ('  '+ str(d) + '/400')[-7:]
    lb.set(lissajous=[4,-5,delta], silent=True)
    if wk == '': wk = lb.RawData['Syzygy']
    if wk != lb.RawData['Syzygy']:
        print('\t{} [{}] {}:{}'
              .format(
                  delta, 
                  lb.startArea, 
                  lb.RawData['Syzygy'], 
                  tb.compCW(lb.RawData['Syzygy'], wk, simple=False)[-1]
              )
             )
        wk = lb.RawData['Syzygy']
#
print(
    "\n\n同じことを AB 型表現形式で見てみます。\n"
    + Command("wk = ''")
    + Command("for d in range(-5, 410):")
    + CommandNext("    delta = ('  '+ str(d) + '/400')[-7:]")
    + CommandNext("    lb.set(lissajous=[4,-5,delta], silent=True)")
    + CommandNext("    if wk == '': wk = lb.RawData['AB']")
    + CommandNext("    if wk != lb.RawData['AB']:")
    + CommandNext("        print(")
    + CommandNext("            '\t{} [{}] {}:{}'")
    + CommandNext("            .format(")
    + CommandNext("                delta, ")
    + CommandNext("                lb.startArea, lb.RawData['AB'], ")
    + CommandNext("                tb.compCW(lb.RawData['AB'], wk, simple=False)[-1]")
    + CommandNext("            )")
    + CommandNext("        )")
    + CommandNext("        wk = lb.RawData['AB']")
)
#
wk = ''
for d in range(-5, 410):
    delta = ('  '+ str(d) + '/400')[-7:]
    lb.set(lissajous=[4,-5,delta], silent=True)
    if wk == '': wk = lb.RawData['AB']
    if wk != lb.RawData['AB']:
        print(
            '\t{} [{}] {}:{}'
            .format(
                delta, 
                lb.startArea, lb.RawData['AB'], 
                tb.compCW(lb.RawData['AB'], wk, simple=False)[-1]
            )
        )
        wk = lb.RawData['AB']
#
print(
    "\n\n" + Function("LinearBraid") + " クラスは Syzygy 列を基盤にしています。syzygy 列が shape sphere 上の道が赤道を通過する様子（３点が一直線上に並ぶ朔の状態）の移り変わりを表すものなので、赤道に対して対称な道も同じ syzygy 列をもちます。赤道に対して対称な道を区別するために、道の始点が北半球にあるか南半球にあるか、もし赤道上であれは北半球に向かっているか南半球に向かっているかを指定します。linear 3-braid は、明示的に始点を指定していない限り、南半球とし、syzygy 列の最初の文字の示す赤道上の区画から北半球に向かうことにしています。Lissajous 3-braid の場合は、パラメータ t=0 での３点の配置に対する shape sphere 上の点を始点としています。"
    + Function("ThreeBraid") + " クラスにおいて、対応する shape sphere 上の道は、syzygy 列の最初の文字の示す赤道の区画を始点とし、sygyzy 列の示す通りに赤道を通過し、syzygy 列の最後の文字の示す赤道の区画を終点とするものを想定しています。始点を指定する " + Keyword("startArea") + " の最初の文字と競合することがありますが、syzygy 列の記述を優先しています。" + Keyword("startArea") + " の符号により、道の向かう方向を定めることで、shape sphere 上の道がただ一つに定まります。その後、shape sphere の６つの領域の通過状況から AB 型表現形式に変換し、AB 型表現形式から他の表現形式に変換しています。AB 型変換形式を syzygy 列に変換する場合は、" + Keyword("startArea") + " で指定した syzygy の状態と向きを始点とし AB 型表現形式に従って shape sphere 上の６つの領域の通過順と、赤道の通過の様子、つまり syzygy 列が定まることになります。この関係では、AB 型表現形式で文字 " + InlineCmd("'A'") + " に対応するところで赤道を横切り、syzygy の状態になりますから、syzygy 列の長さは、AB 型表現形式での文字 " + InlineCmd("'A'") + " の個数に始点の分の１を加えた数になります。Linear 3-braid や Lissajous 3-braid、あるいは " + Function("ThreeBraid") + " クラスにおける " + Keyword("loop") + " の場合は、shape sphere 上の閉路に対応しているため、始点と終点が一致します。従って、この場合の syzygy 列の長さは、AB 型表現形式での文字 " + InlineCmd("'A'") + " の個数に等しくなります。"
)
#
print(
    "\n\n" + SubSect("始点の移動") + "\t" + Function("LinearBraid") + " クラスで扱う linear 3-braid は、線形な Syzygy 列をもつ shape sphere 上の閉路です。閉路の始点を閉路上の任意の点に移動させると、円環状に並んだ文字列としての Syzygy 列の始点が変わります。Syzygy 列の始点の移動には、" + Method("shift()") + " メソッドを使います。" + Method("shift()") + " メソッドは始点を移動させる文字数を表す整数値引数をひとつ持ちます。引数が省略された場合は、１を指定したとみなします。正の整数の場合は前方（文字列表記における右方向）に、負の整数の場合は後方（文字列表記における左方向）に始点を移動させます。\n"
    + Command("lb.set(linear='-4/3', silent=True)")
    + Command("lb.show(brStyle='loop', brType=['Syzygy', 'AB'])")
)
#
lb.set(linear='4/3', silent=True)
lb.show(brStyle='loop', brType=['Syzygy', 'AB'])
#
print(
    Command("lb.shift(3)")
    + Command("lb.show(brStyle='loop', brType=['Syzygy', 'AB'])")
)
#
lb.shift(3)
lb.show(brStyle='loop', brType=['Syzygy', 'AB'])
#
print(
    "\n\n" + SubSect("連結（加法演算）") + "\t加法演算により、" + Function("LinearBraid") + " クラスをつなぐことができます。" + Function("LinearBraid") + " クラスの加法は、始点で切り分けた Syzygy 列を連結し、連結した Syzygy 列によって定まる " + Function("ThreeBraid") + " クラスを戻り値としています。始点をどこに取るかによって、結合されたものが異なります。結合させる位置を指定するために、" + Method("shift()") + " メソッドで予め始点を変更しておく必要があます。また、" + Function("ThreeBraid") + " クラスとの結合（加法）も定義されています。" + Function("LinearBraid") + " クラスの方は始点で切り分けておいて、" + Function("ThreeBraid") + " クラス同士の結合（加法）として AB 型表現形式をつないで与えられる " + Function("ThreeBraid") + " クラスを戻り値としています。"
)
#
print(
    "\n\n" + SubSect("比較") + "\t" + Function("LinearBraid") + " クラスの間の比較演算は、shape sphere 上の閉路として、巡回語の比較で定義されます。" + Function("ThreeBraid") + " クラスは、上端下端のある " + Keyword("braid") + " と、上端下端をつないだ " + Keyword("closure") + " のふたつの立場での比較でした。" + Function("LinearBraid") + " クラスと " + Function("ThreeBraid") + " クラスの比較演算も定義されます。この場合、" + Function("LinearBraid") + " クラスを始点で切り分けて " + Function("ThreeBraid") + " クラスと読み替えて比較します。"
)
#
#
#
#
#
#################################################
#
# LissajousBraid class
#
#################################################
#
print(
    "\n\n\n\n" + SubSect("LissajousBraid class") + "\n"
    + Function("LissajousBraid") + " クラスは、Lissajous 3-braid に関する [KNO1]/[KNO2] で用いられている様々な量を計算するためのクラスです。このクラスにおける Lissajous 曲線は、" + Function("LinearBraid") + " クラスと若干異なる次の形のものとします。"
   + "\n\n\tL(t) = sin(2π m t) + i sin(2π n t + φ)\t\t0≦t≦1"
   + "\n\nLissajous type " + InlineCmd("[m, n, φ]") + "（整数 m, n は３を法として１と合同）は、" + Function("LinearBraid") + " クラスの時と同じ三つ組ですが、フェイズパラメータ φ　（" + Function("LinearBraid") + " クラスのときは δ）の位置が異なっています。" + Function("LinearBraid") + " クラスでの、Lissajous type " + InlineCmd("[n, -m, φ/2πm]") + " のLissajous 曲線を原点中心に -π/2 回転させると、上の Lissajous 曲線に一致します。" + Function("LinearBraid") + " クラスは syzygy 列を基盤としています。" + Function("LissajousBraid") + " クラスでは BDPQ 語 (Frieze pattern) を基盤にしています。[KNO1]/[KNO2] では、main formula が AB 型表現形式で記述されていますが、AB 型表現形式を経由せずに BDPQ 語で記述する手順が与えられ、BDPQ 語を使ってさまざまな話題との関係が描かれています。[KNO1] では、Lissajous type が " + InlineCmd("[m, n, 0]") + " (m≡n≡1 mod 3, m-n≡1 mod 2) のみを扱っています。[KNO2] では、3-braid を形成しうる全ての Lissajous type " + InlineCmd("[m, n, δ]") + "（m, n が共に３で割り切れず、collision-free でない）を扱うことができます。\n"
    + Function("LissajousBraid") + " クラスの使い方も、" + Function("ThreeBraid") + " クラス、" + Function("LinearBraid") + " クラスと同じです。" + Function("LissajousBraid") + " クラス型オブジェクト変数を用意し、" + Method("set()") + " メソッドで Lissajous type を指定し、" + Method("show()") + " メソッドで計算結果などを表示します。" + Method("set()") + " メソッドの引数は Lissajous type " + InlineCmd("[m, n, φ]") + " です。フェイズパラメータ φ を入力する際に、円周率 π を使いたい場合があります。" + Module("math") + " モジュールを " + InlineCmd("import") + " していれば、" + InlineCmd("math.pi") + " で円周率を入力することができます。" + Module("math") + " モジュールを " + InlineCmd("import") + " していなくても、" + InlineCmd("tb.pi") + " により、" + Module("ThreeBraidClass") + " モジュールに組み込まれた円周率を利用できます。\n"
    + Command("lis = tb.LissajousBraid()")
    + Command("lis.set([1,4,1])")
)
#
lis = tb.LissajousBraid()
#
lis.set([1,4,1])
#
print(
    Command("lis.show()")
)
#
lis.show()
#
print(
    "\n\n" + Function("LissajousBraid") + " クラスは、" + Function("LinearBraid") + " クラス型オブジェクトの内部インスタンス変数 " + InlineCmd("'lb'") + " をもちます。" + Method("lis.set()") + " メソッドで " + Function("LissajousBraid") + " クラス型オブジェクト変数に Lissajous Type を定義した時に、" + Function("LinearBraid") + " クラス型オブジェクト変数 " + InlineCmd("lis.lb") + " の " + Method("setAB()") + " メソッド（" + InlineCmd("lis.lb.setAB()") + "）が呼び出され、Linear 3-braid としての Lissajous 3-braid が " + InlineCmd("lis.lb") + " に格納されます。" + Method("set()") + " メソッドで入力された " + Function("LinearBraid") + " クラス型オブジェクトは、Syzygy 列をもとに生成されましたが、" + Function("LinearBraid") + " クラス型オブジェクトの " + Method("setAB()") + " メソッドは、" + Function("LissajousBraid") + " クラス型オブジェクトで計算された AB 型表現形式を使うための特別なメソッドで、" + Function("LinearBraid") + " クラス型オブジェクトへの入力は Syzygy 列をもとにした " + Method("set()") + " メソッドを使ってください。" + InlineCmd("lis.lb.show()") + " により、" + Function("LinearBraid") + " クラス型オブジェクト変数 " + InlineCmd("lis.lb") + " の " + Method("show()") + " メソッドを呼び出すことができます。\n"
    + Command("lis.lb.show()")
)
#
lis.lb.show()
#
print(
    "\n上で " + Function("LinearBraid") + " クラスで扱ったのと同じ例を " + Function("LissajousBraid") + " クラスで計算してみます。\n"
    + Command("lis.set([4,-11,0], silent=True)")
    + Command("lis.show()")
)
#
lis.set([4,-11,0], silent=True)
lis.show()
#
print(
    "\n\n" + Function("LinearBraid") + " クラスと、" + Function("LissajousBraid") + " クラス環境下での " + Function("LinearBraid") + " クラスを、AB 型表現形式と Syzygy 列で比較してみます。\n"
    + Command("lb.set(lissajous=[4,-11,0], silent=True)")
    + Command("lb.show(brStyle=['RawData', 'LinearCancellation', 'loop3'], ")
    + CommandNext("        brType=['AB', 'Syzygy']")
)
#
lb.set(lissajous=[4,-11,0], silent=True)
lb.show(brStyle=['RawData', 'LinearCancellation', 'loop3'], 
        brType=['AB', 'Syzygy'])
#
print(
    "\n" + Command("lis.set([4,-11,0], silent=True)")
    + Command("lis.lb.show(brStyle=['RawData', 'LinearCancellation', 'loop3'], ")
    + CommandNext("            brType=['AB', 'Syzygy'])") + "\n"
)
#
lis.set([4,-11,0], silent=True)
lis.lb.show(brStyle=['RawData', 'LinearCancellation', 'loop3'], 
            brType=['AB', 'Syzygy'])
#
print(
    "\n\n" + Function("LinearBraid") + " クラスでは、linear paramater や Lissajous type から Syzygy 列が計算され、型変換によりその Syzygy 列から AB 型表現形式が生成されます。" + Function("LissajousBraid") + " クラス環境下での " + Function("LinearBraid") + " クラスでは、" + Function("LissajousBraid") + " クラスで計算された AB 型表現形式から Syzygy 列が生成されます。その違いは " + InlineCmd("Raw Data") + " に現れています。" + Function("LinearBraid") + " クラスでは、Syzygy 列は簡約化されておらず、AB 型表現形式では Syzygy 列の文字の重複に対応して文字 " + InlineCmd("'A'") + " の重複が見られます。" + Function("LissajousBraid") + " クラス環境下での " + Function("LinearBraid") + " クラスでは、AB 型表現に文字 " + InlineCmd("'B'") + " の重複が見られますが、計算方法の違いから文字 " + InlineCmd("'A'") + " の重複は見られません。また、Syzygy 列は AB 型表現形式からの型変換過程でほぼ簡約化されています。いずれにせよ、簡約化後は同じです。"
)
#
print(
    "\n" + Command("lis.set([4,19,0], silent=True)")
    + Command("lis.show()")
)
#
lis.set([4,19,0], silent=True)
lis.show()
#
print(
    "\n" + Command("lis.set([4,-23,0], silent=True)")
    + Command("lis.show()")
)
#
lis.set([4,-23,0], silent=True)
lis.show()
#
print(
    "\n" + Command("lis.set([-5,7,11/10*math.pi], silent=True)")
    + Command("lis.show()")
)
#
lis.set([-5,7,11/10*math.pi], silent=True)
lis.show()
#
print(
    "\n" + Command("lis.set([-11,13,23/22*math.pi], silent=True)")
    + Command("lis.show()")
)
#
lis.set([-11,13,23/22*math.pi], silent=True)
lis.show()
#
print(
    "\n" + Command("lis.set([-17,19,35/34*math.pi], silent=True)")
    + Command("lis.show()")
)
#
lis.set([-17,19,35/34*math.pi], silent=True)
lis.show()
#
print(
    "\n" + Command("lis.set([7,-11,17/14*math.pi], silent=True)")
    + Command("lis.show()")
)
#
lis.set([7,-11,15/14*math.pi], silent=True)
lis.show()
#
print(
    "\n" + Command("lis.set([-17,25,41/34*math.pi], silent=True)")
    + Command("lis.show()")
)
#
lis.set([-17,25,41/34*math.pi], silent=True)
lis.show()
#
#
#
#
#
#
print(
    "\n\n" + SubSect("３つのクラスの利用") + "\t" + Function("ThreeBraid") + " クラス、" + Function("LinearBraid") + " クラス、" + Function("LissajousBraid") + " クラスを利用して、表現形式の簡約化、表現形式の間の変換を計算してみます。ただしこれらは、" + Method("tb.tr.") + " 以下に定義された関数を使って計算できます。" + Function("ThreeBraid") + " クラスの " + Method("val()") + " メソッドを使います。AB 型表現形式で与えられた 3-braid (上端下端のあるもの) の Sigma 型表現形式で表すには、" + Method("val()") + " メソッドに引数 " + InlineCmd("'Sigma'") + " を指定します。\n"
    + Command("tb.ThreeBraid('ABBABABBAB').val('Sigma')") + "\n"
    + tb.ThreeBraid('ABBABABBAB').val('Sigma')
    + "\n\n同じことを " + Function("tb.tr.toSigma()") + " 関数でも計算できます。\n"
    + Command("tb.tr.toSigma('ABBABABBAB')") + "\n"
    + str(tb.tr.toSigma('ABBABABBAB'))
)
#
print(
    "\n\n" + Method("val()") + " メソッドは、" + Function("ThreeBraid") + " クラスの内部インスタンス変数 " + InlineCmd("'braid'") + " に格納された値を呼び出しているだけなので、引数には辞書型オブジェクト " + InlineCmd("'braid'") + " の任意のキーを指定できます。実際には７つの表現形式に関する次の７つ " + InlineCmd("'AB'") + ", " + InlineCmd("'Syzygy'") + ", " + InlineCmd("'Sigma'") + ", " + InlineCmd("'BDPQ'") + " (" + InlineCmd("'Frieze'") + "), " + InlineCmd("'RL'") + ", " + InlineCmd("'ST'") + ", " + InlineCmd("'PSL2'") + " を指定してください。"
    + Method("tb.tr.") + " 以下にそれぞれの形式に変換する関数が定義されています。" + Function("tb.tr.toAB") + ", " + Function("tb.tr.toSyzygy") + ", " + Function("tb.tr.toSigma") + ", " + Function("tb,tr,toBDPQ") + " (" + Function("tb.tr.toFrieze") + "), " + Function("tb.tr.toRL") + ", " + Function("tb.tr.toST") + ", " + Function("tb.tr.toPSL2") + " です。ただしこれらの関数では、それぞれの表現形式での簡約化は完了していません。簡約化のためには " + Function("tb.tr.reduce") + " 関数を用意しています。\n"
    + Command("tb.tr.toAB('BBABBBAABBAAABBAABBABAABABB')") + "\n"
    + str(tb.tr.toAB('BBABBBAABBAAABBAABBABAABABB'))
    + Command("tb.tr.reduce(tb.tr.toAB('BBABBBAABBAAABBAABBABAABABB'))") + "\n"
    + str(tb.tr.reduce(tb.tr.toAB('BBABBBAABBAAABBAABBABAABABB')))
    + Command("tb.ThreeBraid('BBABBBAABBAAABBAABBABAABABB').val('AB')") + "\n"
    + tb.ThreeBraid('BBABBBAABBAAABBAABBABAABABB').val('AB')
)
#
print(
    "\n\n" + Method("val()") + " メソッドの第２引数に、" + Method("show()") + " メソッドのキーワード引数 " + Keyword("brStyle") + " で指定できる " + InlineCmd("'braid'") + ", " + Keyword("'closure'") + ", " + InlineCmd("'loop'") + ", " + InlineCmd("loop3") + ", " + InlineCmd("'loop6'") + " を指定し、そのスタイルでの 3-braid の (簡約化された) 表現形式を求めることができます。\n"
    + Command("tb.ThreeBraid('BBABBBAABBAAABBAABBABAABABB').val('AB', 'closure')") + "\n" + tb.ThreeBraid('BBABBBAABBAAABBAABBABAABABB').val('AB', 'closure')
    + "\n\n" + Function("LinearBraid") + " クラス、" + Function("LissajousBraid") + " クラスにも " + Method("val()") + " メソッドが定義されています。linear paramater で定義された linear 3-braid や、Lissajous type で定義された Lissajous 3-braid を７つの表現形式で表すことができます。ただし、shape sphere 上の閉路の３分の１ (loop3) を規定値としています。"
)
#
#
#
#
#
#
# plot
#
if tb.glbl.support_plot_procedures:
    print(
        "\n\n" + Sect("Plot") + "\tグラフ表示のためのライブラリなど (" + InlineCmd("matplotlib") + ", " + InlineCmd("numpy") + ", " + InlineCmd("ipywidgets") + ", " + InlineCmd("IPython") + ", " + InlineCmd("mpl_toolkits") + ") と、jupyter notebook などのグラフ表示可能な端末で、グラフ表示に必要な拡張機能がインストールされていれば、３つのクラス (" + Function("ThreeBraid") + ", " + Function("LinearBraid") + ", " + Function("LissajousBraid") + ") で視覚化メソッド " + Method("plot()") + " が利用できます。"
        + Method("plot()") + " メソッドは、３次元空間内における 3-braid の３次元像 (" + InlineCmd("'braid'") + ")、3-braid の断面における３点の動きの平面像 (" + InlineCmd("'orbit'") + ", " + InlineCmd("'curve'") + ") (3-braid の３次元像を端点側から見たもの)、3-braid に対応する shape sphere 上の道 (" + InlineCmd("'pathSS'") + ", " + InlineCmd("'loopSS'") + ") と平面への投影像 (" + InlineCmd("'path'") + ", " + InlineCmd("'loop'") + ") の４種類です。" + Method("plot()") + " メソッドの第１引数に " + InlineCmd("'braid'") + ", " + InlineCmd("'curve'") + ", " + InlineCmd("'pathSS'") + " (" + InlineCmd("'loopSS'") + "), " + InlineCmd("'path'") + " (" + InlineCmd("'loop'") + ") を指定することで、目的の像が描かれます。"
        + Function("LinearBraid") + " クラスと " + Function("LissajousBraid") + " クラスで扱う 3-braid は、Lissajous 曲線上の３点の動きで生成される 3-braid なので、Lissajous 曲線を使って描くことができます。" + Function("ThreeBraid") + " クラスで扱われる任意の 3-braid には、そのような枠となる曲線が与えられてはいないので、" + Function("LinearBraid") + " クラスや " + Function("LissajousBraid") + " クラスとは別の視覚化手続きを必要とします。最初に、Lissajous 曲線を使った視覚化メソッドの使い方を説明し、その後で " + Function("ThreeBraid") + " クラスの視覚化メソッドの実現手順などについて説明します。"
    )
    #
    #
    print(
        "\n\n" + SubSect("LissajousBraid クラスの視覚化メソッド") + "\t" + Function("LissajousBraid") + " クラスで説明します。" + Function("LinearBraid") + " クラスでも全く同じです。Lissajous type を与えて " + Function("LissajousBraid") + " クラス型オブジェクト変数を用意しておきます。\n\n"
        + Command("lis = tb.LissajousBraid([4,-5, 1], silent=True)")
    )
    #
    lis = tb.LissajousBraid([4,-5, 1], silent=True)
    #
    print(
        "\n\nLissajous 3-braid は、枠となる Lissajous 曲線が具体的に与えられています。平面に描かれる Lissajous 曲線の媒介変数を第３の座標にとることで３次元空間内の曲線になります。媒介変数を 1/3 ずつずらして３本の曲線を描いたものが、Lissajous 3-braid の３次元像です。" + Method("plot('braid')") + " で描かれる３次元像は、媒介変数を上から下に動かしています。上が小さい値で下に行くほどが大きい値になります。\n" 
        + Command("lis.plot('braid')") + "\n"
    )
    #
    if showGraphs:
        lis.plot('braid')
    else:
        lis.plot('braid', snapshot=True)
    #
    print(
        "\n右側に３次元像を動かすためのスライダーがあります。「仰角」のスライダーを動かして上方もしくは下方から眺めることができます。「視線」のすランダーを動かして像を回転させることができます。初期設定では、「仰角」を「０」に、「視線」を「−９０」にしています。"
       + "\n\n3-braid を生成する３点の動きを 3-braid 上に描くことができます。キーワード引数 " + Method("triangles") + " に、真偽値 " + InlineCmd("True") + "を指定してください。デフォルト値は " + InlineCmd("False") + " です。右側の「３動点」のスライダーで動かすことができます。\n"
        + Command("lis.plot('braid', triangles=True)")
    )
    #
    if showGraphs:
        lis.plot('braid', triangles=True)
    else:
        lis.plot('braid', triangles=True, snapshot=True)
    #
    print(
        "右側のスライダーなどが不要な場合は、" + Keyword("snapshot=True") + " と指定してください。キーワード引数 " + Keyword("snapshot") + " に真偽値でなく実数値を指定すると、指定された値における３動点の配置を描きます。\n"
        + Command("lis.plot('braid', snapshot=True)")
    )
    #
    lis.plot('braid', snapshot=True)
    #
    print(
        Command("lis.plot('braid', snapshot=0.05)")
    )
    #
    lis.plot('braid', snapshot=0.05)
    #
    print(
        "初期設定では、" + Function("LissajousBraid") + " クラスの描画像は、媒介変数 t の動く範囲を 0 ~ 1/3 にしています。周期の 1/3 だけを描くようにしています。描画範囲（媒介変数 t の動く範囲）を任意にしていすることができます。キーワード引数 " + Keyword("T") + " に範囲をしていしてください。例えば、" + Keyword("T=(0,1)") + " とすると、一周期分になります。また、" + Keyword("T=(1/3,2/3)") + ", " + Keyword("T=(2/3,1)") + " と指定しすることで、当初描かれた像の残りの 2/3 の部分を 1/3 ごとに描くことができます。また、" + Keyword("T=(-0.05,0.05)") + " と指定することで、t=0 の近くでの様子をみることもできます。\n"
        + Command("lis.plot('braid', T=(0,1))")
    )
    #
    if showGraphs:
        lis.plot('braid', T=(0,1))
    else:
        lis.plot('braid', T=(0,1), snapshot=True)
    #
    print(
        Command("lis.plot('braid', T=(1/3,2/3))")
    )
    #
    if showGraphs:
        lis.plot('braid', T=(1/3,2/3))
    else:
        lis.plot('braid', T=(1/3,2/3), snapshot=True)
    #
    print(
        Command("lis.plot('braid', T=(2/3,1))")
    )
    #
    if showGraphs:
        lis.plot('braid', T=(2/3,1))
    else:
        lis.plot('braid', T=(2/3,1), snapshot=True)
    #
    print(
        Command("lis.plot('braid', T=(-0.05,0.05))")
    )
    #
    if showGraphs:
        lis.plot('braid', T=(-0.05,0.05))
    else:
        lis.plot('braid', T=(-0.05,0.05), snapshot=True)
    #
    print(
        "３動点の動きの平面像（軌道）を描くには、" + Method("plot('orbit')") + " とします。" + Function("LissajousBraid") + " クラスの場合は Lissajous 曲線を描くことになります。媒介変数の範囲によっては、Lissajous 曲線の一部しか描かれない場合もあります。３動点の動きを一つずつ重ねて描くので、後から描いた点が、先に描かれた点の上に描かれ、隠されてしまうことがあります。媒介変数の動く範囲が 1/3 周期分のときは、３点（３色）で Lissajous 曲線が描かれます。1/3 周期より小さいときは一部が描かれず、1/3 周期より大きときは一部が重なり先に描かれた点の軌道が見えなくなります。\n"
        + Command("lis.plot('orbit')")
    )
    #
    if showGraphs:
        lis.plot('orbit')
    else:
        lis.plot('orbit', snapshot=True)
    #
    print(
        Method("plot('orbit')") + " の場合も、３動点を描きたい場合は、" + bold("triangles=True") + " と指定してください。右側のスライダーなどがいらないときはキーワード引数 " + Keyword("snapshot") + " を指定し、媒介変数 t の変動範囲を変更するときはキーワード引数 " + Keyword("T") + " に適当な範囲をしていしてください。これらの使い方は " + Method("plot('braid')") + " と同じです。\n\n"
        + Method("plot('pathSS')") + " で、Lissajous 3-braid に対応する shape sphere (３次元空間内の単位球面) 上の閉路 (Lissajous loop) を描くことができます。"
        + "shape sphere の北極から shape sphere の中心と赤道を通る平面への射影により、shape sphere 上の道を平面に投影することできます。"
        + Method("plot('path')") + " で、Lissajous loop の平面上への射影像を描くことができます。shape sphere 上の北極と南極は、正三角形に対応します。Lissajous 曲線の実部と虚部に異なる重みをつけて、３動点の配置が正三角形にならないようにすれば、shape sphere に描かれた Lissajous loop を平面の有界な範囲に描くことができます。初期値は、(実軸方向の変動範囲)／(虚軸方向の変動範囲)＝３ にしています。\n"
        + Command("lis.plot('pathSS')")
    )
    #
    if showGraphs:
        lis.plot('pathSS')
    else:
        lis.plot('pathSS', snapshot=True)
    #
    print(
        Command("lis.plot('path')")
    )
    #
    if showGraphs:
        lis.plot('path')
    else:
        lis.plot('path', snapshot=True)
    #
    print(
        "これらに対しても、キーワード引数 " + Keyword("triangles") + ", " + Keyword("snapshot") + ", " + Keyword("T") + " を利用できます。ただし shape sphere 上の道において、「３動点」として道の上の点を指すのは少し不自然かもしれません。"
        "\n\n\n" + Function("LissajousBraid") + " クラスと " + Function("LinearBraid") + " クラスの視覚化メソッド (" + Method("plot('braid')") + " , " + Method("plot('orbit')") + ", " + Method("plot('pathSS')") + ", " + Method("plot('path')") + ") は、予め与えられた Lissajous type に対して、Lissajous 3-braid を視覚化するものですが、Lissajous type をインタラクティブに指定して、Lissajous 曲線や Lissajous 3-braid, Lissajous loop を描くために、" + Function("plot_Lissajous") + " 関数を用意しています。図の右側のスライダーを使って、Lissajous type を変更することができます。plot_Lissajous 関数は引数を一つもちます。３次元空間内に Lissajous 3-braid を描くための " + bold("plot_Lissajou('braid')") + "、平面上に Lissajous 曲線を描くための " + bold("plot_Lissajou('orbit')") + "、Lissajous loop を Shape sphere 上に描くための " + bold("plot_Lissajous('pathSS')") + " と " + bold("plot_Lissajous('path')") + ") です。\n"
        + Command("tb.plot_Lissajous('braid')")
    )
    #
    if showGraphs:
        tb.plot_Lissajous('braid')
    else:
        #tb.plot_Lissajous('braid', snapshot=True)
        print("\t\t(図は略します)")
    #
    print(
        Command("tb.plot_Lissajous('orbit')")
    )
    #
    if showGraphs:
        tb.plot_Lissajous('orbit')
    else:
        #tb.plot_Lissajous('orbit', snapshot=True)
        print("\t\t(図は略します)")
    #
    print(
        Command("tb.plot_Lissajous('pathSS')")
    )
    #
    if showGraphs:
        tb.plot_Lissajous('pathSS')
    else:
        #tb.plot_Lissajous('pathSS', snapshot=True)
        print("\t\t(図は略します)")
    #
    print(
        Command("tb.plot_Lissajous('path')")
    )
    #
    if showGraphs:
        tb.plot_Lissajous('path')
    else:
        #tb.plot_Lissajous('path', snapshot=True)
        print("\t\t(図は略します)")
    #
    print(
        "\n\n" + Function("tb.plot_Lissajous") + " 関数に対しても、キーワード引数 " + bold("triangles") + ", " + Keyword("T") + " は機能しますが、" + Keyword("snapshot") + " は使えません。\n"
        "\n\nキーワード引数 " + bold("new") + " により、Lissajous 曲線の定義式が [KNO2] に従うかどうかを指定できます。デフォルト値 " + bold("new=True") + " のとき [KNO2] に従い " + Function("LissajousBraid") + " クラスで使っている Lissajous 曲線の定義式の Lissajous type になります。" + bold("new=False") + " のときは " + Function("LinearBraid") + " クラスで使っている Lissajous 曲線の定義式の Lissajous type になります。\n"
    )
    #
    #
    #
    #
    print(
        "\n\n" + SubSect("一般 3-braid の視覚化") + "\tlinear 3-braid や Lissajous 3-braid は、Lissajous 曲線上を動く３点を使って描くことができます。Lissajous 曲線のような整った枠の与えられていない一般の 3-braid に対する詳細は pdf ファイル（ThreebraidClassModule.pdf）をご覧ください。"
        + "\n\n\n具体的に描いてみます。" + Function("ThreeBraid") + " クラス型オブジェクト変数を用意します。\n"
        + Command("br = tb.ThreeBraid('ABABBABABABBAB')")
        + Command("br.show('braid', ['AB', 'Syzygy', 'Sigma'])") + "\n"
    )
    #
    br = tb.ThreeBraid('ABABBABABABBAB')
    #
    br.show('braid', ['AB', 'Syzygy', 'Sigma'])
    #
    print(
        "\n\nshape sphere 上の道 γ(t) の偏角 Θ(t) は " + Method("plot('Fexp')") + " で描くことができます。syzygy 列により指定されたチェックポイントを通る経路となっていることを確認するためのものです。必要とする場面は多くはないと思います。返り値として、Fourier 係数のリストを持ちます。\n"
        + Command("br.plot('Fexp')")
    )
    #
    if showGraphs:
              print(br.plot('Fexp'))
    else:
              print(br.plot('Fexp', snapshot=True))
    #
    print(
        "\n\n目的の 3-braid の３次元像を描くために必要な shape sphere 上の道は " + Method("plot('pathSS')") + " もしくは " + Method("plot('path')") + " で描くことができます。前者は３次元に置かれた球面上の道で、後者は北極から平面への投影像です。\n"
        + Command("br.plot('pathSS')")
    )
    #
    if showGraphs:
        br.plot("PathSS")
    else:
        br.plot("PathSS", snapshot=True)
    #
    print(
        Command("br.plot('Path')")
    )
    #
    if showGraphs:
        br.plot('Path')
    else:
        br.plot('Path', snapshot=True)
    #
    print(
        "\n\n3-braid の３次元像は " + Method("plot('braid')") + " で描くことができます。"
        + Command("br.plot('braid')")
    )
    #
    if showGraphs:
        br.plot('braid')
    else:
        br.plot('braid', snapshot=True)
    #
    print(
        "\n\n最後に紹介するのは、平面上での３動点の動きの軌跡を描く " + Method("plot('orbit')") + " です。単位正方形内に収まっていた Lissajous 曲線とは異なり、楕円 (長径２、短径１) 内の曲線になります。縦横の表示比率はグラフが描画領域に収まるように自動調整されているため、実際の図形とは異なります。表示比率を揃えたい場合は、引数に " + Keyword("aspect='equal'") + " を指定してください。"
        + "\n\n" + Command("br.plot('orbit')") + "\n\n"
    )
    #
    if showGraphs:
        br.plot('orbit')
    else:
        br.plot('orbit', snapshot=True)
    #
    print(
        "\n\n" + Command("br.plot('orbit', aspect='equal')") + "\n\n"
    )
    #
    if showGraphs:
        br.plot('orbit')
    else:
        br.plot('orbit', snapshot=True)
    #
    print(
        "\n\nここまで描いたものは、ThreeBraid クラスの 'braid' (上端下端のある braid) です。対応する shape sphere 上の道の始点終点を、その点の属する区画内の赤道 (syzygy の状態) まで延長し、3-braid の３次元像は３本の紐の３つの端点が一直線上に並んでいます。'closure' は 'braid' の上端と下端をつないだもので、'loop' は同じ 'braid' を幾つかつないで対応する shape sphere 上の道が閉路となるものでした。'closure' にしろ 'loop' にしろ表現形式を巡回語とみることで、簡約化がすすむことがあります。'braid' としての描画像と 'closure' や 'loop' としての描画像は異なります。上で描いたものは、'braid' です。'closure' や 'loop' として描くには、" + Method("plot()") + " メソッドの第２引数に " + InlineCmd("'braid'") + ", " + InlineCmd("'closure'") + ", " + InlineCmd("'loop'") + " を指定してください。第２引数を省略した場合は、" + InlineCmd("'braid'") + " が補われます。"
        + Keyword("startArea='0+ '") + " で syzygy 列 " + InlineCmd("'012'") + " で与えた 3-braid について、'braid', 'closure', 'loop' での違いをみてみましょう。まずは、" + Function("ThreeBraid") + " クラス型変数を用意します。\n"
        + Command("br = tb.ThreeBraid('012', startArea='0+ ')")
        + Command("br.show(['braid', 'closure', 'loop'], ['AB', 'Syzygy', 'Sigma'])")
    )
    br = tb.ThreeBraid('012', startArea='0+ ')
    #
    br.show(['braid', 'closure', 'loop'], ['AB', 'Syzygy', 'Sigma'])
    #
    print(
        "\n\n３つのスタイル 'braid', 'closure', 'loop' のそれぞれについて、shape sphere 上の道、braid の３次元像、３動点の軌跡を描いてみます。実は、" + Method("plot('all')") + " メソッドで、静止像ではありますが、その３つの図を一度に描くことができます。\n"
        + Command("br.plot('all, 'braid')")
    )
    #
    if showGraphs:
        br.plot('all', 'braid')
    else:
        br.plot('all', 'braid', snapshot=True)
    #
    print(
        Command("br.plot('all', 'closure')")
    )
    #
    if showGraphs:
        br.plot('all', 'closure')
    else:
        br.plot('all', 'closure', snapshot=True)
    #
    print(
        Command("br.plot('all', 'loop')")
    )
    #
    if showGraphs:
        br.plot('all', 'loop')
    else:
        br.plot('all', 'loop', snapshot=True)
    #
    print(
        "もう少し、複雑な例でみてみましょう。\n"
        + Command("br = tb.ThreeBraid('ABBABBA', startArea='0+ ')")
        + Command("br.show(['braid', 'closure', 'loop'], ['AB', 'Syzygy', 'Sigma'])")
    )
    #
    br = tb.ThreeBraid('ABBABBA', startArea='0+ ')
    br.show(['braid', 'closure', 'loop'], ['AB', 'Syzygy', 'Sigma'])
    #
    print(
        Command("br.plot('all, 'braid')")
    )
    #
    if showGraphs:
        br.plot('all', 'braid')
    else:
        br.plot('all', 'braid', snapshot=True)
    #
    print(
        Command("br.plot('all', 'closure')")
    )
    #
    if showGraphs:
        br.plot('all', 'closure')
    else:
        br.plot('all', 'closure', snapshot=True)
    #
    print(
        Command("br.plot('all', 'loop')")
    )
    #
    if showGraphs:
        br.plot('all', 'loop')
    else:
        br.plot('all', 'loop', snapshot=True)
    #
    print(
        Module("ThreeBraidClass") + " モジュールの一般 3-braid の視覚化メソッドに使われる syzygy 列は、" + Function("ThreeBraid") + " クラスで計算された、簡約化された syzygy 列ではありません。" + Method("plot('braid')") + " での３次元像が最も見やすい形になるものを選んでいます。具体的には、簡約化された Sigma 型表現形式に沿った形に描けるように選んでいます。syzygy 列としては簡約化されていない冗長な列になっています。"
    )
    #
    #
    #
    print(
        "\n\n" + SubSect("3-braid の視覚化手続きを使って　Lissajous 3-braid を描く") + "\tLissajous 3-braid は　Lissajous 曲線という枠を使って、視覚化していました。Lissajous 3-braid もまた 3-braid なので、3-braid の視覚化手続きに従って描くことができます。Lissajous 3-braid も Linear 3-braid も対応する shape sphere 上の道は閉路 'loop' になるので、ThreeBraid クラスで 'loop' を指定した描画像になります。" + Function("LissajousBraid") + " クラスと " + Function("LinearBraid") + " クラスの " + Method("plot('braid')") + " メソッド、" + Method("plot('curve')") + " メソッド、" + Method("plot('loopSS')") + " メソッド、" + Method("plot('loop')") + " メソッドのキーワード引数 " + bold("TB") + " に真偽値 True を割り当てることで、それぞれ ThreeBraid クラスの " + Method("plot('braid')") + " メソッド、" + Method("plot('curve')") + " メソッド、" + Method("plot('pathSS')") + " メソッド、" + Method("plot('path')") + " メソッドが使われます。Lissajous 曲線の枠をもとにした、" + Function("LissajousBraid") + " クラスの " + Method("plot('braid')") + " メソッドと、ThreeBraid クラスの " + Method("plot('braid')") + " メソッドで描かれる３次元像を比較してみましょう。"
       + "\n\n\t>>> " + bold("lis = tb.LissajousBraid([4,-5, 1], silent=True)")
        + "\n\t>>> " + bold("lis.plot('braid')") + "\n\n"
        + "\t>>> " + bold("lis.lb.show('loop3', ['AB', 'Syzygy', 'Sigma'])") + "\n\n"
    )
    #
    lis = tb.LissajousBraid([4,-5, 1], silent=True)
    lis.lb.show('loop3', ['AB', 'Syzygy', 'Sigma'])
    #
    if showGraphs:
        lis.plot('braid')
    else:
        lis.plot('braid', snapshot=True)
    #
    print(
        "\n\n\t>>> " + bold("lis.plot('braid', TB=True)") + "\n\n"
    )
    #
    if showGraphs:
        lis.plot('braid', TB=True)
    else:
        lis.plot('braid', TB=True, snapshot=True)
    #
    print(
        "\n\nLissajous 曲線上の３動点の動きを使って描いた Lissajous 3-braid の場合、Lissajous 曲線に従って３動点が全体として回転します。3-braid としては、３動点の絡み方に関心がありますが、３動点全体の回転に目立ってしまっています。"
       + "一方、一般 3-braid の描画手続きでは、描画の第一歩の shape sphere 上の道を homotopy 同値類の中で適切なものを作るために、Syzygy 列を簡約でないやや冗長なものの中から上手く選びました。Syzygy のタイミングで３点の並びが一定の向きに揃うようにできたので、絡み具合を見えやすい図形を描くことができました。実際 3-braid 描画て手続きを利用したものでは、Sigma 表現形式そのままの形で絡まった 3-braid が描かれています。"
        "\n\n両者の違いを見るために、３頂点の軌跡、shape sphere 上の道を比べてみます。" + Method("plot('curve')") + " メソッドでの頂点の軌跡を比べてみます。\n"
        + Command("lis.plot('curve')")
    )
    #
    if showGraphs:
        lis.plot('curve')
    else:
        lis.plot('curve', snapshot=True)
    #
    print(
        "\n\n\t>>> " + bold("lis.plot('curve', TB=True)") + "\n\n"
    )
    #
    if showGraphs:
        lis.plot('curve', TB=True)
    else:
        lis.plot('curve', TB=True, snapshot=True)
    #
    print(
        "\n\nShape sphere 上の閉路についても比べてみます。初期設定では、閉路の 1/3 を表示するようにしています。パラメータ t の変動域を 0≦t≦1/3 としています。全体を表示させたい場合は、キーワード引数 " + bold("T") + " にパラメータ t の変動域を指定してください。例えば、全体像を描くために 0≦t≦1 とする場合は、" + bold("T=(0,1)") + " としてください。あるいは、t=0 の近くのでの振る舞いを調べるために -0.1≦t≦0.1 とするには、" + bold("T=(-0.1,0.1)") + " と指定してください。この指定方法は、視覚化関係のメソッドで共通です。\n\n"
        + Command("lis.plot('loop')") + "\n\n"
    )
    #
    if showGraphs:
        lis.plot('loop')
    else:
        lis.plot('loop', snapshot=True)
    #
    print(
        "\n\n" + Command("lis.plot('loop', T=(0,1))") + "\n\n"
    )
    #
    if showGraphs:
        lis.plot('loop', T=(0,1))
    else:
        lis.plot('loop', T=(0,1), snapshot=True)
    #
    print(
        "\n\n" + Command("lis.plot('loop', TB=True)") + "\n\n"
    )
    #
    if showGraphs:
        lis.plot('loop', TB=True)
    else:
        lis.plot('loop', TB=True, snapshot=True)
    #
    print(
        "\n\n\t>>> " + bold("lis.plot('loop', TB=True, T=(-0.1,0.1)") + "\n\n"
    )
    #
    if showGraphs:
        lis.plot('loop', TB=True, T=(-0.1,0.1))
    else:
        lis.plot('loop', TB=True, T=(-0.1,0.1), snapshot=True)
    #
    print(
        "\n\n\t>>> " + bold("lis.plot('braid')") + "\n\n"
    )
    #
    if showGraphs:
        lis.plot('braid')
    else:
        lis.plot('braid', snapshot=True)
    #
    print(
        "\n\n\t>>> " + bold("lis.plot('braid', T=(1/3,2/3))") + "\n\n"
    )
    #
    if showGraphs:
        lis.plot('braid', T=(1/3,2/3))
    else:
        lis.plot('braid', T=(1/3,2/3), snapshot=True)
    #
    print(
        "\n\n\t>>> " + bold("lis.plot('braid', T=(2/3,1))") + "\n\n"
    )
    #
    if showGraphs:
        lis.plot('braid', T=(2/3,1))
    else:
        lis.plot('braid', T=(2/3,1), snapshot=True)
    #
    print(
        "\n\n\t>>> " + bold("lis.plot('braid', TB=True)") + "\n\n"
    )
    #
    if showGraphs:
        lis.plot('braid', TB=True)
    else:
        lis.plot('braid', TB=True, snapshot=True)
    #
    print(
        "\n\n\t>>> " + bold("lis.plot('braid', TB=True, T=(1/3,2/3))") + "\n\n"
    )
    #
    if showGraphs:
        lis.plot('braid', TB=True, T=(1/3,2/3))
    else:
        lis.plot('braid', TB=True, T=(1/3,2/3), snapshot=True)
    #
    print(
        "\n\n\t>>> " + bold("lis.plot('braid', TB=True, T=(2/3,1))") + "\n\n"
    )
    #
    if showGraphs:
        lis.plot('braid', TB=True, T=(2/3,1))
    else:
        lis.plot('braid', TB=True, T=(2/3,1), snapshot=True)
    #
    #
    print(
        "\n\n" + SubSect("視覚化手続きのみの利用") + "\tThreeBraid クラス型オブジェクト変数を用意することなく、3-braid の視覚化手続きのみを利用するために、" + Method("tb.plot_ThreeBraid 関数") + "を用意しています。使い方は基本的に " + Method("tb.plot_LissajousBraid 関数") + "と同じですが、今のところ、3-braid をインタラクティブに変更するためのインターフェイスは備えられていません。また、3-braid の３つのスタイルのうち、上端下端のある 'braid' の描画のみで、他の 'closure', 'loop' には対応していませんが、描画手続に Fourier 展開を使っているので、shape sphere 上の道が自然に拡張され、パラメータの範囲を適切に延長することで 'closure' や 'loop' に対応する描画像をえることができます。"
        + "ただし１点のみ注意が必要なところがあります。それは、描きたい 3-braid を syzygy 列で与えた場合です。" + Function("ThreeBraid") + " クラスで描画像をつくるためには、まず AB 型表現形式に変換し、Sigma 型表現形式をへて、描画に適切な syzygy を与えています。その過程で適宜簡約化が行われ、与えられたそのままの形ではなく、3-braid として直感的にわかりやすい形に変換されています。" 
        + "ところが syzygy 列で 3-braid を与えるのは、shape sphere 上の道を指定し、その道に対応する 3-braid を考えたい場合です。指定した syzygy 列の指示通りの道を描けばいいのですが、その前後、syzygy 列の定義されていない前後の道をどう考えるのが自然であるかで、２通りの考え方があります。１つ目は syzygy 列により定められた 3-braid をつなぐことで、２つ目は shape sphere 上の道を syzygy 列の指示（の主旨）に従って延長することです。これらは同じではありません。shape sphere 上の道の始点と終点の向きが異なるとき、終点を延長した先が逆向きになります。具体的に説明します。"
        + "syzygy 列 '01'、始点 '0+ ' で与えられた 3-braid で説明します。この 3-braid は AB 型表現形式で 'AB' で表されます。２つつなぐと 'ABAB' で、syzygy 列は '010' になります。syzygy 列 '01' を単純に２つつなぐと '0101' ですが、'01' は '0' が始点で '1' が終点で、この終点で次の列をつながねばなりません。そこで '01' を、偏角が大きくなる方に１つ隣の区画にうつること、と考えれば、始点が '1' の場合は、'12' になります。'01' と '12' を '1' のところでつなぐので、'012' になります。AB 型表現形式で表すと、'ABABB' になります。3-braid の延長は同じ 3-braid をつなぐことの方が自然だと思うので、" + Function("tb.plot_ThreeBraid") + " 関数でもその様にしています。"
       + "ただし、syzygy 列の指示に主眼をおいて延長を考えることもできるように、" + Function("tb.plot_ThreeBraid") + " 関数に " + Keyword("connectSZ") + " を用意しています。syzygy 列で与えられた 3-braid について、" + Keyword("connectSZ=True") + " と指定することで、syzygy 列に従って延長されます。この場合、shape sphere 上の閉路になるために、最大で６倍しなければならない場合があります。Lissajous type が (m,n,*) (m≡n≡1 (3)) の Lissajous 3-braid は、m と n が共に奇数のとき、６分割されます。syzygy 列に限って言えば、syzygy 列の長さが 6*((m-n)/3) で、m,n に依らず平行な６つのブロックに分けられます。3-braid を切り分けるとき、syzygy の状態に切り目を入れるので、syzygy 列を文字列として分けた後で、終点を補う必要があります。syzygy の定める shape sphere 上の道の始点と終点での道の向きは、始点終点のある syzygy 列の文字数が奇数の場合は同じ向きで、偶数の場合は逆向きになります。上記例 '01' は始点終点があり、文字数が偶数なので、逆向きになります。実際、上向き '0+ ' で北半球に向けて出発し、北半球から赤道の区画 '1' に向かって、下向きに進み、終点は下向き'1-' です。この場合、syzygy 列での延長は、AB 列での延長と異なり、もとの 3-braid の鏡映をつなぐことになります。"
       + "m と n の一方が偶数のとき、６分割された syzygy 列に対応する道の始点と終点の向きが異なり、m と n が共に奇数のとき、６分割された syzygy 列に対応する道の始点と終点の向きが一致します。Lissajous 3-braid を６つに分割し、それを「延長」して元の Lissajous 3-braid を描き出すためには、3-braid をつなぐのではなく、syzygy 列をつなぐ方が自然であると言えます。\n"
        + Command("tb.plot_ThreeBraid('01', 'all')")
    )
    #
    if showGraphs:
        tb.plot_ThreeBraid('01', 'all')
    else:
        tb.plot_ThreeBraid('01', 'all', snapshot=True)
    #
    print(
        "パラメータの区間を倍にしてみます。"
        + Command("tb.plot_ThreeBraid('01', 'all', T=(0,1))")
    )
    #
    if showGraphs:
        tb.plot_ThreeBraid('01', 'all', T=(0,1))
    else:
        tb.plot_ThreeBraid('01', 'all', T=(0,1), snapshot=True)
    #
    print(
        "延長した部分だけを描くと。"
        + Command("tb.plot_ThreeBraid('01', 'all', T=(1/2,1))")
    )
    #
    if showGraphs:
        tb.plot_ThreeBraid('01', 'all', T=(1/2,1))
    else:
        tb.plot_ThreeBraid('01', 'all', T=(1/2,1), snapshot=True)
    #
    print(
        "syzygy 列の延長を使って描くと。"
        + Command("tb.plot_ThreeBraid('01', 'all', connectSZ=True)")
    )
    #
    if showGraphs:
        tb.plot_ThreeBraid('01', 'all', connectSZ=True)
    else:
        tb.plot_ThreeBraid('01', 'all', connectSZ=True, snapshot=True)
    #
    print(
        "パラメータの区間を倍に伸ばすと。"
        + Command("tb.plot_ThreeBraid('01', 'all', connectSZ=True, T=(0,2))")
    )
    #
    if showGraphs:
        tb.plot_ThreeBraid('01', 'all', connectSZ=True, T=(0,2))
    else:
        tb.plot_ThreeBraid('01', 'all', connectSZ=True, T=(0,2), snapshot=True)
    #
    print(
        "延長された部分は"
        + Command("tb.plot_ThreeBraid('01', 'all', connectSZ=True, T=(1,2))")
    )
    #
    if showGraphs:
        tb.plot_ThreeBraid('01', 'all', connectSZ=True, T=(1,2))
    else:
        tb.plot_ThreeBraid('01', 'all', connectSZ=True, T=(1,2), snapshot=True)
    #
    print(
        Keyword("connectSZ=True") + " では、'01' が '012' に延長されました。さらにその先は、'012012012...' と延長されることになります。パラメータの範囲を広げていきます。"
        + Command("tb.plot_ThreeBraid('01', 'all', connectSZ=True, T=(0,3))")
    )
    #
    if showGraphs:
        tb.plot_ThreeBraid('01', 'all', connectSZ=True, T=(0,3))
    else:
        tb.plot_ThreeBraid('01', 'all', connectSZ=True, T=(0,3), snapshot=True)
    #
    print(
        Command("tb.plot_ThreeBraid('01', 'all', connectSZ=True, T=(0,4))")
    )
    #
    if showGraphs:
        tb.plot_ThreeBraid('01', 'all', connectSZ=True, T=(0,4))
    else:
        tb.plot_ThreeBraid('01', 'all', connectSZ=True, T=(0,4), snapshot=True)
    #
    print(
        Command("tb.plot_ThreeBraid('01', 'all', connectSZ=True, T=(0,5))")
    )
    #
    if showGraphs:
        tb.plot_ThreeBraid('01', 'all', connectSZ=True, T=(0,5))
    else:
        tb.plot_ThreeBraid('01', 'all', connectSZ=True, T=(0,5), snapshot=True)
    #
    print(
        Command("tb.plot_ThreeBraid('01', 'all', connectSZ=True, T=(0,6))")
    )
    #
    if showGraphs:
        tb.plot_ThreeBraid('01', 'all', connectSZ=True, T=(0,6))
    else:
        tb.plot_ThreeBraid('01', 'all', connectSZ=True, T=(0,6), snapshot=True)
    #
    print(
        "６倍して shape sphere 上の閉路になりました。"
       + "途中３倍したところでも、始点と終点が一致してはいますが、道の向かう向きが反対向きなので、さらに３倍をつないだ６倍長のものの方が適切でしょう。\n\n"
       + "AB 型表現形式など、さまざまな形で与えられた 3-braid に対しても " + Function("tb.plot_ThreeBraid") + " 関数で、3-braid の３次元像 (" + InlineCmd("'braid'") + ")、３動点の軌跡 (" + InlineCmd("'orbit") + ")、shape sphere 上の道 (" + InlineCmd("'pathSS'") + ", " + InlineCmd("'path'") + ") を描くことができます。"
    )
    #
    #
    print(
        "\n\n" + SubSect("図のサイズ、配色など") + "\t図のサイズ、配色などは、" + Method("plot()") + " メソッドの引数に指定することで変更することが可能です。ほとんどは " + Module("matplotlib") + " モジュールのオプションと共通にしていますが、一つ一つ覚えておいて、描画の度にいちいち書き込むのは結構面倒です。" + Function("tb.setOption()") + " 関数を用意しました。この関数を実行すると、幾つか質問されます。変更したい部分について入力することで、以後、その設定で描がかれるようになります。変更しなくてもいい部分については、何も入力せず、[return]/[enter] キーを押し、次に質問に移ってください。また、" + Method("plot()") + " メソッドで指定したい描画用のキーワード引数とその値を、" + Function("tb.setOptions()") + " 関数の引数にすることで、その値のみを直接変更することもできます。冗長になるので、" + Method("plot()") + " メソッドで指定可能な描画用キーワード引数については省略します。"
    )
    #
    #
    #
    print(
        "\n\n" + SubSect("図の保存方法") + "\t視覚化手続きで表示された図を簡単に保存することができます。図の右の " + Method("Save to") + " の右の欄にファイル名を書き、その右の " + Method("Save") + " ボタンを押すだけで、左に表示されているグラフを png 形式のファイルで保存することができます。"
       + "他にも eps, jpg (jpeg), pdf, pgf, ps, raw, rgba, svg, svgz, tif (tiff) 形式に対応しています。拡張子に指定することで、それぞれの形式で保存できます。"
    )
    #
    #
